/*-----------------------------------------
   Auto.C -- Helmsman and Yardsmen
   Version date: 03.09.02
  -----------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "External.h" 

// Prototypes:
void   BestHelmAngle () ;
void   Helmsman () ;
void   ManBraces () ;
void   ManHelm () ;
void   Yardsmen () ;

extern BOOL  CALLBACK HelmDlgProc (HWND, UINT, WPARAM, LPARAM) ;
extern void  CanShadow () ;
extern void  Recorder (int, int, int) ;
extern void  SailPlan () ;
extern void  SetUpDisplay () ;
extern int   Sgn (float) ;
extern void  ShowMessage () ;
extern void  ShowActualHelmAngle () ;
extern void  ShowActualYardAngle () ;
extern void  ShowSelectedHelmAngle() ;
extern void  ShowSelectedYardAngle (int) ;

/* ------------------------------
     BestHelmAngle: calculate best helm angle for automatic helmsman
     Version date: 03.09.02
-------------------------------*/
  void BestHelmAngle ()
  { 
  int    iI ;
  float  fRF, fRF2, fRFmin ;

  fRFmin = 1000000000.0 ;
  for (iI = (5 - iMaxHelmAngle) ; iI <= (iMaxHelmAngle - 5) ; iI++)
    {
    fRF2 = fabs(fRudderArea * sin(iI * fDegToRad)) ; // effective area
    fRF = fabs(sin(iI * fDegToRad)) ; // useful fraction
    fRF = fRF2 * fSpeed * fSpeed * fRF * fRudderFactor / 1000.0 ; // useful force
    fRF = Sgn(fSpeed) * Sgn((float) iI) * fRF * cos(iI * fDegToRad) ; // useful force for leeway
    fRF = -fRF * fShipLength / 2.0 ;
    fRF = fRotationalThrust - fRotatForce[iNrSails] + fRF ;
    if (fabs(fRF) < fabs(fRFmin))
      {
      fRFmin = fRF ;
      iBestAngle = iI ;
      }
    }
  iNewHelmAngle = iBestAngle ;
  bEstablished = TRUE ;
  if (bModernHelm)
    SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
  else  
    SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
  ShowSelectedHelmAngle() ;
  ShowActualHelmAngle() ;
  }


/* ------------------------------
     Helmsman: activate or deactivate automatic helmsman
     Version date: 03.07.08
-------------------------------*/

void Helmsman () 
  {
  if (bHelmsman)
    { 
    bHelmsman = FALSE ;
    bEstablished = FALSE ;
    iTackStatus = 0 ;
    strcpy(sHelmInstruction, "(await orders)"); 
    SetUpDisplay () ;
    sprintf(sMessage, "Helmsman: await orders"); 
    ShowMessage () ;
    if (bRecording)
      Recorder (7, 0, 0) ;
    }
  else
    {
    iBearing = fHeading ;
    DialogBox (hInstance, "HelmBox", hwnd, HelmDlgProc) ;
    }
  SetFocus (hwndCheck[1]) ;
  }

/* ------------------------------
     ManBraces: calculate new yard angles for automatic yardsmen
     Version date: 03.08.27
-------------------------------*/
  void ManBraces ()
  { 
  HDC      hdc ;
  float    fRqdFromWind, fHdgFromWind ;
  float    fTemp ;
  int      iI ;
  int      iNewValue ;
  BOOL     bTemp ;
  int      yThis ;

  hdc = GetDC (hwnd) ;


  if (iTackStatus > 0)
    {
    switch (iTackStatus)
      {
      case 1 :
        if (fNow > (fMyTime+(10.0/iAcceleration[iSpeedFactor])))
          {
          // East down the helm
          iNewHelmAngle = 0 ;
          SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
          ShowSelectedHelmAngle () ;
          ShowActualHelmAngle () ;
          strcpy(sMessage, "Ease down the helm") ; 
          ShowMessage() ;
          iTackStatus = 2 ;
          fMyTime = fNow ;
          }
        break ;
      case 2 :
        if (fNow  > (fMyTime+(10.0/iAcceleration[iSpeedFactor])))
          {
          // Helm's a-lee
          strcpy (sMessage, "Helm's a-lee") ; 
          ShowMessage() ;
          // set 20  lee helm
          if (fRelWindAngle > 0)
            iNewHelmAngle = -20 ;
          else
            iNewHelmAngle = 20 ;
          if (bModernHelm)
            SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
          else  
            SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
          ShowSelectedHelmAngle () ;
          ShowActualHelmAngle () ;
          if (fRelWindAngle > 0)
            iNewYardAngle[3] = -iMaxYardAngle[3] ;
          else
            iNewYardAngle[3] = iMaxYardAngle[3] ;
          SetScrollPos (hwndSlider[3], SB_CTL, -iNewYardAngle[3], TRUE) ;
          strcpy(sMessage, "Haul spanker to windward") ; 
          ShowMessage() ;
          ShowSelectedYardAngle(3) ;
          // ease headsheets
          bTemp = TRUE ;
          for (iI = 0 ; iI < iNrSails ; iI++)
            {
            if (iSailType[iI] == 1)
              {
              iSailOldSSStatus[iI-(iSpanker+1)]=iSailStatus[iI] ;
              if (iSailStatus[iI] < 4)
                {
                fSailAngle[iI]= 45.0 ;
                iSailStatus[iI] = 6 ;
                if (bTemp==TRUE)
                  {
                  strcpy(sMessage, "Ease headsail sheets") ; 
                  ShowMessage() ;
                  bTemp = FALSE ;
                  bEasedJibs = TRUE ;
                  }
                }
              }
            }          
          SetScrollPos (hwndSlider[5], SB_CTL, 0, TRUE) ;
          ShowActualJibAngle () ;
          bSailPlanIsChanged = TRUE ;
          CanShadow () ;
          iTackStatus = 3 ;
          fMyTime = fNow ;
          }
        break ;
      case 3 :
        // helm amidships 
        if (fSpeed < 0.0 && iNewHelmAngle != 0)
          {
          iNewHelmAngle = 0 ;
          if (bModernHelm)
            SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
          else  
            SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
          ShowActualHelmAngle () ;
          ShowSelectedHelmAngle () ;
          strcpy(sMessage, "Helm amidships") ; 
          ShowMessage() ;
          fMyTime = fNow ;
          iTackStatus = 4 ;
          }
        break ;
      case 4 :
        // reverse helm 
        if (fNow  > (fMyTime+(10.0/iAcceleration[iSpeedFactor])))
          {
          if (fRelWindAngle > 0)
            iNewHelmAngle = 20 ;
          else
            iNewHelmAngle = -20 ;
          if (bModernHelm)
            SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
          else  
            SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
          ShowSelectedHelmAngle () ;
          ShowActualHelmAngle () ;
          strcpy(sMessage, "Reverse helm") ; 
          ShowMessage() ;
          iTackStatus = 5 ;
          }
        break ;
      case 5:
        // Reverse mainyards
        if (fabs(fAppWindAngle) > 170.0)
          {
          iNewYardAngle[1] = -Sgn(iYardAngle[1]) * iMaxYardAngle[1] ; 
          SetScrollPos (hwndSlider[1], SB_CTL, iNewYardAngle[1], TRUE) ;
          if (!bBrig)
            {  
            iNewYardAngle[2] = -Sgn(iYardAngle[2]) * iMaxYardAngle[2] ; 
            SetScrollPos (hwndSlider[2], SB_CTL, iNewYardAngle[2], TRUE) ;
            } 
          ShowActualYardAngle(1) ;
          ShowSelectedYardAngle(1) ;
          strcpy(sMessage, "Mainsail haul") ; 
          ShowMessage() ;
          iTackStatus = 6 ;
          }
        break ;
      case 6 :
        // Reset headsails
        if (fabs(fAppWindAngle) < 160.)
          {
          bTemp = TRUE ;
          bEasedJibs = FALSE ;
          for (iI = 0 ; iI < iNrSails ; iI++)
            {
            if (iSailType[iI] == 1)
              {
              if (iSailOldSSStatus[iI-(iSpanker+1)] < 4)
                {
                iSailStatus[iI] = iSailOldSSStatus[iI-(iSpanker+1)] ;
                fSailAngle[iI] = 0 ;
                if (bTemp==TRUE)
                  {
                  sprintf(sMessage, "Haul headsails in tight") ; 
                  ShowMessage() ;
                  bTemp = FALSE ;
                  }
                bSailPlanIsChanged = TRUE ;
                bPlanViewIsChanged = TRUE ;
                }
              }
            SetScrollPos (hwndSlider[5], SB_CTL, iOldJibAngle, TRUE) ;
            ShowActualJibAngle () ;
            iTackStatus = 7 ;
            }
          }
        break ;
      case 7 :
        // Reverse foreyards
        if (fabs(fAppWindAngle) < 110.)
          {
          iNewYardAngle[0] = -Sgn(iYardAngle[0]) * iMaxYardAngle[0] ;
          SetScrollPos (hwndSlider[0], SB_CTL, iNewYardAngle[0], TRUE) ;
          ShowActualYardAngle(0) ;
          ShowSelectedYardAngle(0) ;
          strcpy(sMessage, "Let go and haul") ; 
          ShowMessage() ;
          iTackStatus = 8 ;
          fMyTime = fNow ;
          } 
        break ;
      case 8 :
        // Right the helm
        if (fSpeed > 0.0)
          {
          if (fRelWindAngle > 0)
            iNewHelmAngle = 25 ;
          else
            iNewHelmAngle = -25 ;
          if (bModernHelm)
            SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
          else  
            SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
          ShowSelectedHelmAngle () ;
          ShowActualHelmAngle () ;
          strcpy(sMessage, "Right the helm") ; 
          ShowMessage() ;
          fMyTime = fNow ;
          iTackStatus = 9 ;
          }
        break ;
      case 9 :
        // Haul in spankerboom
        if (fNow  > (fMyTime+(20.0/iAcceleration[iSpeedFactor])))
          {
          iNewYardAngle[3] = -iOldSpankerAngle ;
          SetScrollPos (hwndSlider[3], SB_CTL, -iNewYardAngle[3], TRUE) ;
          ShowSelectedYardAngle(3) ;
          bSailPlanIsChanged = TRUE ;
          bPlanViewIsChanged = TRUE ;
          CanShadow () ;
          strcpy(sMessage, "Haul in spankerboom") ; 
          ShowMessage() ;
          iTackStatus = 10 ;
          }
        break ;
      case 10:
        // tack complete
        if (fabs(fAppWindAngle) < 55+iMaxYardAngle[1])
          {
          fLastDeviation = fHeading - fRequiredHeading ;
          if (fLastDeviation > 180.0)
            fLastDeviation = fLastDeviation - 360.0 ;
          if (fLastDeviation < -180.0)
            fLastDeviation = fLastDeviation + 360.0 ;
        // Reset headsails
        bTemp = TRUE ;
        for (iI = 0 ; iI < iNrSails ; iI++)
          {
          if (iSailType[iI] == 1)
            {
            if (iSailOldSSStatus[iI-(iSpanker+1)] < 4)
              {
              fSailAngle[iI] = -iOldJibAngle ;
              if (bTemp==TRUE)
                {
                sprintf(sMessage, "Reset headsails") ; 
                ShowMessage() ;
                bTemp = FALSE ;
                }
              bSailPlanIsChanged = TRUE ;
              bPlanViewIsChanged = TRUE ;
              }
            }
          SetScrollPos (hwndSlider[5], SB_CTL, iOldJibAngle, TRUE) ;
          ShowActualJibAngle () ;
          }
          strcpy(sMessage, "Tack complete") ; 
          ShowMessage() ;
          // reset main staysails 
          for (iI = 0 ; iI < iNrSails ; iI++)
            {
            if (iSailType[iI] == 3)
              {
              if (iSailOldSSStatus[iI-(iSpanker+1)] < 4)
                {
                iSailStatus[iI] = iSailOldSSStatus[iI-(iSpanker+1)] ;
                fSailAngle[iI] = Sgn(fAppWindAngle) * iOldStaySailAngle ;
                sprintf(sMessage, "Reset %s", sSailName[iI]) ; 
                ShowMessage() ;
                bSailPlanIsChanged = TRUE ;
                bPlanViewIsChanged = TRUE ;
                }
              }
            }
          SetScrollPos (hwndSlider[6], SB_CTL, iOldStaySailAngle, TRUE) ;
          ShowActualStaysailsAngle () ;
          iTackStatus = 0 ;
          }
        break ;
      }
    return ;  
    }
  
  iNewValue = -fAppWindAngle / 2. ;
  if (iNewValue > iMaxYardAngle[1])
     iNewValue = iMaxYardAngle[1] ;
  if (iNewValue < -iMaxYardAngle[1])
     iNewValue = -iMaxYardAngle[1] ;

  if ((sin (((90.0 - iYardAngle[1]) - fAppWindAngle) * fDegToRad)) >= sin (iCloseHauledAngle*fDegToRad) )
    {
    // not close-hauled; wind strikes mainyard sails at angle >= iCloseHauledAngle
    for (iI = 0 ; iI <= (2-bBrig) ; iI++)
      { 
      if (iNewYardAngle[iI] != iNewValue)
        {
        iNewYardAngle[iI] = iNewValue ;
        SetScrollPos (hwndSlider[iI], SB_CTL, iNewYardAngle[iI], TRUE) ;
        yThis = ySliders[iI] ;
        SelectObject(hdc,GetStockObject(NULL_PEN)) ;
        Rectangle (hdc, xSelected, yThis , xSelected + 50, yThis + cyTextHeight );
        SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
        switch (Sgn(iNewYardAngle[iI]))
          {
          case -1 :
            TextOut (hdc, xSelected, yThis, sMessage, 
              sprintf(sMessage, "%3d  port", abs(iNewYardAngle[iI]))) ; 
            break ;
          case 1 :
            TextOut (hdc, xSelected, yThis, sMessage,
              sprintf(sMessage, "%3d  stbd", abs(iNewYardAngle[iI]))) ; 
            break ;
          default :
            TextOut (hdc, xSelected, yThis, "amidships",9) ; 
          }
        ShowSelectedYardAngle (iI) ;
        }
      }
    }
  else
    {
    // ship is by-the-wind 
    if (bHelmsman)
      {
      fRqdFromWind = fRequiredHeading - fWindDirection ;
      if (fRqdFromWind < -180.0)
        fRqdFromWind = fRqdFromWind + 360.0 ;
      if (fRqdFromWind > 180.0)
        fRqdFromWind = fRqdFromWind - 360.0 ;
      fHdgFromWind = fHeading - fWindDirection ;
      if (fHdgFromWind < -180.0)
        fHdgFromWind = fHdgFromWind + 360.0 ;
      if (fHdgFromWind > 180.0)
        fHdgFromWind = fHdgFromWind - 360.0 ;
      // check if tack is required
      if (Sgn(fRqdFromWind) != Sgn(fHdgFromWind))
        {
        // tack is required
        iOldSpankerAngle = fSailAngle[iSpanker] ;
        iOldJibAngle = fSailAngle[iSpanker+1] ;
        for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
          iSailOldSSStatus[iI-(iSpanker+1)] = iSailStatus[iI] ;
          {
          iOldStaySailAngle = fSailAngle[15] ;
          for (iI = 0 ; iI < iNrSails ; iI++)
            { 
            if (iSailType[iI] == 3)
              {
              iSailOldSSStatus[iI-(iSpanker+1)]=iSailStatus[iI] ;
              if (iSailStatus[iI] < 4)
                {
                fSailAngle[iI]= 0.0 ;
                iSailStatus[iI] = 4 ;
                sprintf(sMessage, "Furl %s", sSailName[iI]) ; 
                ShowMessage() ;
                }
              }
            }
          } 
        SetScrollPos (hwndSlider[5], SB_CTL, 0, TRUE) ;
        ShowActualStaysailsAngle () ;
        bSailPlanIsChanged = TRUE ;
        CanShadow () ;
        strcpy(sMessage, "Stand by to tack") ; 
        ShowMessage() ;
        iTackStatus = 1 ;
        fMyTime = fNow ;
        return ;
        }
      else
        {
        if (fabs(fRqdFromWind) < fabs(fHdgFromWind))
          {
          // this is the best atainable heading
          if (fRqdFromWind > 0) 
            fRequiredHeading = ceil(fHeading) ;
          else
            fRequiredHeading = floor(fHeading) ;
          BestHelmAngle() ;
          SelectObject (hdc, GetStockObject(NULL_PEN)) ;
          Rectangle (hdc, xAuto, yMessage+29, xAuto+cxAuto, yMessage+29+cyTextHeight) ;
          SelectObject (hdc, GetStockObject(BLACK_PEN)) ;
          SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
          TextOut (hdc, xAuto+cxAuto/2, yMessage+29, sHelmInstruction, sprintf(sHelmInstruction, "steer %3.0f ", fRequiredHeading)); 
          SetTextAlign (hdc,TA_LEFT|TA_TOP) ;
          sprintf(sMessage, "By-the-wind heading %3.0f ", fRequiredHeading);
          ShowMessage() ;
          }
        }
      }
    }

  ReleaseDC (hwnd, hdc) ;
  }

/* ------------------------------
     ManHelm: calculate new helm position for automatic helmsman
     Version date: 03.09.02
-------------------------------*/

 void ManHelm () 
  {
  int    iDelta ;
  HDC    hdc ;
  float  fHeadingDeviation ;
  int    iI ;
  float  fRF, fRF2, fRFmin ;
  char   cTrend ;

  fHeadingDeviation = fHeading - fRequiredHeading ;
  if (fHeadingDeviation > 180.0)
    fHeadingDeviation = fHeadingDeviation - 360.0 ;
  if (fHeadingDeviation < -180.0)
    fHeadingDeviation = fHeadingDeviation + 360.0 ;

  if (bEstablished)
    {
    if (fHeadingDeviation < -0.3)
      iNewHelmAngle = iBestAngle - 1 ;
    else
      {
      if (fHeadingDeviation > 0.3)
        iNewHelmAngle = iBestAngle + 1 ;
      else
        iNewHelmAngle = iBestAngle ;
      }
    if (bModernHelm)
      SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
    else  
      SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
    ShowSelectedHelmAngle () ;
    ShowActualHelmAngle () ;
    if (fabs(fHeadingDeviation) > 3.0)
      {
      bEstablished = FALSE ;
      hdc = GetDC (hwnd) ;
      SelectObject (hdc, GetStockObject(NULL_PEN)) ;
      Rectangle (hdc, xAuto, yMessage+29, xAuto+cxAuto, yMessage+29+cyTextHeight) ;
      SelectObject (hdc, GetStockObject(BLACK_PEN)) ;
      SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
      TextOut (hdc, xAuto+cxAuto/2, yMessage+29, sHelmInstruction, sprintf(sHelmInstruction, "steer %3.0f ", fRequiredHeading)); 
      SetTextAlign (hdc,TA_LEFT|TA_TOP) ;
      strcpy(sMessage, "Helmsman: watch your helm") ;
      ShowMessage() ;
      ReleaseDC (hwnd, hdc) ;
      return ;
      }
    }
  else
    {
    if (fabs(fHeadingDeviation) < fabs(fLastDeviation))
      cTrend = '-' ;
    else
      cTrend = '+' ;
    fLastDeviation = fHeadingDeviation ;
    if (cTrend == '+')
      { 
      if (fabs(fHeadingDeviation) < 10.0)
        iDelta = 1 ;
      else
        iDelta = 5 ;
      if (abs(iNewHelmAngle) < (iMaxHelmAngle - 10))
        {
        iNewHelmAngle = iNewHelmAngle + iDelta * Sgn(fHeadingDeviation) ;
        if (bModernHelm)
          SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
        else  
          SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
        ShowSelectedHelmAngle() ;
        ShowActualHelmAngle() ;
        }
      }

    if (fabs(fHeadingDeviation) < 0.4)
      {
      // calculate appropriate rudder angle
      fRFmin = 1000000000.0 ;
      iBestAngle = -iMaxHelmAngle ;
      for (iI = (5 - iMaxHelmAngle) ; iI <= (iMaxHelmAngle - 5) ; iI++)
        {
        fRF2 = fabs(fRudderArea * sin(iI * fDegToRad)) ; // effective area
        fRF = fabs(sin(iI * fDegToRad)) ; // useful fraction
        fRF = fRF2 * fSpeed * fSpeed * fRF * fRudderFactor / 1000.0 ; // useful force
        fRF = Sgn(fSpeed) * Sgn((float) iI) * fRF * cos(iI * fDegToRad) ; // useful force for leeway
        fRF = -fRF * fShipLength / 2.0 ;
        fRF = fRotationalThrust + -fRotatForce[iNrSails] + fRF ;
        if (fabs(fRF) < fabs(fRFmin))
          {
          fRFmin = fRF ;
          iBestAngle = iI ;
          }
        }
      iNewHelmAngle = iBestAngle ;
      sprintf(sMessage, "steering %3.0f ", fRequiredHeading) ;
      if (strcmp(sMessage,sHelmInstruction))
        {
        sprintf(sMessage, "Helmsman steering %3.0f ", fRequiredHeading) ;
        ShowMessage() ;
        }
      hdc = GetDC (hwnd) ;
      SelectObject (hdc, GetStockObject(NULL_PEN)) ;
      Rectangle (hdc, xAuto, yMessage+29, xAuto+cxAuto, yMessage+29+cyTextHeight) ;
      SelectObject (hdc, GetStockObject(BLACK_PEN)) ;
      SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
      TextOut (hdc, xAuto+cxAuto/2, yMessage+29, sHelmInstruction, sprintf(sHelmInstruction, "steering %3.0f ", fRequiredHeading)); 
      SetTextAlign (hdc,TA_LEFT|TA_TOP) ;
      ReleaseDC (hwnd, hdc) ;
      if (bModernHelm)
        SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
      else  
        SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
      ShowSelectedHelmAngle() ;
      ShowActualHelmAngle() ;
      bEstablished = TRUE ;
      }
    }
  }

/* ------------------------------
     Yardsmen: activate or deactivate automatic yardsmen
     Version date: 03.07.16
-------------------------------*/

void Yardsmen () 
  {
  HDC      hdc ;

  hdc = GetDC (hwnd) ;
  SelectObject(hdc, GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xAuto, yMessage+65, xAuto+cxAuto, yMessage+65+cyTextHeight) ;
  SelectObject(hdc, GetStockObject(BLACK_PEN)) ;
  if (!bYardsmen)
    {
    bYardsmen = TRUE ;
    SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
    TextOut (hdc, xAuto+cxAuto/2, yMessage+65, " man braces ",12) ; 
    SetTextAlign (hdc,TA_LEFT|TA_TOP) ;
    strcpy(sMessage, "Yardsmen: man braces") ; 
    ShowMessage() ;
    if (bRecording)
      Recorder (6, 1, 0) ;
    SetTextColor(hdc, RGB(160,160,160)) ;  // gray
    TextOut (hdc, xSliders, ySliders[0], "Fore", 4) ; 
    TextOut (hdc, xSliders, ySliders[1], "Main", 4) ; 
    SetTextColor(hdc, RGB(0,0,0)) ;  // black
    }
  else
    {
    bYardsmen = FALSE ;
    SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
    TextOut (hdc, xAuto+cxAuto/2, yMessage+65, "(await orders)",14) ; 
    SetTextAlign (hdc,TA_LEFT|TA_TOP) ;
    strcpy(sMessage, "Yardsmen: await orders") ; 
    ShowMessage() ;
    if (bRecording)
      Recorder (6, 0, 0) ;
    TextOut (hdc, xSliders, ySliders[0], "Fore", 4) ; 
    TextOut (hdc, xSliders, ySliders[1], "Main", 4) ; 
    }
 
  ReleaseDC (hwnd, hdc) ;
  SetFocus (hwndCheck[1]) ;
  }


