/*-----------------------------------------
   Config.C -- Configuration module
   Version date: 99.10.03
  -----------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <commdlg.h>
#include <stdlib.h>
#include "external.h"
#include "menu.h"

// Prototypes:
float    Extract (char *, int) ; 
void     FileInitialize () ;
BOOL     FileOpenDlg (PSTR, PSTR) ;
BOOL     FileSaveDlg (PSTR, PSTR) ;
void     LoadConfiguration () ;
void     LoadDefault () ;
void     ReadLine (HANDLE, char *) ;
void     SaveConfiguration () ;

/* --------------------------------------
   Extract: extracts float from string
   Version date = 99.08.07
----------------------------------------*/

float Extract (char * sBuf, int iStart) 
  {
  int  iI ;
  BOOL bReal ;
  char sSubString[15];

  if (iStart == -1) // search for value after colon
    {
    iI=0 ;
    while (sBuf[iI] != ':')
      iI++ ;
    iStart = iI+1 ;
    }
  else
    iStart-- ;  // position 1 = array element 0

  bReal = FALSE;

  // skip leading blanks
  while (sBuf[iStart] == ' ')
    {
    iStart++ ;
    }

  // extract numeric value
  iI=0 ;
  for (iI=0 ; iI<15 ; iI++)
    sSubString[iI]=' ' ;

  iI=0 ;
  while (sBuf[iStart+iI]!=' ' && sBuf[iStart+iI]!='\0')
    {
    sSubString[iI]=sBuf[iStart+iI] ;
    if (sSubString[iI]=='.')
      bReal = TRUE ;
    iI++ ;
    }
  if (!bReal)
    {
    sSubString[iI]='.';
    iI++ ;
    }
  sSubString[iI]='\0' ;
  return atof(sSubString) ;
  }

/* --------------------------------------
   FileInitialize: modified from Petzold
   Version date = 99.10.03
----------------------------------------*/

void FileInitialize ()
     {
     static char szFilter[] = "Surprise scenerio file (*.SSF)\0*.ssf\0"  \
                              "All Files (*.*)\0*.*\0\0" ;
     oFileName.lStructSize       = sizeof (OPENFILENAME) ;
     oFileName.hwndOwner         = hwnd ;
     oFileName.hInstance         = NULL ;
     oFileName.lpstrFilter       = szFilter ;
     oFileName.lpstrCustomFilter = NULL ;
     oFileName.nMaxCustFilter    = 0 ;
     oFileName.nFilterIndex      = 0 ;
     oFileName.lpstrFile         = NULL ;          // Set in Open and Close functions
     oFileName.nMaxFile          = _MAX_PATH ;
     oFileName.lpstrFileTitle    = NULL ;          // Set in Open and Close functions
     oFileName.nMaxFileTitle     = _MAX_FNAME + _MAX_EXT ;
     oFileName.lpstrInitialDir   = NULL ;
     oFileName.lpstrTitle        = NULL ;
     oFileName.Flags             = 0 ;             // Set in Open and Close functions
     oFileName.nFileOffset       = 0 ;
     oFileName.nFileExtension    = 0 ;
     oFileName.lpstrDefExt       = "ssf" ;
     oFileName.lCustData         = 0L ;
     oFileName.lpfnHook          = NULL ;
     oFileName.lpTemplateName    = NULL ;
     }

/* --------------------------------------
   FileOpenDlg: modified from Petzold
   Version date = 99.10.03
----------------------------------------*/

BOOL FileOpenDlg (PSTR pstrFileName, PSTR pstrTitleName)
     {
     oFileName.lpstrFile         = pstrFileName ;
     oFileName.lpstrFileTitle    = pstrTitleName ;
     oFileName.Flags             = OFN_HIDEREADONLY | OFN_CREATEPROMPT ;

     return GetOpenFileName (&oFileName) ;
     }

/* --------------------------------------
   FileSaveDlg: modified from Petzold
   Version date = 99.10.03
----------------------------------------*/

BOOL FileSaveDlg (PSTR pstrFileName, PSTR pstrTitleName)
     {
     oFileName.lpstrFile         = pstrFileName ;
     oFileName.lpstrFileTitle    = pstrTitleName ;
     oFileName.Flags             = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT ;

     return GetSaveFileName (&oFileName) ;
     }

/* --------------------------------------
   LoadConfiguration
   Version date = 99.09.25
----------------------------------------*/

void LoadConfiguration () 
  {
  char    sBuf [80] ;
  HANDLE  hFile ;
  HMENU   hMenu ;
  int     iI ;

  iMapCounter = 0 ;
  fLongtitude=0.0 ;
  fLatitude = 0.0 ;
  bMaxTrack = FALSE ;
  bTracking = TRUE ;
  SendMessage (hwndCheck[2], BM_SETCHECK, 1, 0);
  hMenu=GetMenu(hwnd) ;
  CheckMenuItem (hMenu, IDM_TRACKSHIP, MF_CHECKED) ;
  CheckMenuItem (hMenu, IDM_DONTTRACKSHIP, MF_UNCHECKED) ;
  bMapMustBePainted = TRUE ;
  bYardsmen = FALSE;
  bHelmsman = FALSE ;
  strcpy(sHelmInstruction,"(await orders)") ;
  
  hFile = CreateFile (sFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0,0) ;
  ReadLine (hFile, sBuf) ;
  ReadLine (hFile, sBuf) ;
  fShipLength = Extract(sBuf,-1) ;
  ReadLine (hFile, sBuf) ;
  fQDeckLength = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fFocleLength = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fGunDeckHeight = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fQDeckHeight = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fShipWidth = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fShipDraught = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fShipTons = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMastDistance[0] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMastDistance[1] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMastDistance[2] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMastDistance[3] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fLowestSailFoot[0] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fLowestSailFoot[1] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fLowestSailFoot[2] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fLowestSailFoot[3] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSetFraction[1] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSetFraction[2] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSetFraction[3] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxYardAngle[0] = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxYardAngle[1] = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxYardAngle[2] = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxYardAngle[3] = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fRudderArea = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxJibAngle = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxHelmAngle = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  ReadLine (hFile, sBuf) ;
  ReadLine (hFile, sBuf) ;
  ReadLine (hFile, sBuf) ;
  ReadLine (hFile, sBuf) ;
  ReadLine (hFile, sBuf) ;
  for (iI=0 ; iI<13 ; iI++)
    {
    ReadLine (hFile, sBuf) ;
    fHeadWidth[iI] = Extract(sBuf,8) ; 
    fFootWidth[iI] = Extract(sBuf,18) ; 
    fSailDepth[iI] = Extract(sBuf,28) ; 
    iMaxReefs[iI] = (int) Extract(sBuf,40) ; 
    iSailStatus[iI] = (int) Extract(sBuf,50) ; 
    fSailAngle[iI] = Extract(sBuf,58) ; 
    fTheorMaxSpeed[iI] = Extract(sBuf,70) ; 
    }
  ReadLine (hFile, sBuf) ;
  iJibSet = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  for (iI=0 ; iI<3 ; iI++)
    {
    ReadLine (hFile, sBuf) ;
    fJibFootWidth[iI] = Extract(sBuf,8) ; 
    fJibDepth[iI] = Extract(sBuf,18) ; 
    fJibTheorMaxSpeed[iI] = Extract(sBuf,30) ; 
    }
  ReadLine (hFile, sBuf) ;
  iHelmAngle = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeading =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSpeed =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fLeewaySpeed =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeelSpeed = 0.0 ;
  fRotationalSpeed = 0.0 ;
  fHeelingAngle =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMadeSail =  (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fWindDirection =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fWindSpeed =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iDisplayParam =  (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  if (sBuf[22]=='T')
    bShowSlipStream = TRUE ;
  else
    bShowSlipStream = FALSE ;
  ReadLine (hFile, sBuf) ;
  iYardLevelShown =  (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMapWidth =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fBowFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSternFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSideFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fWaterDragFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fRudderFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMassFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fRotatInertFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fRotatDragFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeelAngleFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeelInertFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeelDragFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  for (iI=0 ; iI<13 ; iI++)
    {
    ReadLine (hFile, sBuf) ;
    fIsleLat[iI] = Extract(sBuf,1) ; 
    fIsleLong[iI] = Extract(sBuf,12) ; 
    fIsleRadius[iI] = Extract(sBuf,24) ; 
    }
  CloseHandle (hFile) ; 
  }
  
/* --------------------------------------
   LoadDefault
   Version date = 99.10.03
----------------------------------------*/

void LoadDefault () 
  {

  HMENU    hMenu ;

  iMapCounter = 0 ;
  fLongtitude=0.0 ;
  fLatitude = 0.0 ;
  bMaxTrack = FALSE ;
  bTracking = TRUE ;
  SendMessage (hwndCheck[2], BM_SETCHECK, 1, 0);
  hMenu=GetMenu(hwnd) ;
  CheckMenuItem (hMenu, IDM_TRACKSHIP, MF_CHECKED) ;
  CheckMenuItem (hMenu, IDM_DONTTRACKSHIP, MF_UNCHECKED) ;
  bMapMustBePainted = TRUE ;
  bYardsmen = FALSE ;
  bHelmsman = FALSE ;
  strcpy(sHelmInstruction,"(await orders)") ;

  fShipLength = 146.0 ;
  fQDeckLength = 70.0 ; 
  fFocleLength = 34.0 ; 
  fGunDeckHeight = 15.0 ; 
  fQDeckHeight = 22.0 ; 
  fShipWidth = 38.0 ; 
  fShipDraught = 19.0 ; 
  fShipTons = 850.0 ; 
  fMastDistance[0] = 52.0 ; 
  fMastDistance[1] = -15.0 ; 
  fMastDistance[2] = -52.0 ; 
  fMastDistance[3] = 134.0 ; 
  fLowestSailFoot[0] = 26.0 ; 
  fLowestSailFoot[1] = 25.0 ; 
  fLowestSailFoot[2] = 26.0 ; 
  fLowestSailFoot[3] = 35.0 ;
  fSetFraction[0] = 1.0 ; // set 
  fSetFraction[1] = 0.85 ; // one reef
  fSetFraction[2] = 0.70 ; // two reefs
  fSetFraction[3] = 0.55 ; // three reefs
  fSetFraction[4] = 0.0 ; // furled
  fSetFraction[5] = 0.0 ; // blown out 
  iMaxYardAngle[0] = 60 ; 
  iMaxYardAngle[1] = 60 ; 
  iMaxYardAngle[2] = 60 ; 
  iMaxYardAngle[3] = 45 ; 
  fRudderArea = 64.0 ; 
  iMaxJibAngle = 30 ; 
  iMaxHelmAngle = 55 ; 
  fHeadWidth[0] = 26.0 ; 
  fHeadWidth[1] = 31.0 ; 
  fHeadWidth[2] = 42.0 ; 
  fHeadWidth[3] = 60.0 ; 
  fHeadWidth[4] = 26.0 ; 
  fHeadWidth[5] = 34.0 ; 
  fHeadWidth[6] = 47.0 ; 
  fHeadWidth[7] = 66.0 ; 
  fHeadWidth[8] = 19.0 ; 
  fHeadWidth[9] = 26.0 ; 
  fHeadWidth[10] = 36.0 ; 
  fHeadWidth[11] = 50.0 ; 
  fHeadWidth[12] = 0.0 ; 
  fFootWidth[0] = 31.0 ; 
  fFootWidth[1] = 42.0 ; 
  fFootWidth[2] = 62.0 ; 
  fFootWidth[3] = 60.0 ; 
  fFootWidth[4] = 34.0 ; 
  fFootWidth[5] = 47.0 ; 
  fFootWidth[6] = 66.0 ; 
  fFootWidth[7] = 66.0 ; 
  fFootWidth[8] = 26.0 ; 
  fFootWidth[9] = 36.0 ; 
  fFootWidth[10] = 42.0 ; 
  fFootWidth[11] = 50.0 ; 
  fSailDepth[0] = 9.0 ; 
  fSailDepth[1] = 21.0 ; 
  fSailDepth[2] = 35.0 ; 
  fSailDepth[3] = 32.0 ; 
  fSailDepth[4] = 10.0 ; 
  fSailDepth[5] = 26.0 ; 
  fSailDepth[6] = 40.0 ; 
  fSailDepth[7] = 35.0 ; 
  fSailDepth[8] = 8.0 ; 
  fSailDepth[9] = 20.0 ; 
  fSailDepth[10] = 43.0 ; 
  fSailDepth[11] = 32.0 ; 
  iMaxReefs[0] = 0 ; 
  iMaxReefs[1] = 0 ; 
  iMaxReefs[2] = 3 ; 
  iMaxReefs[3] = 2 ; 
  iMaxReefs[4] = 0 ; 
  iMaxReefs[5] = 0 ; 
  iMaxReefs[6] = 3 ; 
  iMaxReefs[7] = 2 ; 
  iMaxReefs[8] = 0 ; 
  iMaxReefs[9] = 0 ; 
  iMaxReefs[10] = 3 ; 
  iMaxReefs[11] = 2 ; 
  iMaxReefs[12] = 0 ; 
  iSailStatus[0] = 4 ; 
  iSailStatus[1] = 0 ; 
  iSailStatus[2] = 0 ; 
  iSailStatus[3] = 0 ; 
  iSailStatus[4] = 4 ; 
  iSailStatus[5] = 0 ; 
  iSailStatus[6] = 0 ; 
  iSailStatus[7] = 0 ; 
  iSailStatus[8] = 4 ; 
  iSailStatus[9] = 0 ; 
  iSailStatus[10] = 0 ; 
  iSailStatus[11] = 0 ; 
  iSailStatus[12] = 0 ; 
  fSailAngle[0] = -56 ; 
  fSailAngle[1] = -56 ; 
  fSailAngle[2] = -56 ; 
  fSailAngle[3] = -56 ; 
  fSailAngle[4] = -56 ; 
  fSailAngle[5] = -56 ; 
  fSailAngle[6] = -56 ; 
  fSailAngle[7] = -56 ; 
  fSailAngle[8] = -56 ; 
  fSailAngle[9] = -56 ; 
  fSailAngle[10] = -56 ; 
  fSailAngle[11] = 0 ; 
  fSailAngle[12] = 13 ; 
  fTheorMaxSpeed[0] = 20.0 ; 
  fTheorMaxSpeed[1] = 25.0 ; 
  fTheorMaxSpeed[2] = 35.0 ; 
  fTheorMaxSpeed[3] = 50.0 ; 
  fTheorMaxSpeed[4] = 20.0 ; 
  fTheorMaxSpeed[5] = 25.0 ; 
  fTheorMaxSpeed[6] = 35.0 ; 
  fTheorMaxSpeed[7] = 50.0 ; 
  fTheorMaxSpeed[8] = 20.0 ; 
  fTheorMaxSpeed[9] = 25.0 ; 
  fTheorMaxSpeed[10] = 35.0 ; 
  fTheorMaxSpeed[11] = 50.0 ; 
  fTheorMaxSpeed[12] = 50.0 ; 
  iJibSet = 0 ; 
  fJibFootWidth[0] = 45.0 ; 
  fJibFootWidth[1] = 40.0 ; 
  fJibFootWidth[2] = 20.0 ; 
  fJibDepth[0] = 60.0 ; 
  fJibDepth[1] = 50.0 ; 
  fJibDepth[2] = 40.0 ; 
  fJibTheorMaxSpeed[0] = 30.0 ; 
  fJibTheorMaxSpeed[1] = 40.0 ; 
  fJibTheorMaxSpeed[2] = 50.0 ; 
  iHelmAngle = 10 ; 
  fHeading =  135.0 ; 
  fSpeed =  5.8 ; 
  fLeewaySpeed =  -1.3 ; 
  fHeelSpeed = 0.0 ;
  fRotationalSpeed = 0.0 ;
  fHeelingAngle = -10.3 ; 
  iMadeSail =  7 ; 
  fWindDirection =  225.0 ; 
  fWindSpeed =  15.0 ; 
  iDisplayParam = 7 ; 
  bShowSlipStream = TRUE ;
  iYardLevelShown = 3 ; 
  fMapWidth =  0.2 ; 
  fBowFactor =  0.3 ; 
  fSternFactor =  5.0 ; 
  fSideFactor =  10.0 ; 
  fWaterDragFactor =  66.0 ; 
  fRudderFactor =  770.0 ; 
  fMassFactor =  1.0 ; 
  fRotatInertFactor =  1.0 ; 
  fRotatDragFactor =  200.0 ; 
  fHeelAngleFactor =  0.025 ; 
  fHeelInertFactor =  0.0002 ; 
  fHeelDragFactor =  0.5 ; 
  fIsleLat[0] = -0.087 ; 
  fIsleLat[1] = -0.1 ; 
  fIsleLat[2] = -0.32 ; 
  fIsleLat[3] = -0.43 ; 
  fIsleLat[4] = -0.26 ; 
  fIsleLat[5] = -5.0 ; 
  fIsleLat[6] = 0.0 ; 
  fIsleLat[7] = 0.0 ; 
  fIsleLat[8] = 0.0 ; 
  fIsleLat[9] = 0.0 ; 
  fIsleLat[10] = 0.0 ; 
  fIsleLat[11] = 0.0 ; 
  fIsleLat[12] = 0.0 ; 
  fIsleLong[0] = 0.035 ; 
  fIsleLong[1] = 0.2 ; 
  fIsleLong[2] = 0.35 ; 
  fIsleLong[3] = 0.18 ; 
  fIsleLong[4] = 0.0 ; 
  fIsleLong[5] = 3.0 ; 
  fIsleLong[6] = 0.0 ; 
  fIsleLong[7] = 0.0 ; 
  fIsleLong[8] = 0.0 ; 
  fIsleLong[9] = 0.0 ; 
  fIsleLong[10] = 0.0 ; 
  fIsleLong[11] = 0.0 ; 
  fIsleLong[12] = 0.0 ; 
  fIsleRadius[0] = 0.016 ; 
  fIsleRadius[1] = 0.05 ; 
  fIsleRadius[2] = 0.015 ; 
  fIsleRadius[3] = 0.012 ; 
  fIsleRadius[4] = 0.008 ; 
  fIsleRadius[5] = 1.5 ; 
  fIsleRadius[6] = 0.0 ; 
  fIsleRadius[7] = 0.0 ; 
  fIsleRadius[8] = 0.0 ; 
  fIsleRadius[9] = 0.0 ; 
  fIsleRadius[10] = 0.0 ; 
  fIsleRadius[11] = 0.0 ; 
  fIsleRadius[12] = 0.0 ; 
  }

/* --------------------------------------
   ReadLine: Reads a line from file
   Version date = 99.08.07
----------------------------------------*/

void ReadLine (HANDLE hFile, char * sBuf)
  {
  char   sBlock[80] ;
  int    iI, iLength ;
  DWORD  dRead ;
 
  ReadFile (hFile, sBlock, 80, &dRead,  NULL) ;
  iI = 0 ;
  while (sBlock[iI]!='\n')
    iI++ ;
  iLength = iI ;
  SetFilePointer (hFile,-80+iLength+1,NULL,FILE_CURRENT) ;	
  for (iI=0 ; iI<80 ; iI++)
    sBuf[iI]=' ' ;
  for (iI=0 ; iI<iLength ; iI++)
    sBuf[iI]=sBlock[iI] ;
  }


/* --------------------------------------
   SaveConfiguration
   Version date = 99.09.12
----------------------------------------*/

void SaveConfiguration () 
  {
  char    sBuffer[3300] ;
  char    sBuf [80] ;
  int     iI ;
  DWORD   dToWrite, dWritten ;
  HANDLE  hFile ;

  hFile = CreateFile (sFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0,0) ;
  dToWrite = 3300 ;
  strcpy (sBuffer, "Surprise 1999????         \n") ;
  sprintf (sBuf, "Ship length         : %04.0f\n", fShipLength) ;
  strcat (sBuffer,sBuf) ; 
  sprintf (sBuf, "Quaterdeck length   : %04.0f\n", fQDeckLength) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Forecastle length   : %04.0f\n", fFocleLength) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Gundeck height      : %04.0f\n", fGunDeckHeight) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Quaterdeck height   : %04.0f\n", fQDeckHeight) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Ship width          : %04.0f\n", fShipWidth) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Ship draught        : %04.0f\n", fShipDraught) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Ship tonnage        : %04.0f\n", fShipTons) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Fore mast distance  : %04.0f\n", fMastDistance[0]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Main mast distance  : %04.0f\n", fMastDistance[1]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Mizzen mast distance: %04.0f\n", fMastDistance[2]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Jib clew distance   : %04.0f\n", fMastDistance[3]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Forecourse foot hgt : %04.0f\n", fLowestSailFoot[0]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Maincourse foot hgt : %04.0f\n", fLowestSailFoot[1]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Mizzencourse foot ht: %04.0f\n", fLowestSailFoot[2]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Jib foot height     : %04.0f\n", fLowestSailFoot[3]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Set fract 1 reef    : %04.2f\n", fSetFraction[1]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Set fract 2 reef    : %04.2f\n", fSetFraction[2]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Set fract 3 reef    : %04.2f\n", fSetFraction[3]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max foreyard angle  : %04d\n", iMaxYardAngle[0]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max mainyard angle  : %04d\n", iMaxYardAngle[1]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max crossjack angle : %04d\n", iMaxYardAngle[2]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max mizzenboom angle: %04d\n", iMaxYardAngle[3]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Rudder area         : %04.0f\n", fRudderArea) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max jib angle       : %04d\n", iMaxJibAngle) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max helm angle      : %04d\n", iMaxHelmAngle) ;
  strcat (sBuffer, sBuf) ; 
  strcpy (sBuf, "Reserved for expansion    \n") ;
  for (iI=0 ; iI<5 ; iI++)
    strcat (sBuffer, sBuf) ; 
  strcpy(sBuf,"Sail HeadWidth FootWidth     Depth  MaxReefs    Status YardAngle Max Speed\n") ;
  strcat (sBuffer, sBuf) ; 
  for (iI=0 ; iI<=12 ; iI++)
    {
    sprintf (sBuf, "  %02d     %05.2f     %05.2f     %05.2f     %05d     %05d     %05.0f     %05.0f\n", 
      iI, fHeadWidth[iI],fFootWidth[iI],fSailDepth[iI],iMaxReefs[iI],iSailStatus[iI],
      fSailAngle[iI],fTheorMaxSpeed[iI]) ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "Jib set             : %04d\n", iJibSet) ;
  strcat (sBuffer, sBuf) ; 
  strcpy(sBuf,"Jib  Footwidth     Depth Max Speed\n") ;
  strcat (sBuffer, sBuf) ; 
  for (iI=0 ; iI<=2 ; iI++)
    {
    sprintf (sBuf, "  %02d     %05.2f     %05.2f     %05.0f\n", 
      iI, fJibFootWidth[iI],fJibDepth[iI],fJibTheorMaxSpeed[iI]) ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "Helm angle          : %04d\n", iHelmAngle) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heading             : %07.2f\n", fHeading) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Speed               : %07.2f\n", fSpeed) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Leeway speed        : %07.2f\n", fLeewaySpeed) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heeling angle       : %07.2f\n", fHeelingAngle) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Sail pattern set    : %04d\n", iMadeSail) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "WindDirection       : %04.0f\n", fWindDirection) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "WindSpeed           : %04.0f\n", fWindSpeed) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Parameter to display: %04d\n", iDisplayParam) ;
  strcat (sBuffer, sBuf) ; 
  if (bShowSlipStream)
    strcpy (sBuf, "Show slipStream     : TRUE \n") ;
  else
    strcpy (sBuf, "Show slipStream     : FALSE\n") ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Yards to display    : %04d\n", iYardLevelShown) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Map width           : %06.3f\n", fMapWidth) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Bow factor          : %012.5f\n", fBowFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Stern factor        : %012.5f\n", fSternFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Side factor         : %012.5f\n", fSideFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Water drag factor   : %012.5f\n", fWaterDragFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Rudder factor       : %012.5f\n", fRudderFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Mass factor         : %012.5f\n", fMassFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Rot. inertia factor : %012.5f\n", fRotatInertFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Rot. drag factor    : %012.5f\n", fRotatDragFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heel angle factor   : %012.5f\n", fHeelAngleFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heel inertia factor : %012.5f\n", fHeelInertFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heel drag factor    : %012.5f\n", fHeelDragFactor) ;
  strcat (sBuffer, sBuf) ; 
  strcpy(sBuf, "Isle Lat   Isle Lon    Isle Radius\n") ;
  strcat (sBuffer, sBuf) ; 
  for (iI=0 ; iI<=12 ; iI++)
    {
    sprintf (sBuf, "%08.4f   %08.4f    %08.4f\n", 
      fIsleLat[iI],fIsleLong[iI],fIsleRadius[iI]) ;
    strcat (sBuffer, sBuf) ; 
    }
  WriteFile (hFile, sBuffer, dToWrite, &dWritten,  NULL) ;
  CloseHandle (hFile) ; 
  }

