/*-----------------------------------------
   Config.C -- Configuration module
   Version date: 03.09.24
  -----------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <commdlg.h>
#include <stdlib.h>
#include "external.h"
#include "menu.h"

// Prototypes:
void     DataDumper () ;
void     EndRecording () ;
void     EndPlayBack () ;
float    Extract (char *, int) ; 
float    Extract2 (char *, char *, int) ; 
void     FileInitialize_SRF () ;
void     FileInitialize_SSF () ;
BOOL     FileOpenDlg (PSTR, PSTR) ;
BOOL     FileSaveDlg (PSTR, PSTR) ;
int      LoadConfiguration (int) ;
void     LoadDefault () ;
void     ReadLine (HANDLE, char *) ;
void     Recorder (int, int, int) ;
void     Replayer () ;
void     SaveConfiguration (int) ;

/* --------------------------------------
   EndPlayBack: ends a playback session
   Version date = 03.09.13
----------------------------------------*/

void EndPlayBack () 
  {
  bPlayBack = FALSE ;
  EnableMenuItem(hMenu, IDM_RECORDER, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_STOPRECORDER, MF_GRAYED) ;
  EnableMenuItem(hMenu, IDM_PLAYBACK, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_STOPPLAYBACK, MF_GRAYED) ;
  EnableMenuItem(hMenu, IDM_SHIPDIMENSIONS, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_SHIPPROPERTIES, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_SAILDIMENSIONS, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_FORENAFT, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_ISLANDS, MF_ENABLED) ;
  CloseHandle (hFile) ; 
  MessageBox (hwnd, "Replay terminated", sMyProgName, MB_ICONINFORMATION | MB_OK) ;
  strcpy(sMyProgName, "HMS Surprise") ;
  //if (bBrig) 
  //  strcpy(sMyProgName, "Brig Simulator") ;
  //else
  //  strcpy(sMyProgName, "Frigate Simulator") ;
  SetWindowText (hwnd, sMyProgName) ;

  }

/* --------------------------------------
   EndRecording: ends a recording session
   Version date = 03.09.13
----------------------------------------*/

void EndRecording () 
  {
  bRecording = FALSE ;
  EnableMenuItem(hMenu, IDM_RECORDER, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_STOPRECORDER, MF_GRAYED) ;
  EnableMenuItem(hMenu, IDM_PLAYBACK, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_STOPPLAYBACK, MF_GRAYED) ;
  EnableMenuItem(hMenu, IDM_SHIPDIMENSIONS, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_SHIPPROPERTIES, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_SAILDIMENSIONS, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_FORENAFT, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_ISLANDS, MF_ENABLED) ;
  Recorder (99, 0, 0) ;
  MessageBox (hwnd, "Recording terminated", sMyProgName, MB_ICONINFORMATION | MB_OK) ;
  strcpy(sMyProgName, "HMS Surprise") ;
  //if (bBrig) 
  //  strcpy(sMyProgName, "Brig Simulator") ;
  //else
  //  strcpy(sMyProgName, "Frigate Simulator") ;
  SetWindowText (hwnd, sMyProgName) ;
  }

/* --------------------------------------
   Extract: extracts float from string
   Version date = 99.08.07
----------------------------------------*/

float Extract (char * sBuf, int iStart) 
  {
  int  iI ;
  BOOL bReal ;
  char sSubString[15];
    
  if (iStart == -1) // search for value after colon
    {
    iI=0 ;
    while (sBuf[iI] != ':')
      iI++ ;
    iStart = iI+1 ;
    }
  else
    iStart-- ;  // position 1 = array element 0

  bReal = FALSE;

  // skip leading blanks
  while (sBuf[iStart] == ' ')
    {
    iStart++ ;
    }

  // extract numeric value
  iI=0 ;
  for (iI=0 ; iI<15 ; iI++)
    sSubString[iI]=' ' ;

  iI=0 ;
  while (sBuf[iStart+iI]!=' ' && sBuf[iStart+iI]!='\0')
    {
    sSubString[iI]=sBuf[iStart+iI] ;
    if (sSubString[iI]=='.')
      bReal = TRUE ;
    iI++ ;
    }
  if (!bReal)
    {
    sSubString[iI]='.';
    iI++ ;
    }
  sSubString[iI]='\0' ;
  return atof(sSubString) ;
  }

/* --------------------------------------
   Extract2: extracts float from string
   Version date = 03.06.09
----------------------------------------*/

float Extract2 (char * sMessage, char * sBuf, int iStart) 
  {
  int  iI ;
  BOOL bReal ;
  char sSubString[15];
    
  if (iStart == -1) 
    {
    // check if this is the correct parameter
    if (strcmp(sMessage , sBuf) != 0)
      {
      strcat (sMessage, "- corrputed input") ; 
      MessageBox (hwnd, sMessage, sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
      return -12345.678 ;
      }
    // search for value after colon
    iI=0 ;
    while (sBuf[iI] != ':')
      iI++ ;
    iStart = iI+1 ;
    }
  else
    iStart-- ;  // position 1 = array element 0

  bReal = FALSE;

  // skip leading blanks
  while (sBuf[iStart] == ' ')
    {
    iStart++ ;
    }

  // extract numeric value
  iI=0 ;
  for (iI=0 ; iI<15 ; iI++)
    sSubString[iI]=' ' ;

  iI=0 ;
  while (sBuf[iStart+iI]!=' ' && sBuf[iStart+iI]!='\0')
    {
    sSubString[iI]=sBuf[iStart+iI] ;
    if (sSubString[iI]=='.')
      bReal = TRUE ;
    iI++ ;
    }
  if (!bReal)
    {
    sSubString[iI]='.';
    iI++ ;
    }
  sSubString[iI]='\0' ;
  return atof(sSubString) ;
  }

/* --------------------------------------
   FileInitialize_SRF: modified from Petzold
   Version date = 03.03.26
----------------------------------------*/

void FileInitialize_SRF ()
     {
     static char szFilter[] = "Simulator recorder file (*.SRF)\0*.srf\0"  \
                              "All Files (*.*)\0*.*\0\0" ;
     oFileName.lStructSize       = sizeof (OPENFILENAME) ;
     oFileName.hwndOwner         = hwnd ;
     oFileName.hInstance         = NULL ;
     oFileName.lpstrFilter       = szFilter ;
     oFileName.lpstrCustomFilter = NULL ;
     oFileName.nMaxCustFilter    = 0 ;
     oFileName.nFilterIndex      = 0 ;
     oFileName.lpstrFile         = NULL ;          // Set in Open and Close functions
     oFileName.nMaxFile          = _MAX_PATH ;
     oFileName.lpstrFileTitle    = NULL ;          // Set in Open and Close functions
     oFileName.nMaxFileTitle     = _MAX_FNAME + _MAX_EXT ;
     oFileName.lpstrInitialDir   = NULL ;
     oFileName.lpstrTitle        = NULL ;
     oFileName.Flags             = 0 ;             // Set in Open and Close functions
     oFileName.nFileOffset       = 0 ;
     oFileName.nFileExtension    = 0 ;
     oFileName.lpstrDefExt       = "srf" ;
     oFileName.lCustData         = 0L ;
     oFileName.lpfnHook          = NULL ;
     oFileName.lpTemplateName    = NULL ;
     }

/* --------------------------------------
   FileInitialize_SSF: modified from Petzold
   Version date = 03.03.26
----------------------------------------*/

void FileInitialize_SSF ()
     {
     static char szFilter[] = "Simulator scenerio file (*.SSF)\0*.ssf\0"  \
                              "All Files (*.*)\0*.*\0\0" ;
     oFileName.lStructSize       = sizeof (OPENFILENAME) ;
     oFileName.hwndOwner         = hwnd ;
     oFileName.hInstance         = NULL ;
     oFileName.lpstrFilter       = szFilter ;
     oFileName.lpstrCustomFilter = NULL ;
     oFileName.nMaxCustFilter    = 0 ;
     oFileName.nFilterIndex      = 0 ;
     oFileName.lpstrFile         = NULL ;          // Set in Open and Close functions
     oFileName.nMaxFile          = _MAX_PATH ;
     oFileName.lpstrFileTitle    = NULL ;          // Set in Open and Close functions
     oFileName.nMaxFileTitle     = _MAX_FNAME + _MAX_EXT ;
     oFileName.lpstrInitialDir   = NULL ;
     oFileName.lpstrTitle        = NULL ;
     oFileName.Flags             = 0 ;             // Set in Open and Close functions
     oFileName.nFileOffset       = 0 ;
     oFileName.nFileExtension    = 0 ;
     oFileName.lpstrDefExt       = "ssf" ;
     oFileName.lCustData         = 0L ;
     oFileName.lpfnHook          = NULL ;
     oFileName.lpTemplateName    = NULL ;
     }

/* --------------------------------------
   FileOpenDlg: modified from Petzold
   Version date = 99.10.03
----------------------------------------*/

BOOL FileOpenDlg (PSTR pstrFileName, PSTR pstrTitleName)
     {
     oFileName.lpstrFile         = pstrFileName ;
     oFileName.lpstrFileTitle    = pstrTitleName ;
     oFileName.Flags             = OFN_HIDEREADONLY | OFN_CREATEPROMPT ;

     return GetOpenFileName (&oFileName) ;
     }

/* --------------------------------------
   FileSaveDlg: modified from Petzold
   Version date = 99.10.03
----------------------------------------*/

BOOL FileSaveDlg (PSTR pstrFileName, PSTR pstrTitleName)
     {
     oFileName.lpstrFile         = pstrFileName ;
     oFileName.lpstrFileTitle    = pstrTitleName ;
     oFileName.Flags             = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT ;

     return GetSaveFileName (&oFileName) ;
     }

/* --------------------------------------
   LoadConfiguration
   Version date = 03.09.24
----------------------------------------*/

int LoadConfiguration (int iSource) 
  {
  char    sBuf [80] ;
  int     iI ;

  iMapCounter = 0 ;
  iMapSkip = 0 ;
  iMapSkipFactor = 1 ;
  bEasedJibs = FALSE ;
  bEasedStaySails = FALSE ;
  SendMessage (hwndCheck[2], BM_SETCHECK, 1, 0);
  bTracking = TRUE ;
  CheckMenuItem (hMenu, IDM_TRACKSHIP, MF_CHECKED) ;
  CheckMenuItem (hMenu, IDM_DONTTRACKSHIP, MF_UNCHECKED) ;
  
  if (iSource == 1) 
    {
    hFile = CreateFile (sFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0,0) ;
    ReadLine (hFile, sBuf) ;
    if ((strcmp(sBuf , "Brig simulator      : 2003.04.09") != 0) && (strcmp(sBuf , "Frigate simulator   : 2003.09.07") != 0))
      {
      MessageBox (hwnd, "Illegal ssf file for this version of the simulator", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
      MessageBox (hwnd, sBuf, sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
      CloseHandle (hFile) ; 
      return 1 ;
      }
    if (strcmp(sBuf , "Brig simulator      : 2003.04.09") == 0)
      bBrig = TRUE ;
    else
      bBrig= FALSE ;
    } 
  if (iSource == 2)
    {
    hFile = CreateFile (sFileName2, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0,0) ;
    ReadLine (hFile, sBuf) ;
    if ((strcmp(sBuf , "Brig recording      : 2003.04.09") != 0) && (strcmp(sBuf , "Frigate recording   : 2003.09.07") != 0))
      {
      MessageBox (hwnd, "Illegal srf file for this version of the simulator", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
      CloseHandle (hFile) ; 
      return 1 ;
      }
    if (strcmp(sBuf , "Brig recording      : 2003.04.09") == 0)
      bBrig = TRUE ;
    else
      bBrig= FALSE ;
    }
  if (bBrig)
    {
    iNrSails = 18 ;
    iSpanker = 10 ;  // number of the spanker in the sail plan
    }
  else
    { 
    iNrSails = 13 ;
    iSpanker = 11 ;
    }
  ReadLine (hFile, sBuf) ;
  fShipLength = Extract(sBuf,-1) ;
  ReadLine (hFile, sBuf) ;
  fQDeckLength = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fFocleLength = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fGunDeckHeight = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fQDeckHeight = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fCORDistance = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHullCentroid = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fShipWidth = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fShipDraught = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fShipTons = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMastDistance[0] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMastDistance[1] = Extract(sBuf,-1) ; 
  if (bBrig)
    fMastDistance[2] = 0 ; // no mizzen mast
  else
    {
    ReadLine (hFile, sBuf) ;
    fMastDistance[2] = Extract(sBuf,-1) ; 
    }     
  ReadLine (hFile, sBuf) ;
  fLowestSailFoot[0] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fLowestSailFoot[1] = Extract(sBuf,-1) ; 
  if (bBrig)
    fLowestSailFoot[2] = 0 ; // no mizzen mast
  else
    {
    ReadLine (hFile, sBuf) ;
    fLowestSailFoot[2] = Extract(sBuf,-1) ; 
    }  
  ReadLine (hFile, sBuf) ;
  fLowestSailFoot[3] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSetFraction[1] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSetFraction[2] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSetFraction[3] = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxYardAngle[0] = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxYardAngle[1] = (int) Extract(sBuf,-1) ; 
  if (bBrig)
    iMaxYardAngle[2] = 0 ; // no mizzen mast
  else
    {
    ReadLine (hFile, sBuf) ;
    iMaxYardAngle[2] = Extract(sBuf,-1) ; 
    }     
  ReadLine (hFile, sBuf) ;
  iMaxYardAngle[3] = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fRudderArea = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxJibAngle = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMaxHelmAngle = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  if (sBuf[22]=='T')
    {
    bModernHelm = TRUE ;
    CheckMenuItem(hMenu, IDM_RUDDERCOMMANDS, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_HELMCOMMANDS, MF_UNCHECKED) ;
    }
  else
    {
    bModernHelm = FALSE ;
    CheckMenuItem(hMenu, IDM_RUDDERCOMMANDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_HELMCOMMANDS, MF_CHECKED) ;
    }
  ReadLine (hFile, sBuf) ;  // skip headers
  for (iI=0 ; iI<=iSpanker ; iI++)
    {
    ReadLine (hFile, sBuf) ;
    fSailHeadWidth[iI] = Extract(sBuf,3) ; 
    fSailFootWidth[iI] = Extract(sBuf,9) ; 
    fSailDepth[iI] = Extract(sBuf,15) ; 
    iSailMaxReefs[iI] = (int) Extract(sBuf,21) ; 
    iSailStatus[iI] = (int) Extract(sBuf,27) ; 
    fSailAngle[iI] = Extract(sBuf,33) ; 
    fSailTheorMaxSpeed[iI] = Extract(sBuf,39) ; 
    }
  ReadLine (hFile, sBuf) ;
  fSpankerPeak = Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;  // skip headers
  if (bBrig)
    {
    for (iI=0 ; iI<7 ; iI++)
      {
      ReadLine (hFile, sBuf) ;
      fSailClewDistance[iI] = Extract(sBuf,3) ; 
      fSailClewElevation[iI] = Extract(sBuf,9) ; 
      fSailTackDistance[iI] = Extract(sBuf,15) ; 
      fSailTackElevation[iI] = Extract(sBuf,21) ; 
      fSailHeadDistance[iI] = Extract(sBuf,27) ; 
      fSailHeadElevation[iI] = Extract(sBuf,33) ; 
      iSailStatus[iSpanker+1+iI] = (int) Extract(sBuf,39) ; 
      fSailAngle[iSpanker+1+iI] = Extract(sBuf,45) ; 
      fSailTheorMaxSpeed[iSpanker+1+iI] = Extract(sBuf,51) ; 
      }
    }
  else
    {
    for (iI=1 ; iI<4 ; iI++)
      {
      ReadLine (hFile, sBuf) ;
      fSailClewDistance[iI] = Extract(sBuf,3) ; 
      fSailClewElevation[iI] = Extract(sBuf,9) ; 
      fSailTackDistance[iI] = Extract(sBuf,15) ; 
      fSailTackElevation[iI] = Extract(sBuf,21) ; 
      fSailHeadDistance[iI] = Extract(sBuf,27) ; 
      fSailHeadElevation[iI] = Extract(sBuf,33) ; 
      iSailStatus[iSpanker+1+iI] = (int) Extract(sBuf,39) ; 
      fSailAngle[iSpanker+1+iI] = Extract(sBuf,45) ; 
      fSailTheorMaxSpeed[iSpanker+1+iI] = Extract(sBuf,51) ; 
      }
    ReadLine (hFile, sBuf) ;
    iJibSet = (int) Extract(sBuf,-1) ; 
    fSailClewDistance[0] = fSailClewDistance[1+iJibSet] ; 
    fSailClewElevation[0] = fSailClewElevation[1+iJibSet] ; 
    fSailTackDistance[0] = fSailTackDistance[1+iJibSet] ; 
    fSailTackElevation[0] = fSailTackElevation[1+iJibSet] ; 
    fSailHeadDistance[0] = fSailHeadDistance[1+iJibSet] ; 
    fSailHeadElevation[0] = fSailHeadElevation[1+iJibSet] ; 
    iSailStatus[iSpanker+1] = iSailStatus[iSpanker+2+iJibSet] ; 
    fSailAngle[iSpanker+1] = fSailAngle[iSpanker+2+iJibSet] ;
    fSailTheorMaxSpeed[iSpanker+1] = fSailTheorMaxSpeed[iSpanker+2+iJibSet] ; 
    }
  ReadLine (hFile, sBuf) ;
  iHelmAngle = (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeading =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSpeed =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fLeewaySpeed =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeelSpeed = 0.0 ;
  fRotationalSpeed = 0.0 ;
  fHeelingAngle =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iMadeSail =  (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fWindDirection =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fWindSpeed =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fLatitude =  Extract(sBuf,-1) ; 
  fLastLat = fLatitude ;
  ReadLine (hFile, sBuf) ;
  fLongitude =  Extract(sBuf,-1) ; 
  fLastLong = fLongitude ;
  if ((fLatitude != 0.00) || (fLongitude != 0.0)) 
    {
    bSavePosition = TRUE ;
    CheckMenuItem(hMenu, IDM_SAVEPOSITION, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_DONTSAVEPOSITION, MF_UNCHECKED) ;
    }
  else
    {
    bSavePosition = FALSE ;
    CheckMenuItem(hMenu, IDM_SAVEPOSITION, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_DONTSAVEPOSITION, MF_CHECKED) ;
    }
  ReadLine (hFile, sBuf) ;
  iDisplayParam =  (int) Extract(sBuf,-1) ; 
  CheckMenuItem(hMenu, IDM_SAILANGLE, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_EXPOSEDPERCENTAGE, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_EFFECTIVEAREA, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_USEFULPERCENTAGE, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_USEFULFORCE, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_FORWARDTHRUST, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_LEEWARDTHRUST, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_ROTATIONALTHRUST, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_HEELINGTHRUST, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, iDisplayParam+322, MF_CHECKED) ;
  ReadLine (hFile, sBuf) ;
  if (sBuf[22]=='T')
    {
    bShowSlipStream = TRUE ;
    CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_UNCHECKED) ;
    }
  else
    {
    bShowSlipStream = FALSE ;
    CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_UNCHECKED) ;
    }
  ReadLine (hFile, sBuf) ;
  if (sBuf[22]=='T')
    {
    bCompassRing = TRUE ;
    CheckMenuItem(hMenu, IDM_COMPASSRING, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_NOCOMPASSRING, MF_UNCHECKED) ;
    }
  else
    {
    bCompassRing = FALSE ;
    CheckMenuItem(hMenu, IDM_NOCOMPASSRING, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_COMPASSRING, MF_UNCHECKED) ;
    }
  ReadLine (hFile, sBuf) ;
  if (sBuf[22]=='T')
    {
    bAutoBearing_2 = TRUE ;
    CheckMenuItem(hMenu, IDM_AUTOBEARING, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_SELECTBEARING, MF_UNCHECKED) ;
    }
  else
    {
    bAutoBearing_2 = FALSE ;
    CheckMenuItem(hMenu, IDM_SELECTBEARING, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_AUTOBEARING, MF_UNCHECKED) ;
    fOffsetAngle = 360.0 ;
    SetWindowText(hwndText, "360") ;
    }
  ReadLine (hFile, sBuf) ;
  if (sBuf[22]=='T')
    {
    bHelmsman = TRUE ;
    bEstablished= FALSE ;
    }
  else
    {
    bHelmsman = FALSE ;
    strcpy(sHelmInstruction,"(await orders)") ;
    }
  ReadLine (hFile, sBuf) ;
  if (sBuf[22]=='T')
    bYardsmen = TRUE ;
  else
    bYardsmen = FALSE ;
  ReadLine (hFile, sBuf) ;
  if (sBuf[22]=='T')
    {
    bCoupleFore = TRUE ;
    SendMessage (hwndCheck[0], BM_SETCHECK, 1, 0);
    }
  else
    {
    bCoupleFore = FALSE ;
    SendMessage (hwndCheck[0], BM_SETCHECK, 0, 0);
    }
  if (!bBrig)
    {
    ReadLine (hFile, sBuf) ;
    if (sBuf[22]=='T')
      {
      bCoupleMizzen = TRUE ;
      SendMessage (hwndCheck[1], BM_SETCHECK, 1, 0);
      }
    else
      {
      bCoupleMizzen = FALSE ;
      SendMessage (hwndCheck[1], BM_SETCHECK, 0, 0);
      }
    }
  ReadLine (hFile, sBuf) ;
  iCloseHauledAngle =  (int) Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  iTackStatus =  (int) Extract(sBuf,-1) ; 
  if (iTackStatus > 0)
    {
    for (iI=0 ; iI<7 ; iI++)
      {
      ReadLine (hFile, sBuf) ;
      iSailOldSSStatus[iI] = Extract(sBuf,25) ; 
      }
    ReadLine (hFile, sBuf) ;
    iOldJibAngle = (int) Extract(sBuf,-1) ; 
    ReadLine (hFile, sBuf) ;
    iOldStaySailAngle = (int) Extract(sBuf,-1) ; 
    ReadLine (hFile, sBuf) ;
    iOldSpankerAngle = (int) Extract(sBuf,-1) ; 
    }
  ReadLine (hFile, sBuf) ;
  fRequiredHeading =  Extract(sBuf,-1) ; 
    {
    float fDeviation ;
    fDeviation = fHeading - fRequiredHeading ;
    if (fDeviation > 180.0)
      fDeviation = fHeading - 360.0 ;
    if (fDeviation < -180.0)
      fDeviation = fDeviation + 360.0 ;
    if (bHelmsman)
      {
      if (fabs(fDeviation) < 1.0)
        sprintf(sHelmInstruction, "steering %3.0f ", fRequiredHeading); 
      else
        sprintf(sHelmInstruction, "steer %3.0f ", fRequiredHeading); 
      }
    else
      strcpy(sHelmInstruction,"(await orders)") ;
    }
  ReadLine (hFile, sBuf) ;
  iYardLevelShown =  (int) Extract(sBuf,-1) ; 
  CheckMenuItem(hMenu, IDM_ROYALYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_TOPGALYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_UPPERTOPSAILYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_LOWERTOPSAILYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_LOWERYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, iYardLevelShown+310, MF_CHECKED) ;
  ReadLine (hFile, sBuf) ;
  fMapWidth =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fBowFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSternFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fSideFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fWaterDragFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fRudderFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fMassFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHullFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fRotatInertFactor =  Extract(sBuf,-1) ; 
  fOldRotatInertFactor = fRotatInertFactor ; 
  ReadLine (hFile, sBuf) ;
  fRotatDragFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeelAngleFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeelInertFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;
  fHeelDragFactor =  Extract(sBuf,-1) ; 
  ReadLine (hFile, sBuf) ;  // reserved for expansion
  ReadLine (hFile, sBuf) ;  // reserved for expansion
  ReadLine (hFile, sBuf) ;  // reserved for expansion
  ReadLine (hFile, sBuf) ;  // reserved for expansion
  ReadLine (hFile, sBuf) ;  // reserved for expansion
  ReadLine (hFile, sBuf) ; // skip header
  for (iI=0 ; iI<13 ; iI++)
    {
    ReadLine (hFile, sBuf) ;
    fIsleLat[iI] = Extract(sBuf,3) ; 
    fIsleLong[iI] = Extract(sBuf,12) ; 
    fIsleRadius[iI] = Extract(sBuf,21) ; 
    }
  ReadLine (hFile, sBuf) ;

  if ((iSource == 1) && (strcmp(sBuf , "End of file") != 0))
    {
    MessageBox (hwnd, "Corrupted data file!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
    CloseHandle (hFile) ; 
    return 1 ;
    } 

  // map
  fMapRadius = fMapWidth / 2.0 ;
  fOneTenth = 0.2 * fMapRadius ;
  bMapMustBePainted = TRUE ;

  // wind
  fRelWindAngle = fHeading - (fWindDirection + 180.0) ;
  if (fRelWindAngle < -180.0) 
    fRelWindAngle = fRelWindAngle + 360.0 ;
  if (fRelWindAngle > 180.0)
    fRelWindAngle = fRelWindAngle - 360.0 ;
  fNewWindDirection = fWindDirection ;
  fNewWindSpeed = fWindSpeed ;

  if (iSource == 1)
    CloseHandle (hFile) ; 

  return 0 ;
  }
  
/* --------------------------------------
   LoadDefault
   Version date = 03.09.17
----------------------------------------*/

void LoadDefault () 
  {

  int      iI ;
  iMapCounter = 0 ;
  iMapSkip = 0 ;
  iMapSkipFactor = 1 ;
  bEasedJibs = FALSE ;
  bEasedStaySails = FALSE ;
  SendMessage (hwndCheck[0], BM_SETCHECK, 0, 0);
  SendMessage (hwndCheck[1], BM_SETCHECK, 0, 0);
  SendMessage (hwndCheck[2], BM_SETCHECK, 1, 0);
  bTracking = TRUE ;
  CheckMenuItem (hMenu, IDM_TRACKSHIP, MF_CHECKED) ;
  CheckMenuItem (hMenu, IDM_DONTTRACKSHIP, MF_UNCHECKED) ;

  strcpy(sHelmInstruction,"(await orders)") ;
  SetWindowText(hwndText,"360") ;
  fOffsetAngle = 360.0 ;

  if (bBrig)
    {
    iNrSails = 18 ;
    iSpanker = 10 ;  // number of the spanker in the sail plan
    fShipLength = 160.0 ;
    fQDeckLength = 10.0 ; 
    fFocleLength = 110.0 ; 
    fGunDeckHeight = 18.0 ; 
    fQDeckHeight = 14.0 ;
    fCORDistance = 10.0 ;
    fHullCentroid = -16.0 ; 
    fShipWidth = 32.0 ; 
    fShipDraught = 15.0 ; 
    fShipTons = 580.0 ; 
    fMastDistance[0] = 27.0 ; 
    fMastDistance[1] = -27.0 ; 
    fMastDistance[2] = 0.0 ;  // no mizzen mast 
    fLowestSailFoot[0] = 19.0 ; 
    fLowestSailFoot[1] = 23.0 ; 
    fLowestSailFoot[2] = 0.0 ; // no mizzen mast  
    fLowestSailFoot[3] = 26.0 ; 
    fSetFraction[0] = 1.0 ; // set 
    fSetFraction[1] = 0.85 ; // one reef
    fSetFraction[2] = 0.70 ; // two reefs
    fSetFraction[3] = 0.55 ; // three reefs
    fSetFraction[4] = 0.0 ; // furled
    fSetFraction[5] = 0.0 ; // blown out 
    iMaxYardAngle[0] = 55 ; 
    iMaxYardAngle[1] = 55 ; 
    iMaxYardAngle[2] = 0 ; // no mizzen yards
    iMaxYardAngle[3] = 12 ; 
    fRudderArea = 64.0 ; 
    iMaxJibAngle = 30 ; 
    iMaxHelmAngle = 45 ; 
    bModernHelm= TRUE ;
    CheckMenuItem(hMenu, IDM_RUDDERCOMMANDS, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_HELMCOMMANDS, MF_UNCHECKED) ;
    fSailHeadWidth[0] = 22.0 ; 
    fSailHeadWidth[1] = 29.0 ; 
    fSailHeadWidth[2] = 41.0 ; 
    fSailHeadWidth[3] = 48.0 ; 
    fSailHeadWidth[4] = 54.0 ; 
    fSailHeadWidth[5] = 22.0 ; 
    fSailHeadWidth[6] = 29.0 ; 
    fSailHeadWidth[7] = 41.0 ; 
    fSailHeadWidth[8] = 48.0 ; 
    fSailHeadWidth[9] = 54.0 ; 
    fSailHeadWidth[10] = 38.0 ; 
    fSailFootWidth[0] = 29.0 ; 
    fSailFootWidth[1] = 41.0 ; 
    fSailFootWidth[2] = 48.0 ; 
    fSailFootWidth[3] = 54.0 ; 
    fSailFootWidth[4] = 54.0 ; 
    fSailFootWidth[5] = 29.0 ; 
    fSailFootWidth[6] = 41.0 ; 
    fSailFootWidth[7] = 48.0 ; 
    fSailFootWidth[8] = 54.0 ; 
    fSailFootWidth[9] = 54.0 ; 
    fSailFootWidth[10] = 38.0 ; 
    fSailDepth[0] = 9.0 ; 
    fSailDepth[1] = 18.0 ; 
    fSailDepth[2] = 14.0 ; 
    fSailDepth[3] = 13.0 ; 
    fSailDepth[4] = 20.0 ; 
    fSailDepth[5] = 9.0 ; 
    fSailDepth[6] = 18.0 ; 
    fSailDepth[7] = 14.0 ; 
    fSailDepth[8] = 13.0 ; 
    fSailDepth[9] = 20.0 ; 
    fSailDepth[10] = 18.0 ; 
    iSailMaxReefs[0] = 0 ; 
    iSailMaxReefs[1] = 0 ; 
    iSailMaxReefs[2] = 0 ; 
    iSailMaxReefs[3] = 0 ; 
    iSailMaxReefs[4] = 0 ; 
    iSailMaxReefs[5] = 0 ; 
    iSailMaxReefs[6] = 0 ; 
    iSailMaxReefs[7] = 0 ; 
    iSailMaxReefs[8] = 0 ; 
    iSailMaxReefs[9] = 0 ; 
    iSailMaxReefs[10] = 0 ; 
    iSailStatus[0] = 0 ; 
    iSailStatus[1] = 0 ; 
    iSailStatus[2] = 0 ; 
    iSailStatus[3] = 0 ; 
    iSailStatus[4] = 0 ; 
    iSailStatus[5] = 0 ; 
    iSailStatus[6] = 0 ; 
    iSailStatus[7] = 0 ; 
    iSailStatus[8] = 0 ; 
    iSailStatus[9] = 4 ; 
    iSailStatus[10] = 0 ; 
    fSailAngle[0] = -49 ; 
    fSailAngle[1] = -49 ; 
    fSailAngle[2] = -49 ; 
    fSailAngle[3] = -49 ; 
    fSailAngle[4] = -49 ; 
    fSailAngle[5] = -49 ; 
    fSailAngle[6] = -49 ; 
    fSailAngle[7] = -49 ; 
    fSailAngle[8] = -49 ; 
    fSailAngle[9] = -49 ; 
    fSailAngle[10] = 10 ; 
    fSailTheorMaxSpeed[0] = 25.0 ; 
    fSailTheorMaxSpeed[1] = 30.0 ; 
    fSailTheorMaxSpeed[2] = 35.0 ; 
    fSailTheorMaxSpeed[3] = 50.0 ; 
    fSailTheorMaxSpeed[4] = 40.0 ; 
    fSailTheorMaxSpeed[5] = 25.0 ; 
    fSailTheorMaxSpeed[6] = 30.0 ; 
    fSailTheorMaxSpeed[7] = 35.0 ; 
    fSailTheorMaxSpeed[8] = 45.0 ; 
    fSailTheorMaxSpeed[9] = 40.0 ; 
    fSailTheorMaxSpeed[10] = 30.0 ; 
    fSpankerPeak = 13 ;
    fSailClewDistance[0]= 71.0 ;
    fSailClewDistance[1]= 66.0 ;
    fSailClewDistance[2]= 60.0 ;
    fSailClewDistance[3]= 53.0 ;
    fSailClewDistance[4]= -3.0 ;
    fSailClewDistance[5]= -6.0 ;
    fSailClewDistance[6]= -8.0 ;
    fSailClewElevation[0]= 39.0 ;
    fSailClewElevation[1]= 35.0 ;
    fSailClewElevation[2]= 29.0 ;
    fSailClewElevation[3]= 24.0 ;
    fSailClewElevation[4]= 60.0 ;
    fSailClewElevation[5]= 46.0 ;
    fSailClewElevation[6]= 26.0 ;
    fSailTackDistance[0]= 102.0 ;
    fSailTackDistance[1]= 94.0 ;
    fSailTackDistance[2]= 89.0 ;
    fSailTackDistance[3]= 78.0 ;
    fSailTackDistance[4]= 20.0 ;
    fSailTackDistance[5]= 21.0 ;
    fSailTackDistance[6]= 21.0 ;
    fSailTackElevation[0]= 26.0 ;
    fSailTackElevation[1]= 25.0 ;
    fSailTackElevation[2]= 22.0 ;
    fSailTackElevation[3]= 20.0 ;
    fSailTackElevation[4]= 59.0 ;
    fSailTackElevation[5]= 45.0 ;
    fSailTackElevation[6]= 18.0 ;
    fSailHeadDistance[0]= 58.0 ;
    fSailHeadDistance[1]= 53.0 ;
    fSailHeadDistance[2]= 48.0 ;
    fSailHeadDistance[3]= 43.0 ;
    fSailHeadDistance[4]= -10.0 ;
    fSailHeadDistance[5]= -12.0 ;
    fSailHeadDistance[6]= -12.0 ;
    fSailHeadElevation[0]= 67.0 ;
    fSailHeadElevation[1]= 58.0 ;
    fSailHeadElevation[2]= 55.0 ;
    fSailHeadElevation[3]= 57.0 ;
    fSailHeadElevation[4]= 88.0 ;
    fSailHeadElevation[5]= 78.0 ;
    fSailHeadElevation[6]= 62.0 ;
    iSailStatus[11] = 0 ; 
    iSailStatus[12] = 0 ; 
    iSailStatus[13] = 0 ; 
    iSailStatus[14] = 0 ; 
    iSailStatus[15] = 0 ; 
    iSailStatus[16] = 0 ; 
    iSailStatus[17] = 0 ; 
    iSailStatus[18] = 0 ; // needed to prevent supression of value on display
    iSailStatus[19] = 0 ; 
    iSailStatus[20] = 0 ; 
    iSailStatus[21] = 0 ; 
    iSailStatus[22] = 0 ; 
    iSailMaxReefs[11] = 0 ; 
    iSailMaxReefs[12] = 0 ; 
    iSailMaxReefs[13] = 0 ; 
    iSailMaxReefs[14] = 0 ; 
    iSailMaxReefs[15] = 0 ; 
    iSailMaxReefs[16] = 0 ; 
    iSailMaxReefs[17] = 0 ; 
    fSailAngle[11] = 15 ; 
    fSailAngle[12] = 15 ; 
    fSailAngle[13] = 15 ; 
    fSailAngle[14] = 15 ; 
    fSailAngle[15] = 15 ; 
    fSailAngle[16] = 15 ; 
    fSailAngle[17] = 15 ; 
    fSailTheorMaxSpeed[11] = 30.0 ; 
    fSailTheorMaxSpeed[12] = 35.0 ; 
    fSailTheorMaxSpeed[13] = 30.0 ; 
    fSailTheorMaxSpeed[14] = 50.0 ; 
    fSailTheorMaxSpeed[15] = 20.0 ; 
    fSailTheorMaxSpeed[16] = 20.0 ; 
    fSailTheorMaxSpeed[17] = 40.0 ; 
    iHelmAngle = 22 ; 
    fHeading =  150.33 ; 
    fSpeed =  4.27 ; 
    fLeewaySpeed =  -0.83 ; 
    fHeelingAngle = -3.0 ; 
    fHeelSpeed = 0.0 ;
    fRotationalSpeed = 0.0 ;
    iMadeSail =  15 ; 
    fWindDirection =  230.0 ; 
    fWindSpeed =  16.0 ;
    bSavePosition = FALSE ;
    CheckMenuItem(hMenu, IDM_SAVEPOSITION, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_DONTSAVEPOSITION, MF_CHECKED) ;
    fLatitude = 0.0 ;
    fLastLat = 0.0 ;
    fLongitude = 0.0 ;
    fLastLong = 0.0 ;
    iDisplayParam = 8 ;
    CheckMenuItem(hMenu, IDM_SAILANGLE, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_EXPOSEDPERCENTAGE, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_EFFECTIVEAREA, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_USEFULPERCENTAGE, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_USEFULFORCE, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_FORWARDTHRUST, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_LEEWARDTHRUST, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_ROTATIONALTHRUST, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_HEELINGTHRUST, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, iDisplayParam+322, MF_CHECKED) ;
    bShowSlipStream = TRUE ;
    CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_UNCHECKED) ;
    bCompassRing = TRUE ;
    CheckMenuItem (hMenu, IDM_COMPASSRING, MF_CHECKED) ;
    CheckMenuItem (hMenu, IDM_NOCOMPASSRING, MF_UNCHECKED) ;
    bEstablished = FALSE ;
    bAutoBearing = FALSE ;
    CheckMenuItem (hMenu, IDM_SELECTBEARING, MF_CHECKED) ;
    CheckMenuItem (hMenu, IDM_AUTOBEARING, MF_UNCHECKED) ;
    fOffsetAngle = 0.0 ;
    bYardsmen = FALSE ;
    bHelmsman = FALSE ;
    bCoupleFore = FALSE ;
    bCoupleMizzen = FALSE ;
    iCloseHauledAngle = 25 ; 
    iTackStatus = 0 ;
    fHullFactor = 0.3 ; 
    fRequiredHeading = -10.0 ;
    iYardLevelShown = 4 ; 
    CheckMenuItem(hMenu, IDM_ROYALYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_TOPGALYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_UPPERTOPSAILYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_LOWERTOPSAILYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_LOWERYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, iYardLevelShown+310, MF_CHECKED) ;
    fMapWidth =  0.2 ; 
    fBowFactor =  0.3 ; 
    fSternFactor =  5.0 ; 
    fSideFactor =  10.0 ; 
    fWaterDragFactor =  100.0 ; 
    fRudderFactor =  770.0 ;
    fMassFactor =  1.0 ; 
    fRotatInertFactor =  0.8 ; 
    fOldRotatInertFactor =  0.8 ; 
    fRotatDragFactor =  200.0 ; 
    fHeelAngleFactor =  0.0800 ; 
    fHeelInertFactor =  0.0002 ; //<<=========================== too small at high simulator speed
    fHeelInertFactor =  0.001 ; 
    fHeelDragFactor =  0.5 ;
    }
  else
    {
    // frigate
    iNrSails = 13 ;
    iSpanker = 11 ;  // number of the spanker in the sail plan
    fShipLength = 146.0 ;
    fQDeckLength = 70.0 ; 
    fFocleLength = 34.0 ; 
    fGunDeckHeight = 15.0 ; 
    fQDeckHeight = 22.0 ;
    fCORDistance = 5.0 ;
    fHullCentroid = 0.0 ; 
    fShipWidth = 38.0 ; 
    fShipDraught = 19.0 ; 
    fShipTons = 850.0 ; 
    fMastDistance[0] = 52.0 ; 
    fMastDistance[1] = -15.0 ; 
    fMastDistance[2] = -52.0 ; 
    fLowestSailFoot[0] = 26.0 ; 
    fLowestSailFoot[1] = 25.0 ; 
    fLowestSailFoot[2] = 61.0 ; 
    fLowestSailFoot[3] = 26.0 ;  
    fSetFraction[0] = 1.0 ; // set 
    fSetFraction[1] = 0.85 ; // one reef
    fSetFraction[2] = 0.70 ; // two reefs
    fSetFraction[3] = 0.55 ; // three reefs
    fSetFraction[4] = 0.0 ; // furled
    fSetFraction[5] = 0.0 ; // blown out 
    iMaxYardAngle[0] = 60 ; 
    iMaxYardAngle[1] = 60 ; 
    iMaxYardAngle[2] = 60 ; 
    iMaxYardAngle[3] = 25 ; 
    fRudderArea = 64.0 ; 
    iMaxJibAngle = 30 ; 
    iMaxHelmAngle = 55 ; 
    bModernHelm= FALSE ;
    CheckMenuItem(hMenu, IDM_RUDDERCOMMANDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_HELMCOMMANDS, MF_CHECKED) ;
    fSailHeadWidth[0] = 26.0 ; 
    fSailHeadWidth[1] = 31.0 ; 
    fSailHeadWidth[2] = 42.0 ; 
    fSailHeadWidth[3] = 60.0 ; 
    fSailHeadWidth[4] = 26.0 ; 
    fSailHeadWidth[5] = 34.0 ; 
    fSailHeadWidth[6] = 47.0 ; 
    fSailHeadWidth[7] = 66.0 ; 
    fSailHeadWidth[8] = 19.0 ; 
    fSailHeadWidth[9] = 26.0 ; 
    fSailHeadWidth[10] = 36.0 ; 
    fSailHeadWidth[11] = 50.0 ; 
    fSailFootWidth[0] = 31.0 ; 
    fSailFootWidth[1] = 42.0 ; 
    fSailFootWidth[2] = 62.0 ; 
    fSailFootWidth[3] = 60.0 ; 
    fSailFootWidth[4] = 34.0 ; 
    fSailFootWidth[5] = 47.0 ; 
    fSailFootWidth[6] = 66.0 ; 
    fSailFootWidth[7] = 66.0 ; 
    fSailFootWidth[8] = 26.0 ; 
    fSailFootWidth[9] = 36.0 ; 
    fSailFootWidth[10] = 42.0 ; 
    fSailFootWidth[11] = 50.0 ; 
    fSailDepth[0] = 9.0 ; 
    fSailDepth[1] = 21.0 ; 
    fSailDepth[2] = 35.0 ; 
    fSailDepth[3] = 32.0 ; 
    fSailDepth[4] = 10.0 ; 
    fSailDepth[5] = 26.0 ; 
    fSailDepth[6] = 40.0 ; 
    fSailDepth[7] = 35.0 ; 
    fSailDepth[8] = 8.0 ; 
    fSailDepth[9] = 20.0 ; 
    fSailDepth[10] = 43.0 ; 
    fSailDepth[11] = 32.0 ; 
    iSailMaxReefs[0] = 0 ; 
    iSailMaxReefs[1] = 0 ; 
    iSailMaxReefs[2] = 3 ; 
    iSailMaxReefs[3] = 2 ; 
    iSailMaxReefs[4] = 0 ; 
    iSailMaxReefs[5] = 0 ; 
    iSailMaxReefs[6] = 3 ; 
    iSailMaxReefs[7] = 2 ; 
    iSailMaxReefs[8] = 0 ; 
    iSailMaxReefs[9] = 0 ; 
    iSailMaxReefs[10] = 3 ; 
    iSailMaxReefs[11] = 2 ; 
    iSailStatus[0] = 4 ; 
    iSailStatus[1] = 0 ; 
    iSailStatus[2] = 0 ; 
    iSailStatus[3] = 0 ; 
    iSailStatus[4] = 4 ; 
    iSailStatus[5] = 0 ; 
    iSailStatus[6] = 0 ; 
    iSailStatus[7] = 0 ; 
    iSailStatus[8] = 4 ; 
    iSailStatus[9] = 0 ; 
    iSailStatus[10] = 0 ; 
    iSailStatus[11] = 0 ; 
    fSailAngle[0] = -55 ; 
    fSailAngle[1] = -55 ; 
    fSailAngle[2] = -55 ; 
    fSailAngle[3] = -55 ; 
    fSailAngle[4] = -55 ; 
    fSailAngle[5] = -55 ; 
    fSailAngle[6] = -55 ; 
    fSailAngle[7] = -55 ; 
    fSailAngle[8] = -55 ; 
    fSailAngle[9] = -55 ; 
    fSailAngle[10] = -55 ; 
    fSailAngle[11] = 13 ; 
    fSailTheorMaxSpeed[0] = 20.0 ; 
    fSailTheorMaxSpeed[1] = 25.0 ; 
    fSailTheorMaxSpeed[2] = 35.0 ; 
    fSailTheorMaxSpeed[3] = 50.0 ; 
    fSailTheorMaxSpeed[4] = 20.0 ; 
    fSailTheorMaxSpeed[5] = 25.0 ; 
    fSailTheorMaxSpeed[6] = 35.0 ; 
    fSailTheorMaxSpeed[7] = 50.0 ; 
    fSailTheorMaxSpeed[8] = 20.0 ; 
    fSailTheorMaxSpeed[9] = 25.0 ; 
    fSailTheorMaxSpeed[10] = 35.0 ; 
    fSailTheorMaxSpeed[11] = 50.0 ; 
    fSpankerPeak = 12 ;
    fSailClewDistance[0]= 89.0 ;
    fSailClewElevation[0]= 36.0 ;
    fSailTackDistance[0]= 134.0 ;
    fSailTackElevation[0]= 35.0 ;
    fSailHeadDistance[0]= 54.0 ;
    fSailHeadElevation[0]= 95.0 ;
    fSailClewDistance[1]= 89.0 ;
    fSailClewElevation[1]= 36.0 ;
    fSailTackDistance[1]= 134.0 ;
    fSailTackElevation[1]= 35.0 ;
    fSailHeadDistance[1]= 54.0 ;
    fSailHeadElevation[1]= 95.0 ;
    fSailClewDistance[2]= 94.0 ;
    fSailClewElevation[2]= 36.0 ;
    fSailTackDistance[2]= 134.0 ;
    fSailTackElevation[2]= 35.0 ;
    fSailHeadDistance[2]= 54.0 ;
    fSailHeadElevation[2]= 95.0 ;
    fSailClewDistance[3]= 114.0 ;
    fSailClewElevation[3]= 36.0 ;
    fSailTackDistance[3]= 134.0 ;
    fSailTackElevation[3]= 35.0 ;
    fSailHeadDistance[3]= 54.0 ;
    fSailHeadElevation[3]= 95.0 ;
    iJibSet = 0 ;
    iSailStatus[12] = 0 ; 
    iSailMaxReefs[12] = 0 ; 
    fSailAngle[12] = 12 ; 
    fSailTheorMaxSpeed[12] = 30.0 ; 
    fSailTheorMaxSpeed[13] = 30.0 ; 
    fSailTheorMaxSpeed[14] = 35.0 ; 
    fSailTheorMaxSpeed[15] = 50.0 ; 
    iHelmAngle = 23 ; 
    fHeading =  135.0 ; 
    fSpeed =  5.8 ; 
    fLeewaySpeed =  -1.3 ; 
    fHeelSpeed = 0.0 ;
    fRotationalSpeed = 0.0 ;
    fHeelingAngle = -11.0 ; 
    iMadeSail =  7 ; 
    fWindDirection =  225.0 ; 
    fWindSpeed =  15.0 ;
    bSavePosition = FALSE ;
    CheckMenuItem(hMenu, IDM_SAVEPOSITION, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_DONTSAVEPOSITION, MF_CHECKED) ;
    fLatitude = 0.0 ;
    fLastLat = 0.0 ;
    fLongitude = 0.0 ;
    fLastLong = 0.0 ;
    iDisplayParam = 8 ;
    CheckMenuItem(hMenu, IDM_SAILANGLE, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_EXPOSEDPERCENTAGE, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_EFFECTIVEAREA, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_USEFULPERCENTAGE, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_USEFULFORCE, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_FORWARDTHRUST, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_LEEWARDTHRUST, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_ROTATIONALTHRUST, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_HEELINGTHRUST, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, iDisplayParam+322, MF_CHECKED) ;
    bShowSlipStream = TRUE ;
    CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_UNCHECKED) ;
    bCompassRing = TRUE ;
    CheckMenuItem (hMenu, IDM_COMPASSRING, MF_CHECKED) ;
    CheckMenuItem (hMenu, IDM_NOCOMPASSRING, MF_UNCHECKED) ;
    bEstablished = FALSE ;
    bAutoBearing = FALSE ;
    CheckMenuItem (hMenu, IDM_SELECTBEARING, MF_CHECKED) ;
    CheckMenuItem (hMenu, IDM_AUTOBEARING, MF_UNCHECKED) ;
    fOffsetAngle = 0.0 ;
    bYardsmen = FALSE ;
    bHelmsman = FALSE ;
    bCoupleFore = FALSE ;
    bCoupleMizzen = FALSE ;
    iCloseHauledAngle = 25 ; 
    iTackStatus = 0 ;
    fHullFactor = 0.3 ; 
    fRequiredHeading = -10.0 ;
    iYardLevelShown = 4 ; 
    CheckMenuItem(hMenu, IDM_ROYALYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_TOPGALYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_UPPERTOPSAILYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_LOWERTOPSAILYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_LOWERYARDS, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, iYardLevelShown+310, MF_CHECKED) ;
    fMapWidth =  0.2 ; 
    fBowFactor =  0.3 ; 
    fSternFactor =  5.0 ; 
    fSideFactor =  10.0 ; 
    fWaterDragFactor =  66.0 ; 
    fRudderFactor =  770.0 ;
    fMassFactor =  1.0 ; 
    fRotatInertFactor =  0.8 ; 
    fOldRotatInertFactor =  1.0 ; 
    fRotatDragFactor =  200.0 ; 
    fHeelAngleFactor =  0.025 ; 
    fHeelInertFactor =  0.0002 ; //<<=========================== too small at high simulator speed
    fHeelInertFactor =  0.001 ; 
    fHeelDragFactor =  0.5 ;
    }
  iSpeedFactor = 0 ; 
  fIsleLat[0] = -0.087 ; 
  fIsleLat[1] = -0.1 ; 
  fIsleLat[2] = -0.32 ; 
  fIsleLat[3] = -0.43 ; 
  fIsleLat[4] = -0.26 ; 
  fIsleLat[5] = -5.0 ; 
  fIsleLat[6] = 0.0 ; 
  fIsleLat[7] = 0.0 ; 
  fIsleLat[8] = 0.0 ; 
  fIsleLat[9] = 0.0 ; 
  fIsleLat[10] = 0.0 ; 
  fIsleLat[11] = 0.0 ; 
  fIsleLat[12] = 0.0 ; 
  fIsleLong[0] = 0.035 ; 
  fIsleLong[1] = 0.2 ; 
  fIsleLong[2] = 0.35 ; 
  fIsleLong[3] = 0.18 ; 
  fIsleLong[4] = 0.0 ; 
  fIsleLong[5] = 3.0 ; 
  fIsleLong[6] = 0.0 ; 
  fIsleLong[7] = 0.0 ; 
  fIsleLong[8] = 0.0 ; 
  fIsleLong[9] = 0.0 ; 
  fIsleLong[10] = 0.0 ; 
  fIsleLong[11] = 0.0 ; 
  fIsleLong[12] = 0.0 ; 
  fIsleRadius[0] = 0.016 ; 
  fIsleRadius[1] = 0.05 ; 
  fIsleRadius[2] = 0.015 ; 
  fIsleRadius[3] = 0.012 ; 
  fIsleRadius[4] = 0.008 ; 
  fIsleRadius[5] = 1.5 ; 
  fIsleRadius[6] = 0.0 ; 
  fIsleRadius[7] = 0.0 ; 
  fIsleRadius[8] = 0.0 ; 
  fIsleRadius[9] = 0.0 ; 
  fIsleRadius[10] = 0.0 ; 
  fIsleRadius[11] = 0.0 ; 
  fIsleRadius[12] = 0.0 ; 

  // needed to prevent supression of value on display
  for (iI=iNrSails; iI<iNrSails+5; iI++)
    iSailStatus[iI] = 0 ;

  // map
  fMapRadius = fMapWidth / 2.0 ;
  fOneTenth = 0.2 * fMapRadius ;
  bMapMustBePainted = TRUE ;

  // wind
  fRelWindAngle = fHeading - (fWindDirection + 180.0) ;
  if (fRelWindAngle < -180.0) 
    fRelWindAngle = fRelWindAngle + 360.0 ;
  if (fRelWindAngle > 180.0)
    fRelWindAngle = fRelWindAngle - 360.0 ;
  fNewWindDirection = fWindDirection ;
  fNewWindSpeed = fWindSpeed ;

  }

/* --------------------------------------
   ReadLine: Reads a line from file
   Version date = 03.04.09
----------------------------------------*/

void ReadLine (HANDLE hFile, char * sBuf)
  {
  char   sBlock[80] ;
  int    iI, iLength ;
  DWORD  dRead ;
 
  ReadFile (hFile, sBlock, 80, &dRead,  NULL) ;
  sBlock[(int) dRead-1]='\r' ;

  iI = 0 ;
  while (sBlock[iI]!='\r')
    iI++ ;
  iLength = iI ;
  SetFilePointer (hFile,-dRead+iLength+2,NULL,FILE_CURRENT) ;	
  for (iI=0 ; iI<80 ; iI++)
    sBuf[iI]=' ' ;
  for (iI=0 ; iI<iLength ; iI++)
    sBuf[iI]=sBlock[iI] ;
  sBuf[iLength]='\0' ;

  }

/* --------------------------------------
   Recorder
   Version date = 03.09.24
----------------------------------------*/

void Recorder (int iParameter, int iElement, int iValue) 
  {
  char    sBuffer [80] ;
  int     iI ;
  DWORD   dToWrite, dWritten ;
  HANDLE  hFile ;

  hFile = CreateFile (sFileName2, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0,0) ;
  SetFilePointer (hFile, 0, 0, FILE_END);
  sprintf (sBuffer, "%012.3f %02d %02d %05d \r\n", fSimTime, iParameter, iElement, iValue) ;
  dToWrite=strlen(sBuffer) ;
  WriteFile (hFile, sBuffer, dToWrite, &dWritten,  NULL) ;
  if (dToWrite!=dWritten)
    {
    MessageBox (hwnd, "Warning: problem saving recording", "Save file", MB_ICONEXCLAMATION | MB_OK) ;
    }
    //sprintf(sMessage, "dWritten %d ", dWritten) ; 
    //MessageBox (hwnd, sMessage, "dWritten", MB_ICONEXCLAMATION | MB_OK) ;
  CloseHandle (hFile) ; 
  }

/* --------------------------------------
   Replayer
   Version date = 03.07.16
----------------------------------------*/

void Replayer () 
  {
  char    sBuffer [80] ;
  int     iI ;

  for (iI=iLastReplayItem+1 ; iI<(iLastReplayItem+11) ; iI++)
    {
    ReadLine (hFile, sBuffer) ;
    fEventTime[iI] = Extract(sBuffer,1) ; 
    iEvent[iI] = Extract(sBuffer,13) ; 
    iEventCode[iI] = Extract(sBuffer,16) ; 
    iEventCode2[iI] = Extract(sBuffer,19) ; 
    if (iEvent[iI] == 99)
      {
      iLastReplayItem = iI ; 
      return 0 ;
      }
    }
    iLastReplayItem = iLastReplayItem + 10 ;
  }

/* --------------------------------------
   SaveConfiguration
   Version date = 03.09.24
----------------------------------------*/

void SaveConfiguration (int iSource) 
  {
  char    sBuffer[3800] ;
  char    sBuf [80] ;
  int     iI ;
  DWORD   dToWrite, dWritten ;
  HANDLE  hFile ;

  if (iSource == 1)
    {
    hFile = CreateFile (sFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0,0) ;
    if (bBrig)
      strcpy (sBuffer, "Brig simulator      : 2003.04.09\r\n") ;
    else
      strcpy (sBuffer, "Frigate simulator   : 2003.09.07\r\n") ;
    }
  else
    {
    hFile = CreateFile (sFileName2, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0,0) ;
    if (bBrig)
      strcpy (sBuffer, "Brig recording      : 2003.04.09\r\n") ;
    else
      strcpy (sBuffer, "Frigate recording   : 2003.09.07\r\n") ;
    }
  sprintf (sBuf, "Ship length         : %04.0f\r\n", fShipLength) ;
  strcat (sBuffer,sBuf) ; 
  sprintf (sBuf, "Quaterdeck length   : %04.0f\r\n", fQDeckLength) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Forecastle length   : %04.0f\r\n", fFocleLength) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Gundeck height      : %04.0f\r\n", fGunDeckHeight) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Quaterdeck height   : %04.0f\r\n", fQDeckHeight) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Coef of Rot distance: %04.0f\r\n", fCORDistance) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Hull Centroid       : %04.0f\r\n", fHullCentroid) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Ship width          : %04.0f\r\n", fShipWidth) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Ship draught        : %04.0f\r\n", fShipDraught) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Ship tonnage        : %04.0f\r\n", fShipTons) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Fore mast distance  : %04.0f\r\n", fMastDistance[0]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Main mast distance  : %04.0f\r\n", fMastDistance[1]) ;
  strcat (sBuffer, sBuf) ; 
  if (!bBrig)
    {
    sprintf (sBuf, "Mizzen mast distance: %04.0f\r\n", fMastDistance[2]) ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "Forecourse foot hgt : %04.0f\r\n", fLowestSailFoot[0]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Maincourse foot hgt : %04.0f\r\n", fLowestSailFoot[1]) ;
  strcat (sBuffer, sBuf) ; 
  if (!bBrig)
    {
    sprintf (sBuf, "Miz. top'sl foot hgt: %04.0f\r\n", fLowestSailFoot[2]) ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "Spanker foot hgt    : %04.0f\r\n", fLowestSailFoot[3]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Set fract 1 reef    : %04.2f\r\n", fSetFraction[1]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Set fract 2 reef    : %04.2f\r\n", fSetFraction[2]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Set fract 3 reef    : %04.2f\r\n", fSetFraction[3]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max foreyard angle  : %04d\r\n", iMaxYardAngle[0]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max mainyard angle  : %04d\r\n", iMaxYardAngle[1]) ;
  strcat (sBuffer, sBuf) ; 
  if (!bBrig)
    {
    sprintf (sBuf, "Max mizzenyard angle: %04d\r\n", iMaxYardAngle[2]) ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "Max spanker angle   : %04d\r\n", iMaxYardAngle[3]) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Rudder area         : %04.0f\r\n", fRudderArea) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max jib angle       : %04d\r\n", iMaxJibAngle) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Max helm angle      : %04d\r\n", iMaxHelmAngle) ;
  strcat (sBuffer, sBuf) ; 
  if (bModernHelm)
    strcpy (sBuf, "Rudder commands     : TRUE \r\n") ;
  else
    strcpy (sBuf, "Rudder commands     : FALSE\r\n") ;
  strcat (sBuffer, sBuf) ; 
  strcpy(sBuf,"Sl HeadW FootW Depth MaxRf Statu SlAng MaxSp\r\n") ;
  strcat (sBuffer, sBuf) ; 
  for (iI=0 ; iI<=iSpanker ; iI++)
    {
    sprintf (sBuf, "%02d %05.0f %05.0f %05.0f %05d %05d %05.0f %05.0f\r\n", 
      iI, fSailHeadWidth[iI],fSailFootWidth[iI],fSailDepth[iI],iSailMaxReefs[iI],iSailStatus[iI],
      fSailAngle[iI],fSailTheorMaxSpeed[iI]) ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "Spanker peak        : %04.0f\r\n", fSpankerPeak) ;
  strcat (sBuffer, sBuf) ; 
  strcpy(sBuf,"Sl ClewD ClewE TackD TackE HeadD HeadE Statu SlAng MaxSp\r\n") ;
  strcat (sBuffer, sBuf) ; 
  if (bBrig)
    {  
    for (iI=0 ; iI<7 ; iI++)
      {
      sprintf (sBuf, "%02d %05.0f %05.0f %05.0f %05.0f %05.0f %05.0f %05d %05.0f %05.0f\r\n", 
        iI, fSailClewDistance[iI],fSailClewElevation[iI],fSailTackDistance[iI],fSailTackElevation[iI],fSailHeadDistance[iI],fSailHeadElevation[iI],
          iSailStatus[iI+1+iSpanker],fSailAngle[iI+1+iSpanker],fSailTheorMaxSpeed[iI+1+iSpanker]) ;
      strcat (sBuffer, sBuf) ; 
      }
    } 
  else
    {  
    for (iI=1 ; iI<4 ; iI++)
      {
      iSailStatus[iSpanker+1+iI] = iSailStatus[iSpanker+1] ;
      fSailAngle[iSpanker+1+iI] = fSailAngle[iSpanker+1] ;
      sprintf (sBuf, "%02d %05.0f %05.0f %05.0f %05.0f %05.0f %05.0f %05d %05.0f %05.0f\r\n", 
        iI, fSailClewDistance[iI],fSailClewElevation[iI],fSailTackDistance[iI],fSailTackElevation[iI],fSailHeadDistance[iI],fSailHeadElevation[iI],
          iSailStatus[iI+1+iSpanker],fSailAngle[iI+1+iSpanker],fSailTheorMaxSpeed[iI+1+iSpanker]) ;
      strcat (sBuffer, sBuf) ; 
      }
    sprintf (sBuf, "Jib selected        : %04d\r\n", iJibSet) ;
    strcat (sBuffer, sBuf) ; 
    } 
  sprintf (sBuf, "Helm angle          : %04d\r\n", iHelmAngle) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heading             : %07.2f\r\n", fHeading) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Speed               : %07.2f\r\n", fSpeed) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Leeway speed        : %07.2f\r\n", fLeewaySpeed) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heeling angle       : %07.2f\r\n", fHeelingAngle) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Sail pattern set    : %04d\r\n", iMadeSail) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "WindDirection       : %04.0f\r\n", fWindDirection) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "WindSpeed           : %04.0f\r\n", fWindSpeed) ;
  strcat (sBuffer, sBuf) ; 
  if (bSavePosition)
    {
    sprintf (sBuf, "Latitude N          : %06.3f\r\n", fLatitude) ;
    strcat (sBuffer, sBuf) ; 
    sprintf (sBuf, "Longitude E         : %06.3f\r\n", fLongitude) ;
    strcat (sBuffer, sBuf) ; 
    }
  else
    {
    strcpy (sBuf, "Latitude N          : 00.000\r\n") ;
    strcat (sBuffer, sBuf) ; 
    strcpy (sBuf, "Longitude E         : 00.000\r\n") ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "Parameter to display: %04d\r\n", iDisplayParam) ;
  strcat (sBuffer, sBuf) ; 
  if (bShowSlipStream)
    strcpy (sBuf, "Show slipstream     : TRUE \r\n") ;
  else
    strcpy (sBuf, "Show slipstream     : FALSE\r\n") ;
  strcat (sBuffer, sBuf) ; 
  if (bCompassRing)
    strcpy (sBuf, "Show compass ring   : TRUE \r\n") ;
  else
    strcpy (sBuf, "Show compass ring   : FALSE\r\n") ;
  strcat (sBuffer, sBuf) ; 
  if (bAutoBearing_2)
    strcpy (sBuf, "Adjust to heading   : TRUE \r\n") ;
  else
    strcpy (sBuf, "Adjust to heading   : FALSE\r\n") ;
  strcat (sBuffer, sBuf) ; 
  if (bHelmsman)
    strcpy (sBuf, "Helmsman active     : TRUE \r\n") ;
  else
    strcpy (sBuf, "Helmsman active     : FALSE\r\n") ;
  strcat (sBuffer, sBuf) ; 
  if (bYardsmen)
    strcpy (sBuf, "Yardsmen active     : TRUE \r\n") ;
  else
    strcpy (sBuf, "Yardsmen active     : FALSE\r\n") ;
  strcat (sBuffer, sBuf) ; 
  if (bCoupleFore)
    strcpy (sBuf, "Foreyards coupled   : TRUE \r\n") ;
  else
    strcpy (sBuf, "Foreyards coupled   : FALSE\r\n") ;
  strcat (sBuffer, sBuf) ; 
  if (!bBrig)
    {
    if (bCoupleMizzen)
      strcpy (sBuf, "Mizzenyards coupled : TRUE \r\n") ;
    else
      strcpy (sBuf, "Mizzenyards coupled : FALSE\r\n") ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "By-the-wind angle   : %04d\r\n", iCloseHauledAngle) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Tack status         : %04d\r\n", iTackStatus) ;
  strcat (sBuffer, sBuf) ; 
  if (iTackStatus > 0)
    {
    for (iI=0 ; iI<7 ; iI++)
      {
      sprintf (sBuf, "Old staysail status : %02d %05d\r\n", iI, iSailOldSSStatus[iI]) ;
      strcat (sBuffer, sBuf) ; 
      }
    sprintf (sBuf, "Old jib angle       : %04d\r\n", iOldJibAngle) ;
    strcat (sBuffer, sBuf) ; 
    sprintf (sBuf, "Old staysail angle  : %04d\r\n", iOldStaySailAngle) ;
    strcat (sBuffer, sBuf) ; 
    sprintf (sBuf, "Old spanker angle   : %04d\r\n", iOldSpankerAngle) ;
    strcat (sBuffer, sBuf) ; 
    }
  sprintf (sBuf, "Required Heading    : %07.2f\r\n", fRequiredHeading) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Yards to display    : %04d\r\n", iYardLevelShown) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Map width           : %06.3f\r\n", fMapWidth) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Bow factor          : %012.5f\r\n", fBowFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Stern factor        : %012.5f\r\n", fSternFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Side factor         : %012.5f\r\n", fSideFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Water drag factor   : %012.5f\r\n", fWaterDragFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Rudder factor       : %012.5f\r\n", fRudderFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Mass factor         : %012.5f\r\n", fMassFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Hull factor         : %012.5f\r\n", fHullFactor) ;
  strcat (sBuffer, sBuf) ; 
  if (iSpeedFactor != 0)
    fRotatInertFactor = fOldRotatInertFactor ;
  sprintf (sBuf, "Rot. inertia factor : %012.5f\r\n", fRotatInertFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Rot. drag factor    : %012.5f\r\n", fRotatDragFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heel angle factor   : %012.5f\r\n", fHeelAngleFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heel inertia factor : %012.5f\r\n", fHeelInertFactor) ;
  strcat (sBuffer, sBuf) ; 
  sprintf (sBuf, "Heel drag factor    : %012.5f\r\n", fHeelDragFactor) ;
  strcat (sBuffer, sBuf) ; 
  strcpy (sBuf, "Reserved            : 00.000\r\n") ;
  strcat (sBuffer, sBuf) ; 
  strcpy (sBuf, "Reserved            : 00.000\r\n") ;
  strcat (sBuffer, sBuf) ; 
  strcpy (sBuf, "Reserved            : 00.000\r\n") ;
  strcat (sBuffer, sBuf) ; 
  strcpy (sBuf, "Reserved            : 00.000\r\n") ;
  strcat (sBuffer, sBuf) ; 
  strcpy (sBuf, "Reserved            : 00.000\r\n") ;
  strcat (sBuffer, sBuf) ; 
  strcpy(sBuf, "Is IsleLat  IsleLon  IsleRadi\r\n") ;
  strcat (sBuffer, sBuf) ; 
  for (iI=0 ; iI<=12 ; iI++)
    {
    sprintf (sBuf, "%02d %08.4f %08.4f %08.4f\r\n", 
      iI,fIsleLat[iI],fIsleLong[iI],fIsleRadius[iI]) ;
    strcat (sBuffer, sBuf) ; 
    }
  if (iSource==1)
    strcat (sBuffer, "End of file\r\n") ; 
  else
    strcat (sBuffer, "Recording..\r\n") ; 
  dToWrite=strlen(sBuffer) ;
  WriteFile (hFile, sBuffer, dToWrite, &dWritten,  NULL) ;
  if (dToWrite!=dWritten)
    MessageBox (hwnd, "Warning: problem saving file", "Save file", MB_ICONEXCLAMATION | MB_OK) ;
  //sprintf(sMessage, "dWritten %d ", dWritten) ; 
  //MessageBox (hwnd, sMessage, "dWritten", MB_ICONEXCLAMATION | MB_OK) ;

  CloseHandle (hFile) ; 
  }

/* --------------------------------------
   DataDumper
   Version date = 03.06.16
----------------------------------------*/

void DataDumper () 
  {
  char    sBuffer[48100] ;
  char    sBuf [80] ;
  int     iI ;
  DWORD   dWritten, dToWrite ;
  HANDLE  hFile ;

  hFile = CreateFile ("DataDump.txt", GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, 0,0) ;
  sprintf (sBuffer, "Data dump\n") ;
  for (iI=0; iI<15 ; iI++)
    {
    sprintf (sBuf, "SGLeft  %04d %08.5f \n", iI, fSailGradientLeft[iI]) ;
    strcat (sBuffer, sBuf) ; 
    sprintf (sBuf, "SGRight %04d %08.5f \n", iI, fSailGradientRight[iI]) ;
    strcat (sBuffer, sBuf) ; 
    sprintf (sBuf, "SG2     %04d %08.5f \n", iI, fSailGradient2[iI]) ;
    strcat (sBuffer, sBuf) ; 
    sprintf (sBuf, "SILeft  %04d %08.5f \n", iI, fSailInterceptLeft[iI]) ;
    strcat (sBuffer, sBuf) ; 
    sprintf (sBuf, "SIRight %04d %08.5f \n", iI, fSailInterceptRight[iI]) ;
    strcat (sBuffer, sBuf) ; 
    sprintf (sBuf, "SI2     %04d %08.5f \n", iI, fSailIntercept2[iI]) ;
    strcat (sBuffer, sBuf) ; 
    }
  dToWrite=strlen(sBuffer) ;
  sprintf(sMessage, "dToWrite %d ", dToWrite) ; 
  MessageBox (hwnd, sMessage, "dToWrite", MB_ICONEXCLAMATION | MB_OK) ;
  WriteFile (hFile, sBuffer, dToWrite, &dWritten,  NULL) ;
  CloseHandle (hFile) ; 
  strcpy (sMessage,"Variables dumped to") ;
  ShowMessage () ;
  strcpy (sMessage,"DataDump.txt") ;
  ShowMessage () ;
  }


