/*-----------------------------------------
   Dlg.C -- Properties/dimensions change dialogue module
   Version date: 03.09.13
  -----------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <commdlg.h>
#include <stdlib.h>
#include "external.h"
#include "menu.h"

// Prototypes:
void     Focusser () ;
void     ForeAftCancel () ;
void     ForeAftChange () ;
void     ForeAftSetUp () ;
void     IslandCancel () ;
void     IslandChange () ;
void     IslandSetUp () ;
void     PropertiesAccept () ;
void     PropertiesCancel () ;
void     PropertiesChange () ;
void     PropertiesPaint () ;
void     PropertiesSetUp () ;
void     ShipDimsCancel () ;
void     ShipDimsChange () ;
void     ShipDimsSetUp () ;
void     ShipPropertyCancel () ;
void     ShipPropertyChange () ;
void     ShipPropertySetUp () ;
void     SquareSailCancel () ;
void     SquareSailChange () ;
void     SquareSailSetUp () ;

extern   void   CanShadow() ;
extern   void   DerivedSailParameters() ;
extern   void   DerivedShipParameters() ;
extern   void   SetUpDisplay () ;
extern   void   ShowMessage() ;

/* --------------------------------------
   Focusser: sets focus in dialog
   Version date = 99.09.05
----------------------------------------*/

void Focusser ()
  {
  if (iFocus < 0)
    iFocus = iFields[iModifyMenu]-1 ;
  if (iFocus >= iFields[iModifyMenu])
    iFocus = 0 ;
  SetFocus(hwndPropText[iFocus]) ; 
  SendMessage(hwndPropText[iFocus],EM_SETSEL, 0, -1) ;
  }


/* --------------------------------------
   ForeAftCancel
   Version date = 03.09.10
----------------------------------------*/

void ForeAftCancel ()
  {
  int   iI, iJ, i1, i2 ;

  // restore original values
  fSailHeadWidth[iSpanker] = fOldValue[0] ;
  fSailFootWidth[iSpanker] = fOldValue[0] ;
  fSailDepth[iSpanker] = fOldValue[1] ;
  fSpankerPeak = fOldValue[2] ;
  fSailTheorMaxSpeed[iSpanker] = fOldValue[3] ;
  iSailMaxReefs[iSpanker] = fOldValue[4] ;

  if (bBrig)
    {
    i1 = 0 ;
    i2 = iNrSails-(iSpanker+1) ;
    }
  else
    {
    i1 = 1 ;
    i2 = 4 ;
    }

  for (iI = i1 ; iI < i2 ; iI ++)
    {
    if (bBrig)
      iJ = iI ;
    else
      iJ = iI-1 ;
    fSailClewDistance[iI] = fOldValue[5+iJ*7] ;
    fSailClewElevation[iI] = fOldValue[6+iJ*7] ;
    fSailTackDistance[iI] = fOldValue[7+iJ*7] ;
    fSailTackElevation[iI] = fOldValue[8+iJ*7] ;
    fSailHeadDistance[iI] = fOldValue[9+iJ*7] ;
    fSailHeadElevation[iI] = fOldValue[10+iJ*7] ;
    fSailTheorMaxSpeed[iI+iSpanker+1] = fOldValue[11+iJ*7];
    }
  }

/* --------------------------------------
   ForeAftChange: a value may have been changed
   Version date = 03.09.10
----------------------------------------*/

void ForeAftChange ()
  {
  int   iI, iJ, i1, i2 ;

  GetWindowText (hwndPropText[0], sMessage, 12);
  fSailHeadWidth[iSpanker] = atof(sMessage) ;
  fSailFootWidth[iSpanker] = atof(sMessage) ;
  GetWindowText (hwndPropText[1], sMessage, 12);
  fSailDepth[iSpanker] = atof(sMessage) ;
  GetWindowText (hwndPropText[2], sMessage, 12);
  fSpankerPeak = (atof(sMessage)) - fSailDepth[iSpanker] ;
  GetWindowText (hwndPropText[3], sMessage, 12);
  fSailTheorMaxSpeed[iSpanker] = atof(sMessage) ;
  GetWindowText (hwndPropText[4], sMessage, 12);
  iSailMaxReefs[iSpanker] = atoi(sMessage) ;

  if (bBrig)
    {
    i1 = 0 ;
    i2 = iNrSails-(iSpanker+1) ;
    }
  else
    {
    i1 = 1 ;
    i2 = 4 ;
    }

  for (iI = i1 ; iI < i2 ; iI ++)
    {
    if (bBrig)
      iJ = iI ;
    else
      iJ = iI-1 ;
    GetWindowText (hwndPropText[5+iJ*7], sMessage, 12);
    fSailClewDistance[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[6+iJ*7], sMessage, 12);
    fSailClewElevation[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[7+iJ*7], sMessage, 12);
    fSailTackDistance[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[8+iJ*7], sMessage, 12);
    fSailTackElevation[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[9+iJ*7], sMessage, 12);
    fSailHeadDistance[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[10+iJ*7], sMessage, 12);
    fSailHeadElevation[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[11+iJ*7], sMessage, 12);
    fSailTheorMaxSpeed[iI+iSpanker+1] = atof(sMessage) ;
    }

  if (!bBrig) 
    {
    fSailClewDistance[0]= fSailClewDistance[iJibSet+1] ;
    fSailClewElevation[0]= fSailClewElevation[iJibSet+1] ;
    fSailTackDistance[0]= fSailTackDistance[iJibSet+1] ;
    fSailTackElevation[0]= fSailTackElevation[iJibSet+1] ;
    fSailHeadDistance[0]= fSailHeadDistance[iJibSet+1] ;
    fSailHeadElevation[0]= fSailHeadElevation[iJibSet+1] ;
    // Recalculate effective head and mean heights of sail
    fSailEffectiveHead[12] = fSailHeadElevation[0] ;
    fSailFoot[12] = fSailTackElevation[0] ;
    fSailMeanHeight[12] = (fSailTackElevation[0]+fSailClewElevation[0]+fSailHeadElevation[0])/3.0;
    fSailSSCentroid[12] = (fSailTackDistance[0]+fSailClewDistance[0]+fSailHeadDistance[0])/3.0;
    xSailPlanHeadLeft[12] = -(fSailTackDistance[0]-fSailHeadDistance[0]) ;
    xSailPlanHeadRight[12] = -(fSailTackDistance[0]-fSailHeadDistance[0]) ;
    xSailPlanFootLeft[12] = -(fSailTackDistance[0]-fSailClewDistance[0]) ;
    xSailPlanFootRight[12] = 0 ;
    ySailPlanHeadLeft[12] = fSailHeadElevation[0] ;
    ySailPlanHeadRight[12] = fSailHeadElevation[0] ;
    ySailPlanFootLeft[12] = fSailClewElevation[0] ;
    ySailPlanFootRight[12] = fSailTackElevation[0] ;
    fSailCorner[12] = fSailClewElevation[0] ;
    }
  }

/* --------------------------------------
   ForeAftSetUp
   Version date = 03.09.10
----------------------------------------*/

void ForeAftSetUp ()
  {
  int  iI, i1, i2, iK ;
  int  yPosn ;

  MoveWindow(hwndProps, 5, 400, 550, 310, TRUE) ;
  MoveWindow(hwndPropOKButton, 480,225,50,20, TRUE) ;
  MoveWindow(hwndPropCancelButton, 480,250,50,20, TRUE) ;

  yPosn = 41;
  iK = 0 ;
  fOldValue[iK]=fSailHeadWidth[iSpanker] ;
  sprintf(sMessage, "%1.1f", fSailHeadWidth[iSpanker]) ;
  MoveWindow(hwndPropText[iK],130,yPosn,40,18,TRUE) ;
  SetWindowText (hwndPropText[iK],sMessage) ;
  iK = iK + 1 ;
  fOldValue[iK]=fSailDepth[iSpanker] ;
  sprintf(sMessage, "%1.1f", fSailDepth[iSpanker]) ;
  MoveWindow(hwndPropText[iK],180,yPosn,40,18,TRUE) ;
  SetWindowText (hwndPropText[iK],sMessage) ;
  iK = iK + 1 ;
  fOldValue[iK]=fSpankerPeak ;
  sprintf(sMessage, "%1.1f", fSpankerPeak+fSailDepth[iSpanker]) ;
  MoveWindow(hwndPropText[iK],230,yPosn,40,18,TRUE) ;
  SetWindowText (hwndPropText[iK],sMessage) ;
  iK = iK + 1 ;
  fOldValue[iK]=fSailTheorMaxSpeed[iSpanker] ;
  sprintf(sMessage, "%1.1f", fSailTheorMaxSpeed[iSpanker]) ;
  MoveWindow(hwndPropText[iK],280,yPosn,40,18,TRUE) ;
  SetWindowText (hwndPropText[iK],sMessage) ;
  iK = iK + 1 ;
  fOldValue[iK]=iSailMaxReefs[iSpanker] ;
  sprintf(sMessage, "%1d", iSailMaxReefs[iSpanker]) ;
  MoveWindow(hwndPropText[iK],330,yPosn,40,18,TRUE) ;
  SetWindowText (hwndPropText[iK],sMessage) ;

  if (bBrig)
    {
    i1 = 0 ;
    i2 = iNrSails-(iSpanker+1) ;
    }
  else
    {
    i1 = 1 ;
    i2 = 4 ;
    }

  for (iI = i1 ; iI < i2 ; iI ++)
    {
    yPosn = 109 + 18*iI ;
    iK = iK + 1 ;
    fOldValue[iK]=fSailClewDistance[iI] ;
    sprintf(sMessage, "%1.1f", fSailClewDistance[iI]) ;
    MoveWindow(hwndPropText[iK],130,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[iK],sMessage) ;
    iK = iK + 1 ;
    fOldValue[iK]=fSailClewElevation[iI] ;
    sprintf(sMessage, "%1.1f", fSailClewElevation[iI]) ;
    MoveWindow(hwndPropText[iK],180,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[iK],sMessage) ; 
    iK = iK + 1 ;
    fOldValue[iK]=fSailTackDistance[iI] ;
    sprintf(sMessage, "%1.1f", fSailTackDistance[iI]) ;
    MoveWindow(hwndPropText[iK],230,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[iK],sMessage) ; 
    iK = iK + 1 ;
    fOldValue[iK]=fSailTackElevation[iI] ;
    sprintf(sMessage, "%1.1f", fSailTackElevation[iI]) ;
    MoveWindow(hwndPropText[iK],280,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[iK],sMessage) ; 
    iK = iK + 1 ;
    fOldValue[iK]=fSailHeadDistance[iI] ;
    sprintf(sMessage, "%1.1f", fSailHeadDistance[iI]) ;
    MoveWindow(hwndPropText[iK],330,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[iK],sMessage) ; 
    iK = iK + 1 ;
    fOldValue[iK]=fSailHeadElevation[iI] ;
    sprintf(sMessage, "%1.1f", fSailHeadElevation[iI]) ;
    MoveWindow(hwndPropText[iK],380,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[iK],sMessage) ; 
    iK = iK + 1 ;
    fOldValue[iK]=fSailTheorMaxSpeed[iI+iSpanker+1] ;
    sprintf(sMessage, "%1.1f",fSailTheorMaxSpeed[iI+iSpanker+1]) ;
    MoveWindow(hwndPropText[iK],430,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[iK],sMessage) ; 
    }
  }

/* --------------------------------------
   IslandCancel
   Version date = 99.09.22
----------------------------------------*/

void IslandCancel ()
  {
  int   iI ;

  // restore original values
  for (iI = 0 ; iI <= 12 ; iI ++)
    {
    fIsleLat[iI] = fOldValue[iI*3] ;
    fIsleLong[iI] = fOldValue[1+iI*3] ;
    fIsleRadius[iI] = fOldValue[2+iI*3];
    }
  }

/* --------------------------------------
   IslandChange: a value may have been changed
   Version date = 99.10.17
----------------------------------------*/

void IslandChange ()
  {
  int   iI ;

  for (iI = 0 ; iI <= 12 ; iI ++)
    {
    GetWindowText (hwndPropText[iI*3], sMessage, 12);
    fIsleLat[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[1+iI*3], sMessage, 12);
    fIsleLong[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[2+iI*3], sMessage, 12);
    fIsleRadius[iI] = atof(sMessage) ;
    }
  bMapMustBePainted = TRUE ;
  }

/* --------------------------------------
   IslandSetUp
   Version date = 99.10.17
----------------------------------------*/

void IslandSetUp ()
  {
  int   iI ;
  int   yPosn ;

  bMapMustBePainted = TRUE ;

  MoveWindow(hwndProps, 5, 255, 318, 295, TRUE) ;
  MoveWindow(hwndPropOKButton, 252,211,50,20, TRUE) ;
  MoveWindow(hwndPropCancelButton, 252,236,50,20, TRUE) ;

  for (iI = 0 ; iI <= 12 ; iI ++)
    {
    yPosn = 23 + 18*iI ;
    fOldValue[iI*3]=fIsleLat[iI] ;
    sprintf(sMessage, "%1.4f", fIsleLat[iI]) ;
    MoveWindow(hwndPropText[iI*3],70,yPosn,50,18,TRUE) ;
    SetWindowText (hwndPropText[iI*3],sMessage) ; 
    fOldValue[1+iI*3]=fIsleLong[iI] ;
    sprintf(sMessage, "%1.4f", fIsleLong[iI]) ;
    MoveWindow(hwndPropText[1+iI*3],130,yPosn,50,18,TRUE) ;
    SetWindowText (hwndPropText[1+iI*3],sMessage) ; 
    fOldValue[2+iI*3]=fIsleRadius[iI] ;
    sprintf(sMessage, "%1.4f",fIsleRadius[iI]) ;
    MoveWindow(hwndPropText[2+iI*3],190,yPosn,50,18,TRUE) ;
    SetWindowText (hwndPropText[2+iI*3],sMessage) ; 
    }
  }

/* --------------------------------------
   PropertiesAccept
   Version date = 03.09.13
----------------------------------------*/

void PropertiesAccept ()
  {
  HDC     hdc ;
  int     iI ;

  DestroyWindow(hwndProps) ;
  DestroyWindow (hwndPropOKButton) ;
  DestroyWindow (hwndPropCancelButton) ;
  for (iI = 0 ; iI < iFields[iModifyMenu] ; iI++)
    DestroyWindow(hwndPropText[iI]) ;
  UpdateWindow (hwnd) ;
  hdc = GetDC (hwnd) ;
  InvalidateRect (hwnd, NULL, TRUE) ;
  ReleaseDC (hwnd, hdc) ;
  SetUpDisplay () ;
  EnableMenuItem(hMenu, IDM_SHIPDIMENSIONS, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_SHIPPROPERTIES, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_SAILDIMENSIONS, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_FORENAFT, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_ISLANDS, MF_ENABLED) ;
  iModifyMenu = 0 ;
  strcpy (sMessage, "Changes implemented") ;
  ShowMessage () ;
  }

/* --------------------------------------
   PropertiesCancel
   Version date = 03.09.13
----------------------------------------*/

void PropertiesCancel ()
  {
  HDC     hdc ;
  int     iI ;

  DestroyWindow(hwndProps) ;
  DestroyWindow (hwndPropOKButton) ;
  DestroyWindow (hwndPropCancelButton) ;
  for (iI = 0 ; iI < iFields[iModifyMenu] ; iI++)
    DestroyWindow(hwndPropText[iI]) ;
  UpdateWindow (hwnd) ;
  hdc = GetDC (hwnd) ;
  InvalidateRect (hwnd, NULL, TRUE) ;
  ReleaseDC (hwnd, hdc) ;
  SetUpDisplay () ;
  EnableMenuItem(hMenu, IDM_SHIPDIMENSIONS, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_SHIPPROPERTIES, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_SAILDIMENSIONS, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_FORENAFT, MF_ENABLED) ;
  EnableMenuItem(hMenu, IDM_ISLANDS, MF_ENABLED) ;

  switch (iModifyMenu)
    {
    case 0 :
      ShipDimsCancel();
      break ;
    case 1 :
      ShipPropertyCancel() ;
      break ;
    case 2 :
      SquareSailCancel() ;
      break ;
    case 3 :
      ForeAftCancel() ;
      break ;
    case 4 :
      IslandCancel() ;
    }
  iModifyMenu = 0 ;
  bPlanViewIsChanged = TRUE ;
  bSailPlanIsChanged = TRUE ;
  DerivedShipParameters () ;
  DerivedSailParameters () ;
  CanShadow() ;
  strcpy (sMessage, "Changes not implemented") ;
  ShowMessage () ;
  }

/* --------------------------------------
   PropertiesChange
   Version date = 03.01.20
----------------------------------------*/

void PropertiesChange ()
  {
  switch (iModifyMenu)
    {
    case 0 :
      ShipDimsChange();
      break ;
    case 1 :
      ShipPropertyChange() ;
      break ;
    case 2 :
      SquareSailChange() ;
      break ;
    case 3 :
      ForeAftChange() ;
      break ;
    case 4 :
      IslandChange() ;
    }
  bPlanViewIsChanged = TRUE ;
  bSailPlanIsChanged = TRUE ;
  DerivedShipParameters () ;
  DerivedSailParameters () ;
  CanShadow () ;
  }

/* --------------------------------------
   PropertiesPaint
   Version date = 03.09.10
----------------------------------------*/

void PropertiesPaint ()
  {
  HDC   hdc ;
  int   iI, i1, i2 ;
  int   xPosn, yPosn ;

  SendMessage(hwndProps, WM_SETFONT, (WPARAM) MyFont, 0) ;
  hdc = GetDC (hwndProps) ;
  SelectObject (hdc, MyFont) ;
  SelectObject (hdc, GetStockObject (LTGRAY_BRUSH)) ;
  SetBkColor (hdc, RGB (192, 192, 192)) ;

  switch (iModifyMenu)
    {
    case 0 :
      SetWindowText(hwndProps,"Modify ship dimensions") ;
      xPosn = 50 ;
      yPosn = 7 ;
      TextOut (hdc, xPosn, yPosn, "Ship length",11) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Quarterdeck length",18) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Forecastle length",17) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Main deck height",15) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Quarterdeck height",18) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Centre of rotation dist.",24) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Hull centroid distance",22) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Ship width",10) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Ship draught",12) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Ship tonnage",12) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Foremast distance",17) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Mainmast distance",17) ; 
      yPosn = yPosn + 18 ;
      if (bBrig)
        SetTextColor(hdc, RGB(160,160,160)) ;  // gray
      TextOut (hdc, xPosn, yPosn, "Mizzenmast distance",19) ; 
      SetTextColor(hdc, RGB(0,0,0)) ;          // black
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Forecourse foot height",22) ; 
      xPosn = 210 ;
      yPosn = 7 ;
      TextOut (hdc, xPosn, yPosn, "Maincourse foot height",22) ; 
      yPosn = yPosn + 18 ;
      if (bBrig)
        SetTextColor(hdc, RGB(160,160,160)) ;  // gray
      TextOut (hdc, xPosn, yPosn, "Mizzencourse foot height",24) ; 
      SetTextColor(hdc, RGB(0,0,0)) ;          // black
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Spanker foot height",19) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Set fraction (1 reef)",21) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Set fraction (2 reefs)",22) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Set fraction (3 reefs)",22) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Max foreyard angle",18) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Max mainyard angle",18) ; 
      yPosn = yPosn + 18 ;
      if (bBrig)
        SetTextColor(hdc, RGB(160,160,160)) ;  // gray
      TextOut (hdc, xPosn, yPosn, "Max mizzenyard angle",20) ; 
      SetTextColor(hdc, RGB(0,0,0)) ;          // black
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Max spanker angle",17) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Max staysail/jib angle",22) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Rudder area",11) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, xPosn, yPosn, "Max helm angle",14) ; 
      break ;
    case 1 :
      SetWindowText(hwndProps,"Modify ship dynamic properties") ;
      xPosn = 100 ;
      yPosn = 7 ;
      TextOut (hdc, xPosn, yPosn, "Water drag factor",17) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Rudder factor",13) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Bow factor",10) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Stern factor",12) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Side factor",11) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Mass Factor",11) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Hull Factor",11) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Rotational inertia factor",25) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Rotational drag factor",22) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Heel angle factor",17) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Heeling inertia factor",22) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "Heeling drag factor",19) ; 
      yPosn = yPosn+18 ;
      TextOut (hdc, xPosn, yPosn, "By-the-wind angle",17) ; 
      break ;
    case 2 :
      SetWindowText(hwndProps,"Modify square sails") ;
      SetTextAlign(hdc, TA_RIGHT|TA_TOP) ;
      for (iI = 0 ; iI <iSpanker ; iI++)
        TextOut (hdc, 95, 25 + iI*18, sSailName[iI],strlen(sSailName[iI])) ; 
      TextOut (hdc, 140, 5, "Head wid.",9) ; 
      TextOut (hdc, 190, 5, "Foot wid.",9) ; 
      TextOut (hdc, 240, 5, "Depth",5) ; 
      TextOut (hdc, 290, 5, "Speed",5) ; 
      TextOut (hdc, 330, 5, "Reefs",5) ; 
      SetTextAlign(hdc, TA_LEFT|TA_TOP) ;
      break ;
    case 3 :
      SetWindowText(hwndProps,"Modify fore-and-aft sails") ;
      SetTextAlign(hdc, TA_RIGHT|TA_TOP) ;
      yPosn = 5 ;
      TextOut (hdc, 170, yPosn, "Width",5) ; 
      TextOut (hdc, 220, yPosn, "Luff",4) ; 
      TextOut (hdc, 270, yPosn, "Leech",5) ; 
      TextOut (hdc, 320, yPosn, "Speed",5) ; 
      TextOut (hdc, 370, yPosn, "Reefs",5) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, 220, yPosn, "depth",5) ; 
      TextOut (hdc, 270, yPosn, "depth",5) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, 125, yPosn, sSailName[iSpanker],strlen(sSailName[iSpanker])) ; 
      yPosn= 77 ;
      TextOut (hdc, 170, yPosn, "Clew",4) ; 
      TextOut (hdc, 220, yPosn, "Clew",4) ; 
      TextOut (hdc, 270, yPosn, "Tack",4) ; 
      TextOut (hdc, 320, yPosn, "Tack",4) ; 
      TextOut (hdc, 370, yPosn, "Head",4) ; 
      TextOut (hdc, 420, yPosn, "Head",4) ; 
      TextOut (hdc, 470, yPosn, "Speed",5) ; 
      yPosn = yPosn + 18 ;
      TextOut (hdc, 170, yPosn, "dist.",5) ; 
      TextOut (hdc, 220, yPosn, "elev.",5) ; 
      TextOut (hdc, 270, yPosn, "dist.",5) ; 
      TextOut (hdc, 320, yPosn, "elev.",5) ; 
      TextOut (hdc, 370, yPosn, "dist.",5) ; 
      TextOut (hdc, 420, yPosn, "elev.",5) ; 
      yPosn = yPosn + 18 ;
      if (bBrig)
        {
        i1 = 0 ;
        i2 = iNrSails-(iSpanker+1) ;
        }
      else
        {
        i1 = 1 ;
        i2 = 4 ;
        }
      for (iI = i1 ; iI < i2 ; iI ++)
        TextOut (hdc, 125, yPosn + iI*18, sSailName[iI+iSpanker+1],strlen(sSailName[iI+iSpanker+1])) ; 
      SetTextAlign(hdc, TA_LEFT|TA_TOP) ;
      break ;
    case 4 :
      SetWindowText(hwndProps,"Modify islands") ;
      for (iI = 0 ; iI <=12 ; iI++)
        TextOut (hdc, 5, 25 + iI*18, sMessage,sprintf(sMessage, "Island nr %1d", iI+1)) ; 
      SetTextAlign(hdc, TA_RIGHT|TA_TOP) ;
      TextOut (hdc, 120, 5, "Lat. N",6) ; 
      TextOut (hdc, 180, 5, "Long. E",7) ; 
      TextOut (hdc, 240, 5, "Radius",6) ; 
      SetTextAlign(hdc, TA_LEFT|TA_TOP) ;
    }
  ReleaseDC (hwndProps,hdc) ;
  }

/* --------------------------------------
   PropertiesSetUp
   Version date = 99.09.22
----------------------------------------*/

void PropertiesSetUp ()
  {
  switch (iModifyMenu)
    {
    case 0 :
      ShipDimsSetUp();
      break ;
    case 1 :
      ShipPropertySetUp() ;
      break ;
    case 2 :
      SquareSailSetUp() ;
      break ;
    case 3 :
      ForeAftSetUp() ;
      break ;
    case 4 :
      IslandSetUp() ;
    }
  }

/* --------------------------------------
   ShipDimsCancel
   Version date = 03.08.24
----------------------------------------*/

void ShipDimsCancel ()
  {
  // restore original values
  fShipLength=fOldValue[0] ;
  fQDeckLength=fOldValue[1] ;
  fFocleLength=fOldValue[2] ;
  fGunDeckHeight=fOldValue[3] ;
  fQDeckHeight=fOldValue[4] ;
  fCORDistance=fOldValue[5] ;
  fHullCentroid=fOldValue[6] ;
  fShipWidth=fOldValue[7] ;
  fShipDraught=fOldValue[8] ;
  fShipTons=fOldValue[9] ;
  fMastDistance[0]=fOldValue[10] ;
  fMastDistance[1]=fOldValue[11] ;
  fMastDistance[2]=fOldValue[12] ;
  fLowestSailFoot[0]=fOldValue[13] ;
  fLowestSailFoot[1]=fOldValue[14] ;
  fLowestSailFoot[2]=fOldValue[15] ;
  fLowestSailFoot[3]=fOldValue[16] ;
  fSetFraction[1]=fOldValue[17] ;
  fSetFraction[2]=fOldValue[18] ;
  fSetFraction[3]=fOldValue[19] ;
  iMaxYardAngle[0]=fOldValue[20] ;
  iMaxYardAngle[1]=fOldValue[21] ;
  iMaxYardAngle[2]=fOldValue[22] ;
  iMaxYardAngle[3]=fOldValue[23] ;
  iMaxJibAngle=fOldValue[24] ;
  fRudderArea=fOldValue[25] ;
  iMaxHelmAngle=fOldValue[26] ;
  }

/* --------------------------------------
   ShipDimsChange: a parameter may have been changed
   Version date = 03.08.24
----------------------------------------*/

void ShipDimsChange ()
  {
  GetWindowText (hwndPropText[0], sMessage, 6);
  fShipLength = atof(sMessage) ;
  GetWindowText (hwndPropText[1], sMessage, 6);
  fQDeckLength = atof(sMessage) ;
  GetWindowText (hwndPropText[2], sMessage, 6);
  fFocleLength = atof(sMessage) ;
  GetWindowText (hwndPropText[3], sMessage, 6);
  fGunDeckHeight = atof(sMessage) ;
  GetWindowText (hwndPropText[4], sMessage, 6);
  fQDeckHeight = atof(sMessage) ;
  GetWindowText (hwndPropText[5], sMessage, 6);
  fCORDistance = atof(sMessage) ;
  GetWindowText (hwndPropText[6], sMessage, 6);
  fHullCentroid = atof(sMessage) ;
  GetWindowText (hwndPropText[7], sMessage, 6);
  fShipWidth = atof(sMessage) ;
  GetWindowText (hwndPropText[8], sMessage, 6);
  fShipDraught = atof(sMessage) ;
  GetWindowText (hwndPropText[9], sMessage, 6);
  fShipTons = atof(sMessage) ;
  GetWindowText (hwndPropText[10], sMessage, 6);
  fMastDistance[0] = atof(sMessage) ;
  GetWindowText (hwndPropText[11], sMessage, 6);
  fMastDistance[1] = atof(sMessage) ;
  GetWindowText (hwndPropText[12], sMessage, 6);
  fMastDistance[2] = atof(sMessage) ;
  GetWindowText (hwndPropText[13], sMessage, 6);
  fLowestSailFoot[0] = atof(sMessage) ;
  GetWindowText (hwndPropText[14], sMessage, 6);
  fLowestSailFoot[1] = atof(sMessage) ;
  GetWindowText (hwndPropText[15], sMessage, 6);
  fLowestSailFoot[2] = atof(sMessage) ;
  GetWindowText (hwndPropText[16], sMessage, 6);
  fLowestSailFoot[3] = atof(sMessage) ;
  GetWindowText (hwndPropText[17], sMessage, 6);
  fSetFraction[1] = atof(sMessage) ;
  GetWindowText (hwndPropText[18], sMessage, 6);
  fSetFraction[2] = atof(sMessage) ;
  GetWindowText (hwndPropText[19], sMessage, 6);
  fSetFraction[3] = atof(sMessage) ;
  GetWindowText (hwndPropText[20], sMessage, 6);
  iMaxYardAngle[0] = atoi(sMessage) ;
  GetWindowText (hwndPropText[21], sMessage, 6);
  iMaxYardAngle[1] = atoi(sMessage) ;
  GetWindowText (hwndPropText[22], sMessage, 6);
  iMaxYardAngle[2] = atoi(sMessage) ;
  GetWindowText (hwndPropText[23], sMessage, 6);
  iMaxYardAngle[3] = atoi(sMessage) ;
  GetWindowText (hwndPropText[24], sMessage, 6);
  iMaxJibAngle = atoi(sMessage) ;
  GetWindowText (hwndPropText[25], sMessage, 6);
  fRudderArea = atof(sMessage) ;
  GetWindowText (hwndPropText[26], sMessage, 6);
  iMaxHelmAngle = atof(sMessage) ;
  }

/* --------------------------------------
   ShipDimsSetUp
   Version date = 03.08.24
----------------------------------------*/

void ShipDimsSetUp ()
  {
  int  iI ;
  int  xPosn, yPosn ;

  MoveWindow(hwndProps, 5, 280, 360, 295, TRUE) ;
  MoveWindow(hwndPropOKButton, 295,214,50,20, TRUE) ;
  MoveWindow(hwndPropCancelButton, 295,239,50,20, TRUE) ;

  for (iI = 0 ; iI < iFields[iModifyMenu] ; iI ++)
    {
    if (iI<14)
      {
      xPosn = 50 ;
      yPosn = 7 +18*iI ;
      }
    else
      {
      xPosn = 210 ;
      yPosn = 7 +18*(iI-14) ;
      }
    switch (iI)
      {
      case 0 :
        fOldValue[iI]=fShipLength ;
        sprintf(sMessage, "%1.1f", fShipLength) ;
        break ;
      case 1 :
        fOldValue[iI]=fQDeckLength ;
        sprintf(sMessage, "%1.1f", fQDeckLength) ;
        break ;
      case 2 :
        fOldValue[iI]=fFocleLength ;
        sprintf(sMessage, "%1.1f", fFocleLength) ;
        break ;
      case 3 :
        fOldValue[iI]=fGunDeckHeight ;
        sprintf(sMessage, "%1.1f", fGunDeckHeight) ;
        break ;
      case 4 :
        fOldValue[iI]=fQDeckHeight ;
        sprintf(sMessage, "%1.1f", fQDeckHeight) ;
        break ;
      case 5 :
        fOldValue[iI]=fCORDistance ;
        sprintf(sMessage, "%1.1f", fCORDistance) ;
        break ;
      case 6 :
        fOldValue[iI]=fHullCentroid ;
        sprintf(sMessage, "%1.1f", fHullCentroid) ;
        break ;
      case 7 :
        fOldValue[iI]=fShipWidth ;
        sprintf(sMessage, "%1.1f", fShipWidth) ;
        break ;
      case 8 :
        fOldValue[iI]=fShipDraught ;
        sprintf(sMessage, "%1.1f", fShipDraught) ;
        break ;
      case 9 :
        fOldValue[iI]=fShipTons ;
        sprintf(sMessage, "%1.1f", fShipTons) ;
        break ;
      case 10 :
        fOldValue[iI]=fMastDistance[0] ;
        sprintf(sMessage, "%1.1f", fMastDistance[0]) ;
        break ;
      case 11 :
        fOldValue[iI]=fMastDistance[1] ;
        sprintf(sMessage, "%1.1f", fMastDistance[1]) ;
        break ;
      case 12 :
        fOldValue[iI]=fMastDistance[2] ;
        sprintf(sMessage, "%1.1f", fMastDistance[2]) ;
        break ;
      case 13 :
        fOldValue[iI]=fLowestSailFoot[0] ;
        sprintf(sMessage, "%1.1f", fLowestSailFoot[0]) ;
        break ;
      case 14 :
        fOldValue[iI]=fLowestSailFoot[1] ;
        sprintf(sMessage, "%1.1f", fLowestSailFoot[1]) ;
        break ;
      case 15 :
        fOldValue[iI]=fLowestSailFoot[2] ;
        sprintf(sMessage, "%1.1f", fLowestSailFoot[2]) ;
        break ;
      case 16 :
        fOldValue[iI]=fLowestSailFoot[3] ;
        sprintf(sMessage, "%1.1f", fLowestSailFoot[3]) ;
        break ;
      case 17 :
        fOldValue[iI]=fSetFraction[1] ;
        sprintf(sMessage, "%1.2f", fSetFraction[1]) ;
        break ;
      case 18 :
        fOldValue[iI]=fSetFraction[2] ;
        sprintf(sMessage, "%1.2f", fSetFraction[2]) ;
        break ;
      case 19 :
        fOldValue[iI]=fSetFraction[3] ;
        sprintf(sMessage, "%1.2f", fSetFraction[3]) ;
        break ;
      case 20 :
        fOldValue[iI]=iMaxYardAngle[0] ;
        sprintf(sMessage, "%d", iMaxYardAngle[0]) ;
        break ;
      case 21 :
        fOldValue[iI]=iMaxYardAngle[1] ;
        sprintf(sMessage, "%1d", iMaxYardAngle[1]) ;
        break ;
      case 22 :
        fOldValue[iI]=iMaxYardAngle[2] ;
        sprintf(sMessage, "%1d", iMaxYardAngle[2]) ;
        break ;
      case 23 :
        fOldValue[iI]=iMaxYardAngle[3] ;
        sprintf(sMessage, "%1d", iMaxYardAngle[3]) ;
        break ;
      case 24:
        fOldValue[iI]=iMaxJibAngle ;
        sprintf(sMessage, "%1d", iMaxJibAngle) ;
        break ;
      case 25 :
        fOldValue[iI]=fRudderArea ;
        sprintf(sMessage, "%1.1f", fRudderArea) ;
        break ;
      case 26 :
        fOldValue[iI]=iMaxHelmAngle ;
        sprintf(sMessage, "%1d", iMaxHelmAngle) ;
      }
    MoveWindow(hwndPropText[iI],xPosn-45,yPosn-3,40,18,TRUE) ;
    SetWindowText (hwndPropText[iI],sMessage) ; 
    }
  if (bBrig)
    {
    EnableWindow (hwndPropText[12], FALSE) ;
    EnableWindow (hwndPropText[15], FALSE) ;
    EnableWindow (hwndPropText[22], FALSE) ;
    }
  }

/* --------------------------------------
   ShipPropertyCancel
   Version date = 03.04.09
----------------------------------------*/

void ShipPropertyCancel ()
  {
  // restore original values
  fWaterDragFactor=fOldValue[0] ;
  fRudderFactor=fOldValue[1] ;
  fBowFactor=fOldValue[2] ;
  fSternFactor=fOldValue[3] ;
  fSideFactor=fOldValue[4] ;
  fMassFactor=fOldValue[5] ;
  fHullFactor=fOldValue[6] ;
  fRotatInertFactor=fOldValue[7] ;
  fRotatDragFactor=fOldValue[8] ;
  fHeelAngleFactor=fOldValue[9] ;
  fHeelInertFactor=fOldValue[10] ;
  fHeelDragFactor=fOldValue[11] ;
  iCloseHauledAngle=fOldValue[12] ;
  }

/* --------------------------------------
   ShipPropertyChange: a value may have been changed
   Version date = 03.04.09
----------------------------------------*/

void ShipPropertyChange ()
  {
  GetWindowText (hwndPropText[0], sMessage, 12);
  fWaterDragFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[1], sMessage, 12);
  fRudderFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[2], sMessage, 12);
  fBowFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[3], sMessage, 12);
  fSternFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[4], sMessage, 12);
  fSideFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[5], sMessage, 12);
  fMassFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[6], sMessage, 12);
  fHullFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[7], sMessage, 12);
  fRotatInertFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[8], sMessage, 12);
  fRotatDragFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[9], sMessage, 12);
  fHeelAngleFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[10], sMessage, 12);
  fHeelInertFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[11], sMessage, 12);
  fHeelDragFactor = atof(sMessage) ;
  GetWindowText (hwndPropText[12], sMessage, 12);
  iCloseHauledAngle = atoi(sMessage) ;
  }

/* --------------------------------------
   ShipPropertySetUp
   Version date = 03.04.09
----------------------------------------*/

void ShipPropertySetUp ()
  {
  int   iI ;
  int   xPosn, yPosn ;

  MoveWindow(hwndProps, 5, 230, 225, 318, TRUE) ;
  MoveWindow(hwndPropOKButton, 160,244,50,20, TRUE) ;
  MoveWindow(hwndPropCancelButton, 160,268,50,20, TRUE) ;

  xPosn = 100 ;
  for (iI = 0 ; iI <= 12 ; iI ++)
    {
    yPosn = 7 +18*iI ;
    switch (iI)
      {
      case 0 :
        fOldValue[iI]=fWaterDragFactor ;
        sprintf(sMessage, "%1.0f", fWaterDragFactor) ;
        break ;
      case 1 :
        fOldValue[iI]=fRudderFactor ;
        sprintf(sMessage, "%1.0f", fRudderFactor) ;
        break ;
      case 2 :
        fOldValue[iI]=fBowFactor ;
        sprintf(sMessage, "%1.5f", fBowFactor) ;
        break ;
      case 3 :
        fOldValue[iI]=fSternFactor ;
        sprintf(sMessage, "%1.5f", fSternFactor) ;
        break ;
      case 4 :
        fOldValue[iI]=fSideFactor ;
        sprintf(sMessage, "%1.5f", fSideFactor) ;
        break ;
      case 5 :
        fOldValue[iI]=fMassFactor ;
        sprintf(sMessage, "%1.5f", fMassFactor) ;
        break ;
      case 6 :
        fOldValue[iI]=fHullFactor ;
        sprintf(sMessage, "%1.5f", fHullFactor) ;
        break ;
      case 7 :
        fOldValue[iI]=fRotatInertFactor ;
        sprintf(sMessage, "%1.5f", fRotatInertFactor) ;
        break ;
      case 8 :
        fOldValue[iI]=fRotatDragFactor ;
        sprintf(sMessage, "%1.5f", fRotatDragFactor) ;
        break ;
      case 9 :
        fOldValue[iI]=fHeelAngleFactor ;
        sprintf(sMessage, "%1.5f", fHeelAngleFactor) ;
        break ;
      case 10 :
        fOldValue[iI]=fHeelInertFactor ;
        sprintf(sMessage, "%1.5f", fHeelInertFactor) ;
        break ;
      case 11 :
        fOldValue[iI]=fHeelDragFactor ;
        sprintf(sMessage, "%1.5f", fHeelDragFactor) ;
        break ;
      case 12 :
        fOldValue[iI]=iCloseHauledAngle ;
        sprintf(sMessage, "%d", iCloseHauledAngle) ;
        break ;
      }
    MoveWindow(hwndPropText[iI],xPosn-95,yPosn-3,80,18,TRUE) ;
    SetWindowText (hwndPropText[iI],sMessage); 
    }
  }

/* --------------------------------------
   SquareSailCancel
   Version date = 03.03.02
----------------------------------------*/

void SquareSailCancel ()
  {
  int   iI ;

  // restore original values
  for (iI = 0 ; iI < iSpanker ; iI ++)
    {
    fSailHeadWidth[iI] = fOldValue[iI*5] ;
    fSailFootWidth[iI] = fOldValue[1+iI*5] ;
    fSailDepth[iI] = fOldValue[2+iI*5];
    fSailTheorMaxSpeed[iI] = fOldValue[3+iI*5];
    iSailMaxReefs[iI] = fOldValue[4+iI*5];
    }
  }

/* --------------------------------------
   SquareSailChange: a value may have been changed
   Version date = 03.03.02
----------------------------------------*/

void SquareSailChange ()
  {
  int   iI ;

  for (iI = 0 ; iI < iSpanker ; iI ++)
    {
    GetWindowText (hwndPropText[iI*5], sMessage, 12);
    fSailHeadWidth[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[1+iI*5], sMessage, 12);
    fSailFootWidth[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[2+iI*5], sMessage, 12);
    fSailDepth[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[3+iI*5], sMessage, 12);
    fSailTheorMaxSpeed[iI] = atof(sMessage) ;
    GetWindowText (hwndPropText[4+iI*5], sMessage, 12);
    iSailMaxReefs[iI] = atoi(sMessage) ;
    }
  }

/* --------------------------------------
   SquareSailSetUp
   Version date = 03.08.24
----------------------------------------*/

void SquareSailSetUp ()
  {
  int  iI ;
  int  yPosn ;

  MoveWindow(hwndProps, 5, 280, 405, 260-(bBrig*18), TRUE) ;
  MoveWindow(hwndPropOKButton, 340,176-(bBrig*18),50,20, TRUE) ;
  MoveWindow(hwndPropCancelButton, 340,201-(bBrig*18),50,20, TRUE) ;

  for (iI = 0 ; iI < iSpanker ; iI ++)
    {
    yPosn = 23 + 18*iI ;
    fOldValue[iI*5]=fSailHeadWidth[iI] ;
    sprintf(sMessage, "%1.1f", fSailHeadWidth[iI]) ;
    MoveWindow(hwndPropText[iI*5],100,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[iI*5],sMessage) ; 
    fOldValue[1+iI*5]=fSailFootWidth[iI] ;
    sprintf(sMessage, "%1.1f", fSailFootWidth[iI]) ;
    MoveWindow(hwndPropText[1+iI*5],150,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[1+iI*5],sMessage) ; 
    fOldValue[2+iI*5]=fSailDepth[iI] ;
    sprintf(sMessage, "%1.1f", fSailDepth[iI]) ;
    MoveWindow(hwndPropText[2+iI*5],200,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[2+iI*5],sMessage) ; 
    fOldValue[3+iI*5]=fSailTheorMaxSpeed[iI] ;
    sprintf(sMessage, "%1.1f", fSailTheorMaxSpeed[iI]) ;
    MoveWindow(hwndPropText[3+iI*5],250,yPosn,40,18,TRUE) ;
    SetWindowText (hwndPropText[3+iI*5],sMessage) ; 
    fOldValue[4+iI*5]=iSailMaxReefs[iI] ;
    sprintf(sMessage, "%1d", iSailMaxReefs[iI]) ;
    MoveWindow(hwndPropText[4+iI*5],300,yPosn,30,18,TRUE) ;
    SetWindowText (hwndPropText[4+iI*5],sMessage) ; 
    }
  }

