/*-----------------------------------------
   MAIN.C -- HMS Surprise main module
   Version date: 03.09.22
  -----------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <commctrl.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "ezfont.h"
#include "Surprise.h"
#include "Menu.h"

// Prototypes:
int WINAPI        WinMain (HINSTANCE, HINSTANCE, PSTR, int) ;
LRESULT CALLBACK  WndProc (HWND, UINT, WPARAM, LPARAM) ;
BOOL CALLBACK     AboutDlgProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  EditProc (HWND, UINT, WPARAM, LPARAM) ;
BOOL CALLBACK     HelmDlgProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  MapWndProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  PropsProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  PlanWndProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT CALLBACK  SailWndProc (HWND, UINT, WPARAM, LPARAM) ;
LRESULT           StatusMessage (HWND, WPARAM, LPARAM) ;
LRESULT CALLBACK  SternWndProc (HWND, UINT, WPARAM, LPARAM) ;
VOID CALLBACK     Timer1Proc (HWND, UINT, UINT, DWORD) ;
VOID CALLBACK     Timer2Proc (HWND, UINT, UINT, DWORD) ;
BOOL CALLBACK     WindDlgProc (HWND, UINT, WPARAM, LPARAM) ;

extern void    Calculate () ;
extern void    CanShadow () ;
extern void    CycleEvents () ;
extern void    DataDumper () ;
extern void    EndPlayBack () ;
extern void    EndRecording () ;
extern void    DecrSail () ;
extern void    DerivedShipParameters () ;
extern void    DerivedSailParameters () ;
extern HFONT   EZCreateFont (HDC, char *, int, int, int, BOOL) ;
extern void    FileInitialize_SRF () ;
extern void    FileInitialize_SSF () ;
extern BOOL    FileOpenDlg (PSTR, PSTR) ;
extern BOOL    FileSaveDlg (PSTR, PSTR) ;
extern void    Focusser () ;
extern void    IncrSail () ;
extern void    LargerJib () ;
extern int     LoadConfiguration (int) ;
extern void    LoadDefault () ;
extern void    Map () ;
extern void    Pause () ;
extern void    PlanView () ;
extern void    PropertiesAccept () ;
extern void    PropertiesCancel () ;
extern void    PropertiesChange () ;
extern void    PropertiesPaint () ;
extern void    PropertiesSetUp () ;
extern void    Recorder (int, int, int) ;
extern void    Replayer () ;
extern void    SailPlan () ;
extern void    SaveConfiguration (int) ;
extern void    SetUpDisplay () ;
extern void    SelectSailParameter () ;
extern int     Sgn (float) ;
extern void    ShowActualJibAngle () ;
extern void    ShowActualStaysailsAngle () ;
extern void    ShowMessage () ;
extern void    ShowParameters () ;
extern void    ShowStatusMessage () ;
extern void    SmallerJib () ;
extern void    SternView () ;
extern void    YardAdjust (int) ;
extern void    Yardsmen () ;
extern void    ZoomIn () ;
extern void    ZoomOut () ;

/*-----------------------------------------
   WinMain: Main windows procedure    
   Version date: 03.09.04
-----------------------------------------*/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
  {
  static char szAppName[] = "simulator" ;
  MSG         msg ;
  WNDCLASSEX  wndclass ;
  HDC         hdc ;
    
  wndclass.cbSize        = sizeof (wndclass) ;
  wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
  wndclass.lpfnWndProc   = WndProc ;
  wndclass.cbClsExtra    = 0 ;
  wndclass.cbWndExtra    = 0 ;
  wndclass.hInstance     = hInstance ;
  hIcon = LoadIcon (hInstance, "Surprise") ;
  wndclass.hIcon         = hIcon ;
  wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
  wndclass.hbrBackground = (HBRUSH) GetStockObject (LTGRAY_BRUSH) ;
  wndclass.lpszMenuName  = "Surprise" ;
  wndclass.lpszClassName = szAppName ;
  wndclass.hIconSm       = LoadIcon (hInstance, "Surprise") ;

  RegisterClassEx (&wndclass) ;

  hwnd = CreateWindow (szAppName, szAppName,
         WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN ,
         CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
         NULL, NULL, hInstance, NULL) ;

  ShowWindow (hwnd,SW_SHOWMAXIMIZED) ;

  UpdateWindow (hwnd) ;
  InitCommonControls() ;

  while (GetMessage (&msg, NULL, 0, 0))
    {
    TranslateMessage (&msg) ;
    DispatchMessage (&msg) ;
    }
  return msg.wParam ;
  }

/*-----------------------------------------
   WndProc: Main window callback procedure    
   Version date: 03.09.22
-----------------------------------------*/
LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  static char     *szChildClass[] = { "Plan", "Sail", "Map", "Stern" } ;
  static WNDPROC   ChildProc[] = { PlanWndProc, SailWndProc, MapWndProc, SternWndProc } ;
  char            *cButtonText[] = { "Zoom in", "Zoom out", "Increase sail",
                   "Decrease sail", "Helmsman", "Yardsmen", "Pause" , "Storm jib", "Working jib"} ;
  static char      szPropsAppName[] = "Props" ;
  WNDCLASSEX       wndclass ;
  HDC              hdc ;
  PAINTSTRUCT      ps ;
  static int       xCurrentScroll;    // current horizontal scrolling position 
  static int       yCurrentScroll;    // current vertical scrolling position 
  static int       xMaxScroll;        // maximum horizontal scrolling position 
  static int       yMaxScroll;        // maximum vertical scrolling position 
  int              i1, i2, iI, iID, iSign , iSlider ;
  int              iFirstSailOfType, iTypeOfSail ;
  int              x, cx ;
  int              y, cy ;
  RECT             rWindow, rect ;
  SCROLLINFO si; 
  int i;                  // loop counter 
  HRESULT hr;
   
  switch (iMsg)
    {
    case WM_CREATE :
      wndclass.cbSize        = sizeof (wndclass) ;
      wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
      wndclass.cbClsExtra    = 0 ;
      wndclass.cbWndExtra    = 0 ;
      wndclass.hInstance     = hInstance ;
      wndclass.hIcon         = NULL ;
      wndclass.hCursor       = LoadCursor (NULL, IDC_CROSS) ;
      wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
      wndclass.lpszMenuName  = NULL ;
      wndclass.hIconSm       = NULL ;

      // Detect and adjust to screen resolution

      ShowWindow (hwnd,SW_SHOWMAXIMIZED) ;
      hdc = GetDC (hwnd) ;

      //create status bar (ID = 19)
      hwndStatus = CreateStatusWindow (WS_CHILDWINDOW | WS_VISIBLE | CCS_BOTTOM ,
        sMyProgName, hwnd, 19) ;

      GetClientRect (hwnd, &rect) ;
      xMax = rect.right-rect.left-16 ;
      yMax = rect.bottom-rect.top-16 ;     
      SetMapMode (hdc, MM_ISOTROPIC) ;
      SetWindowExtEx (hdc, 1600, 1106, NULL) ;
      SetWindowOrgEx (hdc, 0, 0, NULL);
      SetViewportOrgEx (hdc, 0, 0, NULL);
      cxFullScreen=GetSystemMetrics(SM_CXFULLSCREEN);
      cyFullScreen=GetSystemMetrics(SM_CYFULLSCREEN);
      if (cxFullScreen < 800)
        {
        // 640*480 screen
        ShowWindow (hwnd,SW_NORMAL) ;
        MoveWindow (hwnd, 0, 0, xMax, yMax+16, TRUE) ;
        iScreen = 1 ;
        fMagnify= 1.0 ;
        SetViewportExtEx (hdc, 1600, 1106, NULL) ;
        MyFont = (HFONT) EzCreateFont (hdc, "MS Sans Serif", 80, 0, 0, TRUE) ;
        cxChildPixels = 250 ;
        }
      if (cxFullScreen >= 800)
        {
        // 800*600 screen
        iScreen = 1 ;
        fMagnify= 1.0 ;
        SetViewportExtEx (hdc, 1600, 1106, NULL) ;
        MyFont = (HFONT) EzCreateFont (hdc, "MS Sans Serif", 80, 0, 0, TRUE) ;
        cxChildPixels = 250 ;
        } 
      if (cxFullScreen >= 1024)
        {
        // 1024*768 screen
        iScreen = 2 ;
        fMagnify= 800.0/1024.0 ;
        SetViewportExtEx (hdc, 2048, 1442, NULL) ;
        MyFont = (HFONT) EzCreateFont (hdc, "MS Sans Serif", 100, 0, 0, TRUE) ;
        cxChildPixels = 320 ;
        } 
      if (cxFullScreen >= 1152)
        {
        // 1152*864 screen
        iScreen = 3 ;
        fMagnify= 800.0/1152.0 ;
        SetViewportExtEx (hdc, 2304, 1634, NULL) ;
        MyFont = (HFONT) EzCreateFont (hdc, "MS Sans Serif", 113, 0, 0, TRUE) ;
        cxChildPixels = 360 ;
        } 
      if (cxFullScreen >= 1280)
        {
        // 1280*1024 screen
        iScreen = 4 ;
        fMagnify= 800.0/1280.0 ;
        SetViewportExtEx (hdc, 2560, 1954, NULL) ;
        MyFont = (HFONT) EzCreateFont (hdc, "MS Sans Serif", 124, 0, 0, TRUE) ;
        cxChildPixels = 400 ;
        } 
      SelectObject (hdc, MyFont) ;
      SelectObject (hdc, GetStockObject (LTGRAY_BRUSH)) ;
      SetBkColor(hdc, RGB (192,192,192)) ;

      // create brushes
      iI= 1 << (GetDeviceCaps (hdc, BITSPIXEL) * GetDeviceCaps (hdc, PLANES)) ;
      if (iI < 64000)
        {
        // 16 or 256 colours
        hSeaBrush = CreateSolidBrush (RGB (0,0,255)) ;
        hLandBrush = CreateSolidBrush (RGB (0,255,0)) ;
        hSkyBrush = CreateSolidBrush (RGB (0,255,255)) ;
        hDeckBrush = CreateSolidBrush (RGB (128,64,0)) ;
        hBlownBrush = CreateSolidBrush (RGB (128,0,0)) ;
        hForeAftBrush = CreateSolidBrush (RGB (239,239,239)) ;
        hStaySailBrush = CreateSolidBrush (RGB (192,192,192)) ;
        hForeSailBrush = CreateSolidBrush (RGB (128,128,0)) ;
        hMainSailBrush = CreateSolidBrush (RGB (128,128,128)) ;
        }
      else
        {
        hBlownBrush = CreateSolidBrush (RGB (142,0,0)) ;
        hDeckBrush = CreateSolidBrush (RGB (153,102,0)) ;
        hForeAftBrush = CreateSolidBrush (RGB (239,239,239)) ;
        hForeSailBrush = CreateSolidBrush (RGB (128,128,0)) ;
        hLandBrush = CreateSolidBrush (RGB (0,173,0)) ;
        hMainSailBrush = CreateSolidBrush (RGB (128,128,128)) ;
        hSeaBrush = CreateSolidBrush (RGB (0,56,207)) ;
        hSkyBrush = CreateSolidBrush (RGB (0,215,255)) ;
        hStaySailBrush = CreateSolidBrush (RGB (192,192,192)) ;
        }
      ReleaseDC (hwnd, hdc) ;

      // create pens
      hCoastPen = CreatePen(PS_SOLID,0,RGB(200,200,0)) ;
      hDarkGrayPen = CreatePen(PS_SOLID,0,RGB(128,128,128)) ;
      hGrayPen = CreatePen(PS_SOLID,0,RGB(170,170,170)) ;
      hGreenPen = CreatePen(PS_SOLID,0,RGB(0,100,0)) ;
      hHullPen = CreatePen(PS_SOLID,15,RGB(0,0,0)) ;
      hPinkPen = CreatePen(PS_SOLID,0,RGB(255,128,128)) ;
      hSeaBluePen = CreatePen(PS_SOLID,0,RGB(0,56,207)) ;
      hSkyBluePen = CreatePen(PS_SOLID,0,RGB(0,215,255)) ;
      hThickGrayPen = CreatePen(PS_SOLID,10,RGB(170,170,170)) ;
      hThickGreenPen = CreatePen(PS_SOLID,10,RGB(0,173,0)) ;
      hThickPinkPen = CreatePen(PS_SOLID,10,RGB(255,128,128)) ;
      hMastPen = CreatePen(PS_SOLID,6,RGB(0,0,0)) ;
      hRedPen = CreatePen(PS_SOLID,0,RGB(255,0,0)) ;
      hYellowPen = CreatePen(PS_SOLID,0,RGB(255,255,0)) ;

      // create custom child windows for planview, sailplan, map and sternview (ID=0 to 3)
      for (iI = 0 ; iI <= 3 ; iI++)
        {
        wndclass.lpfnWndProc   = ChildProc[iI] ;
        wndclass.lpszClassName = szChildClass[iI] ;
        if (iI == 3)
          wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
        RegisterClassEx (&wndclass) ;
        hwndChild[iI] = CreateWindowEx (WS_EX_CLIENTEDGE, szChildClass[iI], NULL,
          WS_CHILDWINDOW | WS_BORDER | WS_VISIBLE | WS_CLIPSIBLINGS , 
          0, 0, 0, 0, hwnd, (HMENU) iI, hInstance, NULL) ;
        }

      // create scrollbars (ID = 4 to 11)
      for (iI=0 ; iI< 8 ; iI++)
        {
        hwndSlider[iI] = CreateWindow ("scrollbar", NULL,
          WS_CHILDWINDOW | WS_VISIBLE | WS_CLIPSIBLINGS | SBS_HORZ, 0, 0, 0, 0,
          hwnd, (HMENU) (iI+4) , hInstance, NULL) ;
        }
 
      //create check boxes (ID = 20 to 22)
      for (iI = 0 ; iI <= 2 ; iI++)
        {
        hwndCheck[iI] = CreateWindow ("button", NULL,
          WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_AUTOCHECKBOX, 0,0,0,0,
          hwnd, (HMENU) (iI+20), hInstance, NULL) ;
        SendMessage(hwndCheck[iI], WM_SETFONT, (WPARAM) MyFont, 0) ;
        }

      //create buttons (ID= 30 to 38)
      for (iI = 0 ; iI <= 8 ; iI++)
        {
        hwndButton[iI] = CreateWindow ("button", cButtonText[iI],
          WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, 0,0,0,0,
          hwnd, (HMENU) (iI+30), hInstance, NULL) ;
        SendMessage(hwndButton[iI], WM_SETFONT, (WPARAM) MyFont, 0) ;
        }

      //text box (ID=39)
      hwndText = CreateWindowEx (WS_EX_CLIENTEDGE,"edit", NULL,
        WS_CHILD | WS_VISIBLE |WS_BORDER | ES_RIGHT, 0,0,0,0,
        hwnd, (HMENU) 39, hInstance, NULL) ;
      SendMessage(hwndText, WM_SETFONT, (WPARAM) MyFont, 0) ;

      // register window for changing ship/sail dimensions and properties and islands
      wndclass.lpszClassName = szPropsAppName ;
      wndclass.lpfnWndProc   = PropsProc ;
      wndclass.hbrBackground = (HBRUSH) GetStockObject (LTGRAY_BRUSH) ;
      RegisterClassEx (&wndclass) ;

      // create bitmaps for Planview, SailPlan and Sternview
      hdc = GetDC (hwndChild[0]) ;
      hBitmap_0 = CreateCompatibleBitmap (hdc, cxChildPixels, cxChildPixels) ;
      ReleaseDC (hwndChild[0], hdc) ;
      hdc = GetDC (hwndChild[1]) ;
      hBitmap_1 = CreateCompatibleBitmap (hdc, cxChildPixels, cxChildPixels) ;
      ReleaseDC (hwndChild[1], hdc) ;
      hdc = GetDC (hwndChild[3]) ;
      hBitmap_3 = CreateCompatibleBitmap (hdc, 113, 229) ;
      ReleaseDC (hwndChild[3], hdc) ;

      QueryPerformanceFrequency (&liFrequency);
      if (liFrequency.QuadPart == 0)
        {
        MessageBox (hwnd, "Using low resolution counter", sMyProgName, MB_ICONINFORMATION | MB_OK) ;
        srand(time (NULL)) ;
        fLastTime = (float) clock()/CLOCKS_PER_SEC ;
        }
      else
        {
        QueryPerformanceCounter (&liCounter) ;
        fLastTime = (float) (liCounter.QuadPart / (float) liFrequency.QuadPart) ;
        }
      strcpy(sMyProgName, "HMS Surprise") ;
      bBrig = FALSE ;
      //if (bBrig) 
      //  strcpy(sMyProgName, "Brig Simulator") ;
      //else
      //  strcpy(sMyProgName, "Frigate Simulator") ;

      hMenu=GetMenu(hwnd) ;
      FileInitialize_SSF() ;
      hFile = CreateFile ("Default.ssf", GENERIC_READ, 0, NULL, OPEN_EXISTING, 0,0) ;
      if (hFile == INVALID_HANDLE_VALUE)
        {
        CloseHandle (hFile) ;
        SetWindowText (hwnd, sMyProgName) ;
        LoadDefault() ;
        DerivedShipParameters () ;
        DerivedSailParameters () ;
        CanShadow () ; 
        Calculate ();
        }
      else
        {
        CloseHandle (hFile) ;
        strcpy (sFileName, "Default.ssf") ;
        if (LoadConfiguration (1)==1)
          {
          MessageBox (hwnd, "Error loading default file", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
          SetWindowText (hwnd, sMyProgName) ;
          LoadDefault() ;
          DerivedShipParameters () ;
          DerivedSailParameters () ;
          CanShadow () ; 
          Calculate ();
          }
        else
          {
          EnableMenuItem(hMenu, IDM_OPENDEFAULT, MF_ENABLED) ;
          DerivedShipParameters () ;
          DerivedSailParameters () ;
          SetUpDisplay() ;
          CanShadow () ;
          strcpy (sTitleName, "Default.ssf") ;
          strcpy (sMessage, sMyProgName) ;
          strcat (sMessage, " (") ;
          strncat (sMessage, sTitleName, 30) ;
          strcat (sMessage, ")");
          SetWindowText (hwnd, sMessage) ;
          bAutoBearing = bAutoBearing_2 ;
          }
        }

      //initialise scroling
      bScroll = FALSE ;
      xMaxScroll=0 ;
      xCurrentScroll=0 ;
      yMaxScroll=0 ;
      yCurrentScroll=0 ;

      // start timers
      if (!SetTimer (hwnd, 1, 460, (TIMERPROC) Timer1Proc))
        {
        MessageBox (hwnd, "Too many clocks or timers!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
        return FALSE ;
        }
  
      if (!SetTimer (hwnd, 2, 5, (TIMERPROC) Timer2Proc))
        {
        MessageBox (hwnd, "Too many clocks or timers!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
        return FALSE ;
        }

      return 0 ;

    case WM_MOUSEMOVE :
      // display default text in status bar
      //if (bBrig)
        //LoadString (hInstance, IDS_DEFAULT_BRIG, sMessage,120) ;
      //else
        //LoadString (hInstance, IDS_DEFAULT_FRIGATE, sMessage,120) ;
      LoadString (hInstance, IDS_DEFAULT, sMessage,120) ;
      ShowStatusMessage() ;
      return 0 ;

    case WM_COMMAND :
      switch (LOWORD (wParam))
        { 
        case IDM_EXIT :
          if (bRecording)
            EndRecording () ;
          if (bPlayBack)
            EndPlayBack () ;
          SendMessage (hwnd, WM_CLOSE, 0, 0) ;
          return 0 ;
        case IDM_FRIGATE :
        case IDM_BRIG :
          bPaused = TRUE ;
          if (LOWORD (wParam)==IDM_FRIGATE)
            {
            bBrig = FALSE ;
            //strcpy(sMyProgName, "Frigate Simulator") ;
            EnableMenuItem (hMenu, IDM_UPPERTOPSAILYARDS, MF_GRAYED) ;
            ModifyMenu (hMenu, IDM_LOWERTOPSAILYARDS, MF_BYCOMMAND && MF_STRING, IDM_LOWERTOPSAILYARDS, "T&opsail yards") ;
            }
          if (LOWORD (wParam)==IDM_BRIG)
            {
            bBrig = TRUE ;
            //strcpy(sMyProgName, "Brig Simulator") ;
            EnableMenuItem (hMenu, IDM_UPPERTOPSAILYARDS, MF_ENABLED) ;
            ModifyMenu (hMenu, IDM_LOWERTOPSAILYARDS, MF_BYCOMMAND && MF_STRING, IDM_LOWERTOPSAILYARDS, "&Lower topsail yards") ;
            }
          SetWindowText (hwnd, sMyProgName) ;
          CheckMenuItem(hMenu, iSpeedFactor+370, MF_UNCHECKED) ;
          iSpeedFactor = 0 ;
          CheckMenuItem(hMenu, iSpeedFactor+370, MF_CHECKED) ;
          if (!SetTimer (hwnd, 1, 460/iAcceleration[iSpeedFactor], (TIMERPROC) Timer1Proc))
            {
            MessageBox (hwnd, "Too many clocks or timers!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
            return FALSE ;
            }
          if (iSpeedFactor==0)
            fRotatInertFactor = fOldRotatInertFactor ;
          else
            {
            fOldRotatInertFactor = fRotatInertFactor ;
            fRotatInertFactor =  0.2 ;
            }
          if (bRecording)
            EndRecording () ;
          if (bPlayBack)
            EndPlayBack () ;
          LoadDefault () ;
          DerivedShipParameters () ;
          DerivedSailParameters () ;
          CanShadow () ;
          SelectSailParameter() ;
          bPaused = FALSE ;
          sFileName[0] = '\0' ;
          return 0 ;
        case IDM_OPENDEFAULT : 
          strcpy (sFileName, "Default.ssf") ;
          if (LoadConfiguration (1)==1)
            {
            MessageBox (hwnd, "Error loading default file", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
            SetWindowText (hwnd, sMyProgName) ;
            LoadDefault() ;
            DerivedShipParameters () ;
            DerivedSailParameters () ;
            CanShadow () ; 
            Calculate ();
            }
          else
            {
            EnableMenuItem(hMenu, IDM_OPENDEFAULT, MF_ENABLED) ;
            DerivedShipParameters () ;
            DerivedSailParameters () ;
            SetUpDisplay() ;
            CanShadow () ;
            strcpy (sTitleName, "Default.ssf") ;
            strcpy (sMessage, sMyProgName) ;
            strcat (sMessage, " (") ;
            strncat (sMessage, sTitleName, 30) ;
            strcat (sMessage, ")");
            SetWindowText (hwnd, sMessage) ;
            bAutoBearing = bAutoBearing_2 ;
            }
          return 0 ;
        case IDM_OPEN : 
          if (bRecording)
            EndRecording () ;
          if (bPlayBack)
            EndPlayBack () ;
          FileInitialize_SSF() ;
          bAutoBearing = FALSE ;
          CheckMenuItem(hMenu, iSpeedFactor+370, MF_UNCHECKED) ;
          iSpeedFactor = 0 ;
          CheckMenuItem(hMenu, iSpeedFactor+370, MF_CHECKED) ;
          if (!SetTimer (hwnd, 1, 460/iAcceleration[iSpeedFactor], (TIMERPROC) Timer1Proc))
            {
            MessageBox (hwnd, "Too many clocks or timers!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
            return FALSE ;
            }
          if (iSpeedFactor==0)
            fRotatInertFactor = fOldRotatInertFactor ;
          else
            {
            fOldRotatInertFactor = fRotatInertFactor ;
            fRotatInertFactor =  0.2 ;
            }
          if (FileOpenDlg (sFileName,sTitleName)) 
            {
            if (LoadConfiguration (1)==1)
              {
              LoadDefault () ;
              DerivedShipParameters () ;
              DerivedSailParameters () ;
              CanShadow () ;
              SetWindowText (hwnd, sMyProgName) ;
              sFileName[0] = '\0' ;
              return 0 ;
              }
            DerivedShipParameters () ;
            DerivedSailParameters () ;
            SetUpDisplay() ;
            CanShadow () ;
            strcpy (sMessage, sMyProgName) ;
            strcat (sMessage, " (") ;
            strncat (sMessage, sTitleName, 30) ;
            strcat (sMessage, ")");
            SetWindowText (hwnd, sMessage) ;
            bAutoBearing = bAutoBearing_2 ;
            return 0 ;
            }
          else
            {
            bAutoBearing = bAutoBearing_2 ;
            return 1 ;
            }
        case IDM_SAVEDEFAULT :
          EnableMenuItem(hMenu, IDM_OPENDEFAULT, MF_ENABLED) ;
          strcpy (sMessage,sFileName) ; 
          strcpy(sFileName,"Default.ssf") ;
          SaveConfiguration(1) ;
          strcpy (sFileName, sMessage) ; 
          bAutoBearing = bAutoBearing_2 ;
          return 0 ;
        case IDM_SAVE : 
          if (bRecording)
            EndRecording () ;
          if (bPlayBack)
            EndPlayBack () ;
          FileInitialize_SSF() ;
          bAutoBearing = FALSE ;
          if (sFileName[0])
            { 
            SaveConfiguration(1) ;
            bAutoBearing = bAutoBearing_2 ;
            return 0 ;
            }
        case IDM_SAVEAS : 
          if (bRecording)
            EndRecording () ;
          if (bPlayBack)
            EndPlayBack () ;
          FileInitialize_SSF() ;
          bAutoBearing = FALSE ;
          if (FileSaveDlg (sFileName,sTitleName)) 
            {
            SaveConfiguration(1) ;
            strcpy (sMessage, sMyProgName) ;
            strcat (sMessage, " (") ;
            strncat (sMessage, sTitleName, 30) ;
            strcat (sMessage, ")") ;
            SetWindowText (hwnd, sMessage) ;
            }
          bAutoBearing = bAutoBearing_2 ;
          return 0 ;
        case IDM_SAVEPOSITION :
          bSavePosition = TRUE ;
          CheckMenuItem(hMenu, IDM_DONTSAVEPOSITION, MF_UNCHECKED) ;
          CheckMenuItem(hMenu, IDM_SAVEPOSITION, MF_CHECKED) ;
          return 0 ;
        case IDM_DONTSAVEPOSITION :
          bSavePosition = FALSE ;
          CheckMenuItem(hMenu, IDM_DONTSAVEPOSITION, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_SAVEPOSITION, MF_UNCHECKED) ;
          return 0 ;
        case IDM_RECORDER :
          FileInitialize_SRF() ;
          if (FileSaveDlg (sFileName2,sTitleName2))
            {
            SaveConfiguration(2) ;
            EnableMenuItem(hMenu, IDM_RECORDER, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_STOPRECORDER, MF_ENABLED) ;
            EnableMenuItem(hMenu, IDM_PLAYBACK, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_STOPPLAYBACK, MF_GRAYED) ;
            CheckMenuItem(hMenu, iSpeedFactor+370, MF_UNCHECKED) ;
            iSpeedFactor = 0 ;
            CheckMenuItem(hMenu, iSpeedFactor+370, MF_CHECKED) ;
            if (!SetTimer (hwnd, 1, 460/iAcceleration[iSpeedFactor], (TIMERPROC) Timer1Proc))
              {
              MessageBox (hwnd, "Too many clocks or timers!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
              return FALSE ;
              }
            if (iSpeedFactor==0)
              { 
              fRotatInertFactor = fOldRotatInertFactor ;
              }
            else
              {
              fOldRotatInertFactor = fRotatInertFactor ;
              fRotatInertFactor =  0.2 ;
              }
            EnableMenuItem(hMenu, IDM_SHIPDIMENSIONS, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_SHIPPROPERTIES, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_SAILDIMENSIONS, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_FORENAFT, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_ISLANDS, MF_GRAYED) ;
            strcpy (sMessage, sMyProgName) ;
            strcat (sMessage, " - recording voyage (") ;
            strncat (sMessage, sTitleName2, 30) ;
            strcat (sMessage, ")") ;
            SetWindowText (hwnd, sMessage) ;
            bRecording = TRUE ;
            iTick = 0 ;
            iLastRecTime = 0 ;
            fRecordStartTime = fNow ;
            fLastSimTime = 0.0 ;
            fLastRealTime = 0.0 ;
            }
          return 0 ;
        case IDM_STOPRECORDER :
          EndRecording() ;
          sFileName[0] = '\0' ;
          return 0 ;
        case IDM_PLAYBACK :
          FileInitialize_SRF() ;
          bAutoBearing = FALSE ;
          CheckMenuItem(hMenu, iSpeedFactor+370, MF_UNCHECKED) ;
          iSpeedFactor = 0 ;
          CheckMenuItem(hMenu, iSpeedFactor+370, MF_CHECKED) ;
          if (!SetTimer (hwnd, 1, 460/iAcceleration[iSpeedFactor], (TIMERPROC) Timer1Proc))
            {
            MessageBox (hwnd, "Too many clocks or timers!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
            return FALSE ;
            }
          if (iSpeedFactor==0)
            fRotatInertFactor = fOldRotatInertFactor ;
          else
            {
            fOldRotatInertFactor = fRotatInertFactor ;
            fRotatInertFactor =  0.2 ;
            }
          EnableMenuItem(hMenu, IDM_SHIPDIMENSIONS, MF_GRAYED) ;
          EnableMenuItem(hMenu, IDM_SHIPPROPERTIES, MF_GRAYED) ;
          EnableMenuItem(hMenu, IDM_SAILDIMENSIONS, MF_GRAYED) ;
          EnableMenuItem(hMenu, IDM_FORENAFT, MF_GRAYED) ;
          EnableMenuItem(hMenu, IDM_ISLANDS, MF_GRAYED) ;
          if (!SetTimer (hwnd, 1, 460/iAcceleration[iSpeedFactor], (TIMERPROC) Timer1Proc))
            {
            MessageBox (hwnd, "Too many clocks or timers!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
            return FALSE ;
            }
          if (FileOpenDlg (sFileName2,sTitleName2)) 
            {
            if (LoadConfiguration (2)==1)
              {
              LoadDefault () ;
              DerivedShipParameters () ;
              DerivedSailParameters () ;
              CanShadow () ;
              strcpy (sMessage, sMyProgName);
              SetWindowText (hwnd, sMessage) ;
              sFileName2[0] = '\0' ;
              return 0 ;
              }
            DerivedShipParameters () ;
            DerivedSailParameters () ;
            SetUpDisplay() ;
            CanShadow () ;
            EnableMenuItem(hMenu, IDM_RECORDER, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_STOPRECORDER, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_PLAYBACK, MF_GRAYED) ;
            EnableMenuItem(hMenu, IDM_STOPPLAYBACK, MF_ENABLED) ;
            strcpy (sMessage, sMyProgName) ;
            strcat (sMessage, " - replaying voyage (") ;
            strncat (sMessage, sTitleName2, 30) ;
            strcat (sMessage, ")") ;
            SetWindowText (hwnd, sMessage) ;
            SetWindowText (hwnd, sMessage) ;
            bPlayBack = TRUE ;
            fRecordStartTime = fNow ;
            fLastSimTime = 0.0 ;
            fLastRealTime = 0.0 ;
            iLastReplayItem = -1 ;
            iLastRecTime = 0 ;
            Replayer () ;
            return 0 ;
            }
          else
            {
            return 1 ;
            }
          return 0 ;
        case IDM_STOPPLAYBACK :
          sFileName[0] = '\0' ;
          EndPlayBack () ;
          return 0 ;
        case IDC_ZOOMIN :
          ZoomIn() ;
          return 0 ;
        case IDC_ZOOMOUT :
          ZoomOut() ;
          return 0 ;
        case IDC_SMALLERJIB :
          SmallerJib() ;
          return 0 ;
        case IDC_LARGERJIB :
          LargerJib() ;
          return 0 ;
        case IDC_YARDSMEN :
          Yardsmen() ;
          return 0 ;
        case IDC_HELMSMAN :
          Helmsman() ;
          return 0 ;
        case IDC_PAUSE :
          Pause() ;
          return 0 ;
        case IDC_BEARING:
          if (HIWORD (wParam) == EN_SETFOCUS)
            {
            SetWindowText(hwndText,"") ;
            return 0 ;
            }
          if (GetWindowText(hwndText, sMessage, 12) == 3)
            {
            i1 = atoi(sMessage) ;
            if ((i1 < 0) || (i1 > 360))
              {
              MessageBox (hwnd,"Must be between 000 and 360","Bearing", MB_OK|MB_ICONERROR) ;
              SetWindowText(hwndText,"360") ;
              fOffsetAngle = 360.0 ;
              }
            else
              fOffsetAngle = (float) i1 ;
            bPlanViewIsChanged = TRUE ;
            SetFocus (hwndCheck[1]) ;
            }
          return 0 ;
        case IDM_CONVENTIONAL :
          bParalax = FALSE ;
          CheckMenuItem(hMenu, IDM_CONVENTIONAL, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_PERSPECTIVE, MF_UNCHECKED) ;
          return 0 ;
        case IDM_PERSPECTIVE :
          bParalax = TRUE ;
          CheckMenuItem(hMenu, IDM_CONVENTIONAL, MF_UNCHECKED) ;
          CheckMenuItem(hMenu, IDM_PERSPECTIVE, MF_CHECKED) ;
          return 0 ;
        case IDM_TRACKSHIP :
          bTracking = TRUE ;
          SendMessage (hwndCheck[2], BM_SETCHECK, 1, 0);
          CheckMenuItem(hMenu, IDM_TRACKSHIP, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_DONTTRACKSHIP, MF_UNCHECKED) ;
          return 0 ;
        case IDM_DONTTRACKSHIP :
          bTracking = FALSE ;
          SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0);
          CheckMenuItem(hMenu, IDM_DONTTRACKSHIP, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_TRACKSHIP, MF_UNCHECKED) ;
          return 0 ;
        case IDC_TRACKSHIP :
          bTracking = (int) SendMessage (hwndCheck[2], BM_GETCHECK, 0, 0) ;
          if (bTracking==TRUE)
            {
            CheckMenuItem (hMenu, IDM_TRACKSHIP, MF_CHECKED) ;
            CheckMenuItem (hMenu, IDM_DONTTRACKSHIP, MF_UNCHECKED) ;
            if ((fLongitude < fLeftEdge) || (fLongitude > fRightEdge))
              bMapMustBePainted = TRUE ;
            if ((fLatitude < fBottomEdge) || (fLatitude > fTopEdge))
              bMapMustBePainted = TRUE ;
            }
          else
            {
            CheckMenuItem (hMenu, IDM_TRACKSHIP, MF_UNCHECKED) ;
            CheckMenuItem (hMenu, IDM_DONTTRACKSHIP, MF_CHECKED) ;
            }
          return 0 ;
        case IDC_COUPLEFORE :
          bCoupleFore = (int) SendMessage (hwndCheck[0], BM_GETCHECK, 0, 0) ;
          if (bRecording)
            Recorder (11, (int) bCoupleFore, 0) ;
          return 0 ;
        case IDC_COUPLEMIZZEN :
          bCoupleMizzen = (int) SendMessage (hwndCheck[1], BM_GETCHECK, 0, 0) ;
          if (bRecording)
            Recorder (10, (int) bCoupleMizzen, 0) ;
          return 0 ;
        case IDC_INCRSAIL :
          IncrSail() ;
          return 0 ;
        case IDC_DECRSAIL :
          DecrSail() ;
          return 0 ;
        case IDM_CLEARTRACK :
          iMapCounter = 0 ;
          iMapSkip = 0 ;
          iMapSkipFactor = 1;
          bMapMustBePainted = TRUE ;
          return 0 ;
        case IDM_ROYALYARDS :
        case IDM_TOPGALYARDS :
        case IDM_UPPERTOPSAILYARDS :
        case IDM_LOWERTOPSAILYARDS :
        case IDM_LOWERYARDS :
          CheckMenuItem(hMenu, iYardLevelShown+310, MF_UNCHECKED) ;
          iYardLevelShown = LOWORD (wParam)-310 ; 
          CheckMenuItem(hMenu, iYardLevelShown+310, MF_CHECKED) ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_SLIPSTREAM :
          CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_UNCHECKED) ;
          bShowSlipStream = TRUE ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_NOSLIPSTREAM :
          CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_UNCHECKED) ;
          bShowSlipStream = FALSE ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_SELECTBEARING :
          CheckMenuItem(hMenu, IDM_SELECTBEARING, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_AUTOBEARING, MF_UNCHECKED) ;
          bAutoBearing = FALSE ;
          bAutoBearing_2 = FALSE ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_AUTOBEARING :
          CheckMenuItem(hMenu, IDM_SELECTBEARING, MF_UNCHECKED) ;
          CheckMenuItem(hMenu, IDM_AUTOBEARING, MF_CHECKED) ;
          bAutoBearing = TRUE ;
          bAutoBearing_2 = TRUE ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_COMPASSRING :
          CheckMenuItem(hMenu, IDM_COMPASSRING, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_NOCOMPASSRING, MF_UNCHECKED) ;
          bCompassRing = TRUE ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_NOCOMPASSRING :
          CheckMenuItem(hMenu, IDM_NOCOMPASSRING, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_COMPASSRING, MF_UNCHECKED) ;
          bCompassRing = FALSE ;
          bPlanViewIsChanged = TRUE ;
          return 0 ;
        case IDM_SAILANGLE :
        case IDM_WINDSTRIKINGANGLE :
        case IDM_EXPOSEDPERCENTAGE :
        case IDM_EFFECTIVEAREA :
        case IDM_USEFULPERCENTAGE :
        case IDM_USEFULFORCE :
        case IDM_FORWARDTHRUST :
        case IDM_LEEWARDTHRUST :
        case IDM_ROTATIONALTHRUST :    
        case IDM_HEELINGTHRUST :
          CheckMenuItem(hMenu, iDisplayParam+322, MF_UNCHECKED) ;
          iDisplayParam = LOWORD (wParam)-322 ;
          CheckMenuItem(hMenu, iDisplayParam+322, MF_CHECKED) ;
          SelectSailParameter() ;
          return 0 ;
        case IDM_SHIPDIMENSIONS :
        case IDM_SHIPPROPERTIES :
        case IDM_SAILDIMENSIONS :
        case IDM_FORENAFT       :
        case IDM_ISLANDS        : 
          iModifyMenu = LOWORD (wParam) - IDM_SHIPDIMENSIONS;
          hwndProps = CreateWindowEx (WS_EX_TOPMOST, szPropsAppName, NULL, 
            WS_OVERLAPPED | WS_CAPTION | WS_THICKFRAME | WS_CLIPCHILDREN | WS_MINIMIZEBOX | 
            WS_POPUP | WS_CLIPCHILDREN ,
            0, 0, 0, 0, NULL, NULL, hInstance, NULL) ;
          ShowWindow (hwndProps,SW_SHOW) ;
          hwndPropOKButton = CreateWindow ("button", "&OK",
            WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, 0,0,0,0,
            hwndProps, (HMENU) IDC_PROP_OK, hInstance, NULL) ;
          hwndPropCancelButton = CreateWindow ("button", "&Cancel",
            WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | BS_PUSHBUTTON, 0,0,0,0,
            hwndProps, (HMENU) IDC_PROP_CANCEL, hInstance, NULL) ;
          SendMessage(hwndPropOKButton, WM_SETFONT, (WPARAM) MyFont, 0) ;
          SendMessage(hwndPropCancelButton, WM_SETFONT, (WPARAM) MyFont, 0) ;
          for (iI=0 ; iI<iFields[iModifyMenu] ; iI++)
            {
            hwndPropText[iI] = CreateWindowEx (WS_EX_CLIENTEDGE,"edit", NULL,
              WS_CHILD | WS_VISIBLE |WS_BORDER | ES_RIGHT, 0,0,0,0,
              hwndProps, (HMENU) iI, hInstance, NULL) ;
              SendMessage(hwndPropText[iI], WM_SETFONT, (WPARAM) MyFont, 0) ;
            fnOldEditProc = (WNDPROC) SetWindowLong (hwndPropText[iI], GWL_WNDPROC, (LONG) EditProc) ;
            }
          PropertiesSetUp() ;
          iFocus = 0 ;
          Focusser () ;
          return 0 ;
        case IDM_RUDDERCOMMANDS :
          bModernHelm = TRUE ;
          CheckMenuItem(hMenu, IDM_RUDDERCOMMANDS, MF_CHECKED) ;
          CheckMenuItem(hMenu, IDM_HELMCOMMANDS, MF_UNCHECKED) ;
          SetScrollPos (hwndSlider[4], SB_CTL, -iNewHelmAngle, TRUE) ;
          iNewHelmAngle = -GetScrollPos (hwndSlider[4], SB_CTL) ;
          return 0 ;
        case IDM_HELMCOMMANDS :
          bModernHelm = FALSE ;
          CheckMenuItem(hMenu, IDM_RUDDERCOMMANDS, MF_UNCHECKED) ;
          CheckMenuItem(hMenu, IDM_HELMCOMMANDS, MF_CHECKED) ;
          SetScrollPos (hwndSlider[4], SB_CTL, iNewHelmAngle, TRUE) ;
          iNewHelmAngle = GetScrollPos (hwndSlider[4], SB_CTL) ;
          return 0 ;
        case IDM_ABOUT :
          DialogBox (hInstance, "AboutBox", hwnd, AboutDlgProc) ;
          return 0 ;
        case IDM_HELP :
          WinHelp (hwnd, "Surprise.hlp", HELP_CONTENTS, 0) ;
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
          SetUpDisplay () ;
          return 0 ;
        case IDM_SPEED_1 :
        case IDM_SPEED_2 :
        case IDM_SPEED_4 :
        case IDM_SPEED_8 :
        case IDM_SPEED_16 :
        case IDM_SPEED_32 :
          CheckMenuItem(hMenu, iSpeedFactor+370, MF_UNCHECKED) ;
          iSpeedFactor = LOWORD (wParam)-370 ; 
          CheckMenuItem(hMenu, iSpeedFactor+370, MF_CHECKED) ;
          if (!SetTimer (hwnd, 1, 460/iAcceleration[iSpeedFactor], (TIMERPROC) Timer1Proc))
            {
            MessageBox (hwnd, "Too many clocks or timers!", sMyProgName, MB_ICONEXCLAMATION | MB_OK) ;
            return FALSE ;
            }
          if (iSpeedFactor==0)
            fRotatInertFactor = fOldRotatInertFactor ;
          else
            {
            fOldRotatInertFactor = fRotatInertFactor ;
            fRotatInertFactor =  0.2 ;
            }
          return 0 ;
        case IDM_DATADUMP :
          DataDumper () ;
          return 0 ;
        }

    case WM_MENUSELECT :
      return StatusMessage (hwnd, wParam, lParam) ;

    case WM_PAINT :
      hdc = BeginPaint (hwnd, &ps) ;
      EndPaint (hwnd, &ps) ;
      SetUpDisplay() ;
      PlanView() ;
      SailPlan() ;
      return 0 ;

    case WM_SIZE :
      // Retrieve the dimensions of the client area 
      xClient = LOWORD (lParam); 
      yClient = HIWORD (lParam); 

      {
      WINDOWPLACEMENT wp ;
      wp.length=sizeof(WINDOWPLACEMENT) ;
      GetWindowPlacement (hwnd, &wp) ;
      if (wp.showCmd==SW_SHOWMAXIMIZED)
        {
        xCurrentScroll = 0 ;
        yCurrentScroll = 0 ;
        xScrollPos = 0 ;
        yScrollPos = 0 ;
        bScroll = TRUE ;
        }
      }

      // Set the horizontal scrolling range and page size 
      xMaxScroll = xMax ;
      xCurrentScroll = min(xCurrentScroll,xMaxScroll) ; 
      si.cbSize = sizeof(si) ; 
      si.fMask  = SIF_RANGE | SIF_POS | SIF_PAGE ; 
      si.nPage  = xClient ; 
      si.nPos   = xCurrentScroll ; 
      si.nMin   = 0 ; 
      si.nMax   = xMaxScroll ; 
      SetScrollInfo(hwnd, SB_HORZ, &si, TRUE) ; 
      // Set the vertical scrolling range and page size
      yMaxScroll= yMax ;
      yCurrentScroll = min(yCurrentScroll,yMaxScroll) ; 
      GetClientRect (hwndStatus, &rect) ;
      si.cbSize = sizeof(si) ; 
      si.fMask  = SIF_RANGE | SIF_PAGE | SIF_POS ; 
      si.nPage  = yClient + (rect.bottom-rect.top) ; 
      si.nPos   = yCurrentScroll ; 
      si.nMin   = 0 ; 
      si.nMax   = yMaxScroll ; 
      SetScrollInfo(hwnd, SB_VERT, &si, TRUE) ; 

      // status bar
      GetWindowRect (hwndStatus, &rWindow) ;
      cy = rWindow.bottom - rWindow.top ;
      x = 0 ;
      y = yClient - cy ;
      cx = xClient ;
      MoveWindow (hwndStatus, x, y, cx, cy, TRUE) ;

      return 0 ;

    case WM_HSCROLL :
      iID = GetWindowLong ((HWND) lParam, GWL_ID) ;
      iSlider = (int) iID-4 ;
      if (lParam == 0)
        iSlider = 99 ;  
      switch (iSlider)
        {
        case 0 :  
        case 1 :
        case 2 :  
          // brace yards
          if (bYardsmen)
            break ;
          switch (LOWORD(wParam)) 
            {
            case SB_LINEUP :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-1 ;
              break ;
            case SB_LINEDOWN :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+1 ;
              break ;
            case SB_PAGEUP :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-10 ;
              break ;
            case SB_PAGEDOWN :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+10 ;
              break ;
            case SB_THUMBPOSITION : 
              iNewYardAngle[iSlider]=(int) HIWORD(wParam) ;
              if (iNewYardAngle[iSlider] > 360)
                iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-65536 ;
              break ;
            case SB_THUMBTRACK :
              return ;
            }
          SetScrollPos (hwndSlider[iSlider], SB_CTL, iNewYardAngle[iSlider], TRUE) ;
          YardAdjust(iSlider) ;
          if (bRecording)
            Recorder (2, iSlider, iNewYardAngle[iSlider]) ;
          break ;
        case 3 :  
          // brace spanker boom
          switch (LOWORD(wParam)) 
            {
            case SB_LINEUP :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+1 ;
              break ;
            case SB_LINEDOWN :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-1 ;
              break ;
            case SB_PAGEUP :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+10 ;
              break ;
            case SB_PAGEDOWN :
              iNewYardAngle[iSlider]=iNewYardAngle[iSlider]-10 ;
              break ;
            case SB_THUMBPOSITION :
              iNewYardAngle[iSlider]=-(int) HIWORD(wParam) ;
              if (iNewYardAngle[iSlider] < -360)
                iNewYardAngle[iSlider]=iNewYardAngle[iSlider]+65536 ;
            case SB_THUMBTRACK :
              return ;
              break ;
            }
          SetScrollPos (hwndSlider[iSlider], SB_CTL, -iNewYardAngle[iSlider], TRUE) ;
          YardAdjust(iSlider) ;
          if (bRecording)
            Recorder (2, iSlider, iNewYardAngle[iSlider]) ;
          break ;
        case 4 :  
          // helm
          if (bHelmsman)
            if (bModernHelm)
              SetScrollPos (hwndSlider[iSlider], SB_CTL, -iNewHelmAngle, TRUE) ;
            else
              SetScrollPos (hwndSlider[iSlider], SB_CTL, iNewHelmAngle, TRUE) ;
          else
            {
            switch (LOWORD(wParam)) 
              {
              case SB_LINEUP :
                if (bModernHelm)
                  iNewHelmAngle=iNewHelmAngle+1 ;
                else
                  iNewHelmAngle=iNewHelmAngle-1 ;
                break ;
              case SB_LINEDOWN :
                if (bModernHelm)
                  iNewHelmAngle=iNewHelmAngle-1 ;
                else
                  iNewHelmAngle=iNewHelmAngle+1 ;
                break ;
              case SB_PAGEUP :
                if (bModernHelm)
                  iNewHelmAngle=iNewHelmAngle+10;
                else
                  iNewHelmAngle=iNewHelmAngle-10 ;
                break ;
              case SB_PAGEDOWN :
                if (bModernHelm)
                  iNewHelmAngle=iNewHelmAngle-10;
                else
                  iNewHelmAngle=iNewHelmAngle+10 ;
                break ;
              case SB_THUMBPOSITION :
                if (bModernHelm)
                  iNewHelmAngle= - (int) HIWORD(wParam) ;
                else
                  iNewHelmAngle= (int) HIWORD(wParam) ;
                if (iNewHelmAngle > 360)
                  iNewHelmAngle=iNewHelmAngle-65536 ;
                if (iNewHelmAngle < -360)
                  iNewHelmAngle=iNewHelmAngle+65536 ;
                break ;
              case SB_THUMBTRACK :
                return ;
                break ;
              default:
                break ;
              }
            if (bModernHelm)
              {
              SetScrollPos (hwndSlider[iSlider], SB_CTL, -iNewHelmAngle, TRUE) ;
              iNewHelmAngle = -GetScrollPos (hwndSlider[iSlider], SB_CTL) ;
              }
            else 
              {   
              SetScrollPos (hwndSlider[iSlider], SB_CTL, iNewHelmAngle, TRUE) ;
              iNewHelmAngle = GetScrollPos (hwndSlider[iSlider], SB_CTL) ;
              } 
            if (bRecording)
              Recorder (1, iNewHelmAngle, 0) ;
            ShowSelectedHelmAngle () ;
            }
          break ;
        case 5 :  
        case 6 :
          // jibs and staysails
          if (iSlider == 5)
            {
            iTypeOfSail = 1 ;
            iFirstSailOfType = iFirstForeStaySail ;
            }
          else
            {
            iTypeOfSail = 3 ;
            iFirstSailOfType = iFirstMainStaySail ;
            }
          iSign = Sgn (fSailAngle[iFirstSailOfType]) ;
          if (iSign == 0)
            iSign = Sgn (fAppWindAngle) ;
          switch (LOWORD(wParam)) 
            {
            case SB_LINEUP :
              if ((int) fabs(fSailAngle[iFirstSailOfType]) == 45)
                {
                // sails were eased
                if (iSlider == 5)
                  bEasedJibs = FALSE ;
                else
                  bEasedStaySails = FALSE ;
                for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
                  {
                  if (iSailType[iI] == iTypeOfSail)
                    {
                    if (iSailStatus[iI] == 6)
                      iSailStatus[iI] = 0 ;
                    fSailAngle[iI] = iMaxJibAngle * iSign ;
                    }
                  }
                bSailPlanIsChanged = TRUE ;
                }
              else
                {
                if ((int) fabs(fSailAngle[iFirstSailOfType]) > 0)
                  {
                  for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
                    {
                    if (iSailType[iI] == iTypeOfSail)
                      fSailAngle[iI]=fSailAngle[iI] - (float) iSign ;
                    }
                  }
                }
              break ;
              case SB_LINEDOWN :
              if ((int) fabs(fSailAngle[iFirstSailOfType]) < iMaxJibAngle)
                {
                for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
                  {
                  if (iSailType[iI] == iTypeOfSail)
                    fSailAngle[iI]=fSailAngle[iI] + (float) iSign ;
                  }
                }
              else
                {
                // sail is now eased
                if (iSlider == 5)
                  bEasedJibs = TRUE ;
                else
                  bEasedStaySails = TRUE ;
                for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
                  {
                  if (iSailType[iI] == iTypeOfSail)
                    {
                    if (iSailStatus[iI] == 0)
                      iSailStatus[iI] = 6 ; 
                    fSailAngle[iI] = 45 * iSign ;
                    }
                  }
                bSailPlanIsChanged = TRUE ;
                }
              break ;
            case SB_PAGEUP :
              if ((int) fabs(fSailAngle[iFirstSailOfType]) == 45)
                break;
              if ((int) fabs(fSailAngle[iFirstSailOfType]) > 4)
                {
                for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
                  {
                  if (iSailType[iI] == iTypeOfSail)
                    {
                    fSailAngle[iI]=fSailAngle[iI] - 5 * (float) iSign ;
                    }
                  }
                }
              break ;
            case SB_PAGEDOWN :
              if ((int) fabs(fSailAngle[iFirstSailOfType]) < (iMaxJibAngle-4))
                {
                for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
                  {
                  if (iSailType[iI] == iTypeOfSail)
                    fSailAngle[iI]=fSailAngle[iI] + 5 * (float) iSign ;
                  }       
                }
              break ;
            case SB_THUMBPOSITION :
              if ((int) fabs(fSailAngle[iFirstSailOfType]) == 45)
                break;
              for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
                {
                if (iSailType[iI] == iTypeOfSail)
                  fSailAngle[iI]= iSign * (float) HIWORD(wParam) ;
                }
              break ;
            case SB_THUMBTRACK :
              return ;
            default :
              break ;
            }
          SetScrollPos (hwndSlider[iSlider], SB_CTL, (int) fSailAngle[iFirstSailOfType], TRUE) ;
          if (bRecording)
            Recorder (8, iSlider, fSailAngle[iFirstSailOfType]) ;
          if (iSlider == 5)
            ShowActualJibAngle () ;
          else
            ShowActualStaysailsAngle () ;
          break ;
        case 99 :  
          // window scrollbar
          { 
          int xDelta;     // xDelta = new_pos - current_pos  
          int xNewPos;    // new position 
          int yDelta = 0; 
 
          switch (LOWORD(wParam)) 
            { 
            // User clicked the shaft left of the scroll box. 
            case SB_PAGEUP: 
              xNewPos = xCurrentScroll - 50; 
              break; 
            // User clicked the shaft right of the scroll box. 
            case SB_PAGEDOWN: 
              xNewPos = xCurrentScroll + 50; 
              break; 
            // User clicked the left arrow. 
            case SB_LINEUP: 
              xNewPos = xCurrentScroll - 5; 
              break; 
            // User clicked the right arrow. 
            case SB_LINEDOWN: 
              xNewPos = xCurrentScroll + 5; 
              break; 
            // User dragged the scroll box. 
            case SB_THUMBPOSITION: 
              xNewPos = HIWORD(wParam); 
              break; 
            default: 
              xNewPos = xCurrentScroll; 
            } 
 
          // New position must be between 0 and the screen width 
          xNewPos = max(0, xNewPos); 
          xNewPos = min(xMaxScroll-xClient, xNewPos); 

          // Scroll if the current position changes
          if (xNewPos != xCurrentScroll)
            { 
            bScroll = TRUE; 
            hdc = GetDC (hwnd) ;
            InvalidateRect (hwnd, NULL, TRUE) ;
            ReleaseDC (hwnd, hdc) ;
            }
   
          xCurrentScroll = xNewPos; 
          xScrollPos=xCurrentScroll ;

          // Reset the scroll bar. 
          si.cbSize = sizeof(si); 
          si.fMask  = SIF_POS; 
          si.nPos   = xCurrentScroll; 
          SetScrollInfo(hwnd, SB_HORZ, &si, TRUE); 
          }
        } 
      return 0;

    case WM_VSCROLL: 
      { 
      int xDelta = 0; 
      int yDelta;     // yDelta = new_pos - current_pos 
      int yNewPos;    // new position 
 
      switch (LOWORD(wParam)) 
        { 
        // User clicked the shaft above the scroll box. 
        case SB_PAGEUP: 
          yNewPos = yCurrentScroll - 50; 
          break; 
        // User clicked the shaft below the scroll box. 
        case SB_PAGEDOWN: 
          yNewPos = yCurrentScroll + 50; 
          break; 
        // User clicked the top arrow. 
        case SB_LINEUP: 
          yNewPos = yCurrentScroll - 5; 
          break; 
        // User clicked the bottom arrow. 
        case SB_LINEDOWN: 
          yNewPos = yCurrentScroll + 5; 
          break; 
        // User dragged the scroll box. 
        case SB_THUMBPOSITION: 
          yNewPos = HIWORD(wParam); 
          break; 
        default: 
          yNewPos = yCurrentScroll; 
          } 
 
        // New position must be between 0 and the screen height. 
        GetClientRect (hwndStatus, &rect) ;
        yNewPos = max(0, yNewPos); 
        yNewPos = min(yMaxScroll + (rect.bottom - rect.top) - yClient, yNewPos); 
 
        // Scroll if the current position is changed
        if (yNewPos != yCurrentScroll) 
          { 
          bScroll = TRUE ; 
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
          }
 
        yCurrentScroll = yNewPos; 
        yScrollPos=yCurrentScroll * fMagnify;
 
        // Reset the scroll bar. 
        si.cbSize = sizeof(si); 
        si.fMask  = SIF_POS; 
        si.nPos   = yCurrentScroll; 
        SetScrollInfo(hwnd, SB_VERT, &si, TRUE);

      } 

      return 0;

    case WM_DESTROY :
      if (bRecording)
        EndRecording () ;
      if (bPlayBack)
         EndPlayBack () ;
      KillTimer (hwnd, 1) ;
      KillTimer (hwnd, 2) ;
      DeleteObject (hBlownBrush) ;
      DeleteObject (hDeckBrush) ;
      DeleteObject (hForeAftBrush) ;
      DeleteObject (hForeSailBrush) ;
      DeleteObject (hLandBrush) ;
      DeleteObject (hMainSailBrush) ;
      DeleteObject (hSeaBrush) ;
      DeleteObject (hSkyBrush) ;
      DeleteObject (hStaySailBrush) ;

      DeleteObject (hCoastPen) ;
      DeleteObject (hDarkGrayPen) ;
      DeleteObject (hGrayPen) ;
      DeleteObject (hGreenPen) ;
      DeleteObject (hHullPen) ;
      DeleteObject (hPinkPen) ;
      DeleteObject (hSeaBluePen) ;
      DeleteObject (hSkyBluePen) ;
      DeleteObject (hThickGrayPen) ;
      DeleteObject (hThickGreenPen) ;
      DeleteObject (hThickPinkPen) ;
      DeleteObject (hMastPen) ;
      DeleteObject (hRedPen) ;
      DeleteObject (hYellowPen) ;

      DeleteObject (hBitmap_0) ;
      DeleteObject (hBitmap_1) ;
      DeleteObject (hBitmap_3) ;
      hdc = GetDC (hwnd) ;
      DeleteObject (SelectObject(hdc,GetStockObject(SYSTEM_FONT)));  
      ReleaseDC (hwnd, hdc) ;
      PostQuitMessage (0) ;
      return 0 ;
    }
  return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   AboutDlgProc: About Dialog box callback procedure    
   Version date: 03.09.04
-----------------------------------------*/
BOOL CALLBACK AboutDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  HDC   hdc ;
  int   cxDlg, cyDlg, cxSail, cySail ;
  RECT  rcDlg , rcSail ;

  switch (iMsg)
    {
    case WM_INITDIALOG :
      GetWindowRect (hwndChild[1], &rcSail) ;
      cxSail = rcSail.right - rcSail.left ;
      cySail = rcSail.bottom - rcSail.top ;
      GetWindowRect (hDlg, &rcDlg) ;
      cxDlg = rcDlg.right - rcDlg.left ;
      cyDlg = rcDlg.bottom - rcDlg.top ;
      SendMessage (hDlg, WM_SETICON, (WPARAM)TRUE, (LPARAM)hIcon );
      MoveWindow (hDlg, rcSail.left+((cxSail-cxDlg)/ 2), rcSail.top+((cySail-cyDlg)/ 2), cxDlg, cyDlg, TRUE) ;
      return TRUE ;
    case WM_COMMAND :
      switch (LOWORD (wParam))
        {
        case IDOK :
        case IDCANCEL :
          EndDialog (hDlg, 0) ;
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
          return TRUE ;
        }
      break ;
    }
  return FALSE ;
  }

/*-----------------------------------------
   EditProc: configuration input subclass callback procedure    
   Version date: 03.08.24
-----------------------------------------*/

LRESULT CALLBACK EditProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  int    iI ;

  switch (iMsg)
    {
    case WM_KEYDOWN :
      for (iI=0 ; iI<iFields[iModifyMenu] ; iI++)
        if (GetFocus()==hwndPropText[iI])
          iFocus=iI;
      if (wParam == VK_TAB || wParam == VK_RETURN)
        {
        PropertiesChange() ;
        GetKeyState (VK_SHIFT) < 0 ? iFocus-- : iFocus++ ;
        Focusser() ;
        }
      if (wParam == VK_DOWN)
        {
        PropertiesChange() ;
        iFocus++ ;
        if (!IsWindowEnabled(hwndPropText[iFocus]))
          iFocus++ ;
        Focusser() ;
        }
      if (wParam == VK_UP)
        {
        PropertiesChange() ;
        iFocus-- ;
        if (!IsWindowEnabled(hwndPropText[iFocus]))
          iFocus-- ;
        Focusser() ;
        }
      break ;
    case WM_LBUTTONDOWN :
      if (iModifyMenu==4)
        bMapMustBePainted=TRUE ;
      PropertiesChange() ;
    case WM_SYSCHAR :
      if (wParam==99)
        PropertiesCancel() ; //alt-C
      if (wParam==111)
        {
        PropertiesChange() ;
        PropertiesAccept() ; //alt-O
        }
    }

  return CallWindowProc (fnOldEditProc, hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   HelmDlgProc: Heading dialog box callback procedure    
   Version date: 03.09.04
-----------------------------------------*/
BOOL CALLBACK HelmDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  float fAngle ;
  HDC   hdc ;
  int   cxDlg, cyDlg, cxSail, cySail ;
  RECT  rcDlg , rcSail ;
  int   iTemp ;

  switch (iMsg)
    {
    case WM_INITDIALOG :
      GetWindowRect (hwndChild[1], &rcSail) ;
      cxSail = rcSail.right - rcSail.left ;
      cySail = rcSail.bottom - rcSail.top ;
      GetWindowRect (hDlg, &rcDlg) ;
      cxDlg = rcDlg.right - rcDlg.left ;
      cyDlg = rcDlg.bottom - rcDlg.top ;
      SendMessage (hDlg, WM_SETICON, (WPARAM)TRUE, (LPARAM)hIcon );
      MoveWindow (hDlg, rcSail.left+((cxSail-cxDlg)/ 2), rcSail.top+((cySail-cyDlg)/ 2), cxDlg, cyDlg, TRUE) ;
      iTackStatus = 0 ;
      bHelmsman = FALSE ;
      bEstablished = FALSE ;
      SetDlgItemInt (hDlg, IDD_HEADING, iBearing, FALSE) ;
      return TRUE ;
    case WM_COMMAND :
      switch (LOWORD (wParam))
        {
        case IDD_OK :
          fRequiredHeading = (float) GetDlgItemInt (hDlg, IDD_HEADING, NULL, TRUE) ;
          if ((fRequiredHeading > 360) || (fRequiredHeading< 0))
            {
            fRequiredHeading = iBearing ;
            EndDialog (hDlg, 0) ;
            strcpy(sHelmInstruction, "(await orders)"); 
            SetUpDisplay () ;
            sprintf(sMessage, "Helmsman: await orders"); 
            ShowMessage () ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal heading","Automatic helmsman", MB_OK|MB_ICONERROR) ;
            bHelmsman = FALSE ;
            if (bRecording)
              Recorder (7, 0, 0) ;
            hdc = GetDC (hwnd) ;
            InvalidateRect (hwnd, NULL, TRUE) ;
            ReleaseDC (hwnd, hdc) ;
	    return TRUE ;
            }
          EndDialog (hDlg, 0) ;
          bHelmsman = TRUE ;
          if (bRecording)
            Recorder (7, 1, (int) fRequiredHeading) ;
          bEstablished = FALSE ;
          sprintf(sHelmInstruction, "steer %3.0f    ", fRequiredHeading); 
          SetUpDisplay () ;
          fAngle = fRequiredHeading - (fWindDirection + 180.0) ;
          if (fAngle < -180.0)
            fAngle = fAngle + 360.0 ;
          if (fAngle > 180.0)
            fAngle = fAngle - 360.0 ;
          if (fabs(fAngle) <= 105.0)
            {
            sprintf(sMessage, "Helmsman: steer %3.0f ", fRequiredHeading); 
            ShowMessage () ;
            }
          else
            {
            sprintf(sMessage, "Warning: heading of %3.0f  may be too close to wind", fRequiredHeading); 
            ShowMessage () ;
            }
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
	  return TRUE ;
        case IDD_CANCEL :
          EndDialog (hDlg, 0) ;
          bHelmsman = FALSE ;
          if (bRecording)
            Recorder (7, 0, 0) ;
          strcpy(sHelmInstruction, "(await orders)"); 
          SetUpDisplay () ;
          sprintf(sMessage, "Helmsman: await orders"); 
          ShowMessage () ;
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
          return TRUE ;
        case IDD_DECR_HEADING :
          iTemp = GetDlgItemInt(hDlg, IDD_HEADING, NULL, TRUE)-1 ;
          if (iTemp<0)
            iTemp = 359 ;
          SetDlgItemInt (hDlg, IDD_HEADING, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_INCR_HEADING :
          iTemp = GetDlgItemInt(hDlg, IDD_HEADING, NULL, TRUE)+1 ;
          if (iTemp>360)
            iTemp = 1 ;
          SetDlgItemInt (hDlg, IDD_HEADING, iTemp, FALSE) ;
	  return TRUE ;
        }
     break ;
     }
   return FALSE ;
 }

/*-----------------------------------------
   MapWndProc: Map window callback procedure    
   Version date: 03.09.07
-----------------------------------------*/
LRESULT CALLBACK MapWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  int   xClick ;
  int   yClick ;
  char  sMes[70] ;
  int   iSector ;

  switch (iMsg)
    {
    case WM_MOUSEMOVE :
      // display helpful text in status bar
      xClick = LOWORD (lParam) ;
      yClick = HIWORD (lParam) ;
      iSector = 0 ;
      if (yClick > cxMap/3)
        iSector = 3 ;
      if (yClick > 2 * cxMap/3)
        iSector = 6 ;
      if (xClick > cxMap/3)
        iSector = iSector+1 ;
      if (xClick > 2 * cxMap/3)
        iSector = iSector+1 ;
      LoadString (hInstance, IDS_MAP+iSector, sMessage,120) ;
      if (iModifyMenu == 4)
        {
        fNewLong = fLeftEdge + xClick*fMapWidth/cxMap ;
        fNewLat =  fTopEdge - yClick*fMapWidth/cxMap ;
        sprintf(sMes, "; right-click to place island at N Lat = %5.3f E Long = %5.3f", fNewLat, fNewLong) ;
        strcat(sMessage,sMes) ;
        }
      ShowStatusMessage() ;
      return 0 ;
    case WM_LBUTTONDOWN :
      xClick=LOWORD (lParam) ;
      yClick=HIWORD (lParam) ;
      if (xClick < cxMap/3)
        {
        fLeftEdge = fLeftEdge - fMapRadius ;
        fRightEdge = fRightEdge - fMapRadius ;
        bTracking = FALSE ;
        bMapMustBePainted = TRUE ;
        SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0) ;
        } 
      if (xClick > 2*cxMap/3)
        {
        fLeftEdge = fLeftEdge + fMapRadius ;
        fRightEdge = fRightEdge + fMapRadius ;
        bTracking = FALSE ;
        bMapMustBePainted = TRUE ;
        SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0) ;
        } 
      if (yClick < cxMap/3)
        {
        fBottomEdge = fBottomEdge + fMapRadius ;
        fTopEdge = fTopEdge + fMapRadius ;
        bTracking = FALSE ;
        bMapMustBePainted = TRUE ;
        SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0) ;
        } 
      if (yClick > 2*cxMap/3)
        {
        fBottomEdge = fBottomEdge - fMapRadius ;
        fTopEdge = fTopEdge - fMapRadius ;
        bTracking = FALSE ;
        bMapMustBePainted = TRUE ;
        SendMessage (hwndCheck[2], BM_SETCHECK, 0, 0) ;
        } 
      if (iModifyMenu == 4)
        SetFocus(hwndPropText[(iFocus/3)*3]) ; 
      return 0 ;  
    case WM_RBUTTONDOWN :
      if (iModifyMenu == 4)
        {
        // move selected island to this position
        xClick=LOWORD (lParam) ;
        yClick=HIWORD (lParam) ;
        fIsleLat[iFocus/3] = fNewLat ;
        fIsleLong[iFocus/3] = fNewLong ;
        sprintf(sMessage, "%1.4f", fIsleLat[iFocus/3]) ;
        SetWindowText (hwndPropText[(iFocus/3)*3],sMessage) ; 
        sprintf(sMessage, "%1.4f", fIsleLong[iFocus/3]) ;
        SetWindowText (hwndPropText[1+(iFocus/3)*3],sMessage) ; 
        SetFocus(hwndPropText[(iFocus/3)*3]) ; 
        bMapMustBePainted = TRUE ;
        }  
       return 0 ;
    }
  return DefWindowProc(hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   PropsProc: Properties adjustment callback procedure    
   Version date: 03.09.10
-----------------------------------------*/
LRESULT CALLBACK PropsProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  HDC          hdc ;
  PAINTSTRUCT  ps ;

  switch (iMsg)     
    {
    case WM_PAINT :
      hdc = BeginPaint (hwndProps, &ps) ;
      EndPaint (hwndProps, &ps) ;
      PropertiesPaint() ;
      return 0;  
    case WM_COMMAND :
      switch (LOWORD (wParam))
        {
        case IDC_PROP_OK :
          PropertiesChange() ;
          PropertiesAccept () ;
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
          return 0 ;
        case IDC_PROP_CANCEL :
          PropertiesCancel () ;
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
          return 0 ;
        }
    case WM_MOUSEMOVE :
      // display default text in status bar
      //if (bBrig)
      //  LoadString (hInstance, IDS_DEFAULT_BRIG, sMessage,120) ;
      //else
      //  LoadString (hInstance, IDS_DEFAULT_FRIGATE, sMessage,120) ;
      LoadString (hInstance, IDS_DEFAULT, sMessage,120) ;
      ShowStatusMessage() ;
      return 0 ;
    }
    return DefWindowProc(hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   PlanWndProc: Planview callback procedure    
   Version date: 03.02.20
-----------------------------------------*/

LRESULT CALLBACK PlanWndProc (HWND hwnd2, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  float  fTemp ;
  HDC    hdc ;
  int    xPos, yPos ;

  switch (iMsg) 
    {
    case WM_LBUTTONDOWN :
      if (iBearing > -1)
        DialogBox (hInstance, "HelmBox", hwnd, HelmDlgProc) ;
      return 0 ;
    case WM_RBUTTONDOWN :
      DialogBox (hInstance, "WindBox", hwnd, WindDlgProc) ;
      return 0 ;
    case WM_MOUSEMOVE :
      xPos = -((cxChildPixels/2.) - LOWORD(lParam)) ;  // horizontal position of cursor
      yPos = HIWORD(lParam);  // vertical position of cursor 
      if (yPos < (cxChildPixels/2))
        yPos = -(yPos-(cxChildPixels/2)) ;
      else
        yPos = (cxChildPixels/2)-yPos ;
      fTemp = ((float) yPos) /xPos ;
      iBearing = (atan(fTemp) / fDegToRad) ;
      if (xPos >= 0 )
        iBearing = 90 - iBearing ;
      if (xPos < 0 )
        iBearing = 270 - iBearing ;
      if (xPos == 0 && yPos >0)
        iBearing = 0 ;
      if (xPos == 0 && yPos <0)
        iBearing = 180 ;
      hdc = GetDC (hwnd) ;
      SelectObject(hdc,GetStockObject(NULL_PEN)) ;
      Rectangle (hdc, xBearing+2, yBearing, xBearing - 25, yBearing + 12) ;
      if (sqrt ((xPos*xPos)+(yPos*yPos)) < cxChildPixels/4)
        { 
        iBearing = -fWindDirection ;
        // display helpful text in status bar
        LoadString (hInstance, IDS_PLANVIEW_CENT, sMessage,120) ;
        ShowStatusMessage() ;
        }
      else
        { 
        iBearing = iBearing + (int) fOffsetAngle ;
        if (iBearing > 360)
          iBearing = iBearing - 360 ;
        SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
        SetTextAlign (hdc, TA_RIGHT|TA_TOP) ;
        TextOut (hdc, xBearing, yBearing, sMessage, sprintf(sMessage, "%3d ", iBearing)); 
        SetTextAlign (hdc, TA_LEFT|TA_TOP) ;
        fBearingTime = (float) clock()/CLOCKS_PER_SEC ;
        // display helpful text in status bar
        LoadString (hInstance, IDS_PLANVIEW_EDGE, sMessage,120) ;
        ShowStatusMessage() ;
        }
      ReleaseDC (hwnd, hdc) ;
      return 0 ;
    default :
      return DefWindowProc(hwnd2, iMsg, wParam, lParam) ;
    } 
  }

/*-----------------------------------------
   SailWndProc: Sailplan callback procedure    
   Version date: 03.09.13
-----------------------------------------*/
LRESULT CALLBACK SailWndProc (HWND hwnd2, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {

  switch (iMsg)
    {
    case WM_MOUSEMOVE :
      // first display helpful text in status bar
      if (bParalax)
        LoadString (hInstance, IDS_SAILPLAN_P, sMessage,120) ;
      else
        if (HIWORD(lParam)>0.85*cxChildPixels)  // cursor is underwater 
          LoadString (hInstance, IDS_SAILPLAN_U, sMessage,120) ;
        else
          LoadString (hInstance, IDS_SAILPLAN_C, sMessage,120) ;
      ShowStatusMessage() ;
      return 0 ;
    case WM_LBUTTONDOWN :
      if (bParalax)
        {
        LoadString (hInstance, IDS_SAILPLAN_C, sMessage,120) ;
        ShowStatusMessage() ;
        bParalax = FALSE ;

        CheckMenuItem(hMenu, IDM_CONVENTIONAL, MF_CHECKED) ;
        CheckMenuItem(hMenu, IDM_PERSPECTIVE, MF_UNCHECKED) ;
        bSailPlanIsChanged = TRUE ;
        return 0 ;
        }

      if (HIWORD(lParam)>0.85*cxChildPixels)  // clicked underwater 
        {
        CheckMenuItem(hMenu, IDM_CONVENTIONAL, MF_UNCHECKED) ;
        CheckMenuItem(hMenu, IDM_PERSPECTIVE, MF_CHECKED) ;
        } 

      if (iClicks < 9)
        {
        iClicks = iClicks++ ;
        xClicks[iClicks] = LOWORD (lParam) ;
        yClicks[iClicks] = HIWORD (lParam) ;
        }
      else
        {
        strcpy (sMessage,"Yardsmen overworked") ;
        ShowMessage () ;
        } 
      return 0;  

    }
  return DefWindowProc(hwnd2, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   StatusMessage: Store message for status bar    
   Version date: 03.10.09
-----------------------------------------*/

LRESULT StatusMessage (HWND hwnd, WPARAM wParam, LPARAM lParam)
  {
  UINT       fuFlags = (UINT) HIWORD (wParam) ;
  
  LoadString (hInstance, (UINT) LOWORD(wParam), sMessage,120) ;

  if (fuFlags == 0xFFFF)  // menu closed
    { 
    //if (bBrig)
    //  LoadString (hInstance, IDS_DEFAULT_BRIG, sMessage,120) ;
    //else
    //  LoadString (hInstance, IDS_DEFAULT_FRIGATE, sMessage,120) ;
    LoadString (hInstance, IDS_DEFAULT, sMessage,120) ;
    }
  // display saved status message
  ShowStatusMessage() ;
  return 0 ;
  }

/*-----------------------------------------
   SternWndProc: Stern view callback procedure    
   Version date: 99.06.20
-----------------------------------------*/
LRESULT CALLBACK SternWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  return DefWindowProc(hwnd, iMsg, wParam, lParam) ;
  }

/*-----------------------------------------
   WindDlgProc: Wind velocity/direction dialog box callback procedure    
   Version date: 03.09.04
-----------------------------------------*/
BOOL CALLBACK WindDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
  {
  HDC   hdc ;
  int   cxDlg, cyDlg, cxSail, cySail ;
  RECT  rcDlg , rcSail ;
  int   iTemp ;

  switch (iMsg)
    {
    case WM_INITDIALOG :
      GetWindowRect (hwndChild[1], &rcSail) ;
      cxSail = rcSail.right - rcSail.left ;
      cySail = rcSail.bottom - rcSail.top ;
      GetWindowRect (hDlg, &rcDlg) ;
      cxDlg = rcDlg.right - rcDlg.left ;
      cyDlg = rcDlg.bottom - rcDlg.top ;
      MoveWindow (hDlg, rcSail.left+((cxSail-cxDlg)/ 2), rcSail.top+((cySail-cyDlg)/ 2), cxDlg, cyDlg, TRUE) ;
      SendMessage (hDlg, WM_SETICON, (WPARAM)TRUE, (LPARAM)hIcon );
      if (iBearing < 0)
        iBearing= -iBearing ;
      SetDlgItemInt (hDlg, IDD_BEARING, iBearing, FALSE) ;
      SetDlgItemInt (hDlg, IDD_B_RATE, iWindDirIncr, FALSE) ;
      SetDlgItemInt (hDlg, IDD_SPEED, (int) fWindSpeed, FALSE) ;
      SetDlgItemInt (hDlg, IDD_S_RATE, iWindSpeedIncr, FALSE) ;
      return TRUE ;
    case WM_COMMAND :
      switch (LOWORD (wParam))
        {
        case IDD_OK :
          fNewWindDirection = (float) GetDlgItemInt (hDlg, IDD_BEARING, NULL, TRUE) ;
          if ((fNewWindDirection > 360) || (fNewWindDirection< 0))
            {
            fNewWindDirection = fWindDirection ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal wind direction","Wind direction", MB_OK|MB_ICONERROR) ;
            }
          iWindDirIncr= GetDlgItemInt (hDlg, IDD_B_RATE, NULL, TRUE) ;
          if ((iWindDirIncr > 256) || (iWindDirIncr< 1))
            {
            iWindDirIncr = 128 ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal wind direction change rate","Wind direction", MB_OK|MB_ICONERROR) ;
            }
          fNewWindSpeed = (float) GetDlgItemInt (hDlg, IDD_SPEED, NULL, TRUE) ;
          if ((fNewWindSpeed > 70) || (fNewWindSpeed< 0))
            {
            fNewWindSpeed = fWindSpeed ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal wind speed","Wind speed", MB_OK|MB_ICONERROR) ;
            }
          iWindSpeedIncr= GetDlgItemInt (hDlg, IDD_S_RATE, NULL, TRUE) ;
          if ((iWindSpeedIncr > 256) || (iWindSpeedIncr< 1))
            {
            iWindSpeedIncr = 128 ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            MessageBox (hwnd,"Illegal wind speed change rate","Wind speed", MB_OK|MB_ICONERROR) ;
            }
          EndDialog (hDlg, 0) ;
          if (fNewWindDirection!=fWindDirection)
            {
            sprintf(sMessage, "Wind direction changing to %3.0f  at %3d /min", fNewWindDirection, iWindDirIncr); 
            ShowMessage () ;
            fDeltaWindDir = iWindDirIncr/120. ;
            bWindDirChange = TRUE ;
            if (bRecording)
              {
              Recorder (12, 1, (int) fNewWindDirection) ;
              Recorder (12, 2, iWindDirIncr) ;
              }
            }
          if (fNewWindSpeed!=fWindSpeed)
            {
            sprintf(sMessage, "Wind speed changing to %3.0f kts at %3d kts/min", fNewWindSpeed, iWindSpeedIncr); 
            ShowMessage () ;
            fDeltaWindSpeed = iWindSpeedIncr/120. ;
            bWindSpeedChange = TRUE ;
            if (bRecording)
              {
              Recorder (13, 1, (int) fNewWindSpeed) ;
              Recorder (13, 2, iWindSpeedIncr) ;
              }
            }
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
	  return TRUE ;
	case IDD_CANCEL :
          EndDialog (hDlg, 0) ;
          strcpy(sMessage, "No wind change"); 
          ShowMessage () ;
          hdc = GetDC (hwnd) ;
          InvalidateRect (hwnd, NULL, TRUE) ;
          ReleaseDC (hwnd, hdc) ;
	  return TRUE ;
	case IDD_DECR_BEARING :
          iTemp = GetDlgItemInt(hDlg, IDD_BEARING, NULL, TRUE)-1 ;
          if (iTemp<0)
            iTemp = 359 ;
          SetDlgItemInt (hDlg, IDD_BEARING, iTemp, FALSE) ;
	  return TRUE ;
	case IDD_INCR_BEARING :
          iTemp = GetDlgItemInt(hDlg, IDD_BEARING, NULL, TRUE)+1 ;
          if (iTemp>360)
            iTemp = 1 ;
          SetDlgItemInt (hDlg, IDD_BEARING, iTemp, FALSE) ;
	  return TRUE ;
	case IDD_DECR_B_RATE :
          iTemp = GetDlgItemInt(hDlg, IDD_B_RATE, NULL, TRUE)/2 ;
          if (iTemp<1)
            {
            MessageBeep (MB_ICONEXCLAMATION) ;	
            iTemp = 1 ;
            }
          SetDlgItemInt (hDlg, IDD_B_RATE, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_INCR_B_RATE :
          iTemp = GetDlgItemInt(hDlg, IDD_B_RATE, NULL, TRUE)*2 ;
          if (iTemp>128)
            {
            MessageBeep (MB_ICONEXCLAMATION) ;	
            iTemp = 128 ;
            }
          SetDlgItemInt (hDlg, IDD_B_RATE, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_DECR_SPEED :
          iTemp = GetDlgItemInt(hDlg, IDD_SPEED, NULL, TRUE)-1 ;
          if (iTemp<0)
            {
            iTemp = 0 ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            }
          SetDlgItemInt (hDlg, IDD_SPEED, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_INCR_SPEED :
          iTemp = GetDlgItemInt(hDlg, IDD_SPEED, NULL, TRUE)+1 ;
          if (iTemp>70)
            {
            iTemp = 70 ;
            MessageBeep (MB_ICONEXCLAMATION) ;	
            }
          SetDlgItemInt (hDlg, IDD_SPEED, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_DECR_S_RATE :
          iTemp = GetDlgItemInt(hDlg, IDD_S_RATE, NULL, TRUE)/2 ;
          if (iTemp<1)
            {
            MessageBeep (MB_ICONEXCLAMATION) ;	
            iTemp = 1 ;
            }
          SetDlgItemInt (hDlg, IDD_S_RATE, iTemp, FALSE) ;
	  return TRUE ;
        case IDD_INCR_S_RATE :
          iTemp = GetDlgItemInt(hDlg, IDD_S_RATE, NULL, TRUE)*2 ;
          if (iTemp>128)
            {
            MessageBeep (MB_ICONEXCLAMATION) ;	
            iTemp = 128 ;
            }
          SetDlgItemInt (hDlg, IDD_S_RATE, iTemp, FALSE) ;
	  return TRUE ;
        }
    break ;
    }
  return FALSE ;
  }

/*-----------------------------------------
   Timer1Proc: Screen update timer callback procedure (called every 460 mSec)   
   Version date: 03.07.18
-----------------------------------------*/
VOID CALLBACK Timer1Proc (HWND hwnd, UINT iMsg, UINT iTimerID, DWORD dwTime)
  {
  HDC hdc ;

  if (bPaused)
    return ;
  if (bScroll)
    {
    hdc = GetDC (hwnd) ;
    SetWindowOrgEx (hdc, xScrollPos*fMagnify, yScrollPos*fMagnify, NULL);
    InvalidateRect (hwnd, NULL, TRUE) ;
    ReleaseDC (hwnd, hdc) ;
    bScroll = FALSE ;
    SetUpDisplay() ;
    }    
  if (bRecording)
    {
    iTick = iTick + 1 ;
    if (iTick > 20)
      {
      // save position every 10 seconds
      iTick = 0 ;
      Recorder (4, floor(fLatitude), (fLatitude-floor(fLatitude))*100000) ;
      Recorder (5, floor(fLongitude), (fLongitude-floor(fLongitude))*100000) ;
      }
    }
  CycleEvents () ;
  if (!bScroll)
    ShowParameters () ;
  if (bSailPlanIsChanged || (bPlanViewIsChanged && bParalax))
    SailPlan() ;
  if (bPlanViewIsChanged)
    PlanView(fOffsetAngle) ;
  Map () ;
  if (bSternViewIsChanged)
    SternView() ;
  return ;
  }

/*-----------------------------------------
   Timer2Proc: Ship state calculation timer callback procedure (called every 5 mSec)
   Version date: 99.10.23 
-----------------------------------------*/
VOID CALLBACK Timer2Proc (HWND hwnd, UINT iMsg, UINT iTimerID, DWORD dwTime)
  {
  if (bPaused)
    return ;
  Calculate () ;
  }
