/*-----------------------------------------
   MAP.C -- Map procedures module
   Version date: 03.04.05
-------------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "external.h"

// Prototypes:
void  Map () ;
void  ShowMapWidth() ;
void  ZoomIn () ;
void  ZoomOut () ;

/*-----------------------------------------
   Map: Updates map    
   Version date: 03.04.05
-----------------------------------------*/
void Map() 
  {
  float  fCentreLon, fCentreLat ;
  float  fDist ;
  float  fMapScale ;
  SIZE   size ;
  char   cText[25] ;
  HDC    hdc ;
  int    iI ;

  fMapScale = 1000./fMapWidth ;

  // check for edge of map if map is tracking ship
  if (bTracking && ((fLongitude > fRightEdge-fOneTenth) || (fLongitude < fLeftEdge+fOneTenth) ||
                    (fLatitude > fTopEdge-fOneTenth) || (fLatitude < fBottomEdge+fOneTenth)))
    bMapMustBePainted = TRUE ;

  if (bMapMustBePainted)
    { 
    if (bTracking)
      {
      fLeftEdge = 0.0 ;
      fRightEdge = fMapWidth ;
      fBottomEdge = 0.0 ;
      fTopEdge = fMapWidth ;
      while (fLongitude > fRightEdge-fOneTenth)
        {
        fLeftEdge = fLeftEdge + fMapRadius ;
        fRightEdge = fRightEdge + fMapRadius ;
        }
      while (fLongitude < (fLeftEdge+fOneTenth))
        {
        fRightEdge = fRightEdge - fMapRadius ;
        fLeftEdge = fLeftEdge - fMapRadius ;
        }
      while (fLatitude > fTopEdge-fOneTenth)
        {
        fBottomEdge = fBottomEdge + fMapRadius ;
        fTopEdge = fTopEdge + fMapRadius ;
        }
      while (fLatitude < (fBottomEdge+fOneTenth))
        {
        fTopEdge = fTopEdge - fMapRadius ;
        fBottomEdge = fBottomEdge - fMapRadius ;
        }
      }
     
    // show coordinates of north-west and south=east corners
    hdc = GetDC (hwnd) ;
    if (fTopEdge >=0)
      {
      if (fLeftEdge >= 0)
        sprintf(cText, "%4.2f nm N, %4.2f nm E", fabs(fTopEdge), fabs(fLeftEdge)) ; 
      else
        sprintf(cText, "%4.2f nm N, %4.2f nm W", fabs(fTopEdge), fabs(fLeftEdge)) ; 
      }
    else
      {
      if (fLeftEdge >= 0)
        sprintf(cText, "%4.2f nm S, %4.2f nm E", fabs(fTopEdge), fabs(fLeftEdge)) ; 
      else
        sprintf(cText, "%4.2f nm S, %4.2f nm W", fabs(fTopEdge), fabs(fLeftEdge)) ; 
      }
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, 535, 4, 660, 18) ;
    SelectObject(hdc, hSkyBrush) ;
    SetBkMode(hdc, TRANSPARENT) ;
    GetTextExtentPoint32(hdc,cText, strlen(cText),&size) ; 
    Rectangle (hdc, 535, 4, 539 + size.cx, 18) ;
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    TextOut (hdc, 536, 4, cText, strlen(cText)) ;
    if (fBottomEdge >=0)
      {
      if (fRightEdge >= 0)
        sprintf(cText, "%4.2f nm N, %4.2f nm E", fabs(fBottomEdge), fabs(fRightEdge)) ; 
      else
        sprintf(cText, "%4.2f nm N, %4.2f nm W", fabs(fBottomEdge), fabs(fRightEdge)) ; 
      }
    else
      {
      if (fRightEdge >= 0)
        sprintf(cText, "%4.2f nm S, %4.2f nm E", fabs(fBottomEdge), fabs(fRightEdge)) ; 
      else
        sprintf(cText, "%4.2f nm S, %4.2f nm W", fabs(fBottomEdge), fabs(fRightEdge)) ; 
      }
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    SelectObject(hdc, GetStockObject(LTGRAY_BRUSH)) ;
    Rectangle (hdc, 630, 245, 756, 258) ;
    SelectObject(hdc, hSkyBrush) ;
    SetBkMode(hdc, TRANSPARENT) ;
    GetTextExtentPoint32(hdc,cText, strlen(cText),&size) ; 
    Rectangle (hdc, 750-size.cx, 245, 754, 258) ;
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
    TextOut (hdc, 752, 245, cText, strlen(cText)) ;
    SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
    SetBkMode(hdc, OPAQUE) ;
    SelectObject(hdc, GetStockObject(LTGRAY_BRUSH)) ;
    ReleaseDC (hwnd, hdc) ; 

    // position map
    hdc = GetDC (hwndChild[2]) ;
    SetMapMode (hdc, MM_ANISOTROPIC) ;
    SetWindowExtEx (hdc, 1000, 1000, NULL) ;
    SetViewportExtEx (hdc, cxMap, -cxMap, NULL) ;
    SetViewportOrgEx (hdc, 0, cxMap, NULL) ;
    // (0,0) = bottom left

    SelectObject (hdc, (HPEN) hSeaBluePen) ; //sea blue
    SelectObject (hdc, hSeaBrush) ;
    Rectangle (hdc, 0, 0, 1000, 1000) ;
    SelectObject (hdc,GetStockObject(BLACK_PEN)) ; 

    // draw islands
    fCentreLon = fRightEdge - fMapRadius ;
    fCentreLat = fTopEdge - fMapRadius ;
    SelectObject(hdc, (HPEN) hCoastPen) ;
    if (iModifyMenu == 4)
      {
      // determine which island is selected
      for (iI=0 ; iI<iFields[iModifyMenu] ; iI++)
        if (GetFocus()==hwndPropText[iI])
          iFocus=iI;
      }
    for (iI = 0 ; iI <= 12 ; iI++)
      {
      if (fIsleRadius[iI] > 0.00001)
        {
        if ((iModifyMenu == 4) && (iI == (int) (iFocus)/3))
          {
          // this island is to be modified (draw dark red)
          SelectObject(hdc, hBlownBrush) ;
          }
        else  
          SelectObject(hdc, hLandBrush) ;  // draw green
        fDist = sqrt((fCentreLon - fIsleLong[iI]) * (fCentreLon - fIsleLong[iI]) + 
          (fCentreLat - fIsleLat[iI]) * (fCentreLat - fIsleLat[iI])) ;
        if (fIsleRadius[iI] > (fDist + 2.0 * fMapRadius))
          Rectangle (hdc, fLeftEdge, fBottomEdge, fRightEdge, fTopEdge) ;
        else
          Ellipse (hdc, 
            ((fIsleLong[iI]-fIsleRadius[iI])-fLeftEdge)*fMapScale, 
            ((fIsleLat[iI]+fIsleRadius[iI])-fBottomEdge)*fMapScale, 
            ((fIsleLong[iI]+fIsleRadius[iI])-fLeftEdge)*fMapScale, 
            ((fIsleLat[iI]-fIsleRadius[iI])-fBottomEdge)*fMapScale ) ;
        } 
      }

    //redraw track
    SetPixel (hdc, (int) ((fLon[0]-fLeftEdge)*fMapScale), (int) ((fLat[0]-fBottomEdge)*fMapScale), RGB(170,170,170)) ;
    SelectObject(hdc, hGrayPen) ;
    MoveToEx (hdc, (int) ((fLon[0]-fLeftEdge)*fMapScale), (int) ((fLat[0]-fBottomEdge)*fMapScale), NULL) ;
    for (iI = 1 ; iI < iMapCounter ; iI++)
      LineTo (hdc, (int) ((fLon[iI]-fLeftEdge)*fMapScale), (int) ((fLat[iI]-fBottomEdge)*fMapScale)) ;
    LineTo (hdc, (int) ((fLastLong-fLeftEdge)*fMapScale), (int) ((fLastLat-fBottomEdge)*fMapScale)) ;
    } 
  else
    {
    // map has not been redrawn
    hdc = GetDC (hwndChild[2]) ;
    SetMapMode (hdc, MM_ANISOTROPIC) ;
    SetWindowExtEx (hdc, 1000, 1000, NULL) ;
    SetViewportExtEx (hdc, cxMap, -cxMap, NULL) ;
    SetViewportOrgEx (hdc, 0, cxMap, NULL) ;
    }
  
  // draw new position
  SelectObject(hdc, hGrayPen) ;
  MoveToEx (hdc, (int) ((fLastLong-fLeftEdge)*fMapScale), (int) ((fLastLat-fBottomEdge)*fMapScale), NULL) ;
  LineTo (hdc, (int) ((fLongitude-fLeftEdge)*fMapScale), (int) ((fLatitude-fBottomEdge)*fMapScale)) ;
  ReleaseDC (hwndChild[2], hdc) ;
  fLastLat = fLatitude ;
  fLastLong = fLongitude ;

  bMapMustBePainted = FALSE ;

  }

/*-----------------------------------------
   ShowMapWidth
   Version date: 99.10.03
-----------------------------------------*/

void ShowMapWidth()
  {
  HDC    hdc ;
  int    xThis, yThis ;

  hdc = GetDC (hwnd) ;
  xThis = 760 ;
  yThis = 115 ;
  if (fMapWidth>10)
    xThis=xThis - 5 ; 
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, 755, yThis, 790, yThis + cyTextHeight) ;
  TextOut (hdc, xThis, yThis, sMessage, sprintf(sMessage, "%4.2f", fMapWidth)) ; 
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  ReleaseDC (hwnd, hdc) ;
  }

/*-----------------------------------------
   ZoomIn: Zooms map in    
   Version date: 03.01.26
-----------------------------------------*/

void ZoomIn() 
  {
  
  if (fMapWidth > 0.04)
    {
    fMapWidth = fMapWidth / 5.0 ;
    fMapRadius = fMapWidth / 2.0 ;
    fOneTenth = 0.2 * fMapRadius ;
    ShowMapWidth() ;
    bMapMustBePainted = TRUE ;
    if (!bTracking)
      {
      fLeftEdge = fLeftEdge + 2 * fMapWidth ;
      fBottomEdge = fBottomEdge + 2 * fMapWidth ;
      fRightEdge = fLeftEdge + fMapWidth ;
      fTopEdge = fBottomEdge + fMapWidth ;
      }
    }

  EnableWindow (hwndButton[1], TRUE) ; //zoom out enabled
  if (fMapWidth < 0.1)
    EnableWindow (hwndButton[0], FALSE) ; //zoom in disabled
  SetFocus (hwndCheck[1]) ;

  }

/*-----------------------------------------
   ZoomOut: Zooms map out    
   Version date: 03.01.26
-----------------------------------------*/

void ZoomOut() 
  {
  
  if (fMapWidth < 20.0)
    {
    fMapWidth = fMapWidth * 5.0 ;
    fMapRadius = fMapWidth / 2.0 ;
    fOneTenth = 0.2 * fMapRadius ;
    ShowMapWidth() ;
    bMapMustBePainted = TRUE ;
    if (!bTracking)
      {
      fLeftEdge = fLeftEdge - 0.4 * fMapWidth ;
      fBottomEdge = fBottomEdge - 0.4 * fMapWidth ;
      fRightEdge = fLeftEdge + fMapWidth ;
      fTopEdge = fBottomEdge + fMapWidth ;
      }
    }

  EnableWindow (hwndButton[0], TRUE) ; //zoom in enabled
  if (fMapWidth > 15.)
    EnableWindow (hwndButton[1], FALSE) ; //zoom out disabled
  SetFocus (hwndCheck[1]) ;

  }
