/* --------------------------------------
   Procedure SailPlan
   Version date = 00.07.16
----------------------------------------*/

#include <windows.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "external.h"

#define PI        3.1415927
#define DEGTORAD  PI/180.0

// Prototypes:
void  CheckJibStatus() ;
void  DrawMast(HDC, int, float) ;
void  DrawSail(HDC, char) ;
void  JibChange (int) ;
void  JibSelect () ;
void  SailPlan () ;
void  ShowBow (HDC, float) ;
void  ShowStern (HDC, float) ;

extern int   Sgn (float) ;
extern void  ShowActualJibAngle () ;

/*-----------------------------------------
   CheckJibStatus
   Version date: 99.06.28
  -----------------------------------------*/
void CheckJibStatus()
  {
  
  if (iSailStatus[12] == 4)
    {
    // jib is furled; can change jib
    EnableWindow (hwndJib, FALSE) ;
    switch (iJibSet)
      {
      case 0 :
        SetWindowText(hwndButton[6], "Working") ;
        SetWindowText(hwndButton[7], "Storm") ;
        break ;
      case 1 :
        SetWindowText(hwndButton[6], "Storm") ;
        SetWindowText(hwndButton[7], "Large") ;
        break ;
      case 2 :
        SetWindowText(hwndButton[6], "Large") ;
        SetWindowText(hwndButton[7], "Working") ;
      }
    EnableWindow (hwndButton[6], TRUE) ;
    EnableWindow (hwndButton[7], TRUE) ;
    }  
  else
    {
    EnableWindow (hwndJib, TRUE) ;
    EnableWindow (hwndButton[6], FALSE) ;
    EnableWindow (hwndButton[7], FALSE) ;
    }
  ShowActualJibAngle () ;
  }

/* ------------------------------
     DrawMast
     Version date: 99.09.19
-------------------------------*/

void DrawMast(HDC hdcMem, int iMast, float fFactor)
  {
  SelectObject (hdcMem, (HPEN) CreatePen(PS_SOLID, 6, RGB(0,0,0))) ;
  MoveToEx (hdcMem, fMastDistance[iMast]*fSailScale*fFactor,fGunDeckHeight*fSailScale, NULL) ;
  LineTo (hdcMem, fMastDistance[iMast]*fSailScale*fFactor,fMastHeight[iMast]*fSailScale) ;
  DeleteObject(SelectObject(hdcMem,GetStockObject(BLACK_PEN))) ; 
  }
 
/* ------------------------------
     DrawSail
     Version date: 99.06.02
-------------------------------*/

void DrawSail(HDC hdc, char cSl)
  {
  float   fBandBase, fBandTop ;
  float   fBotLeft, fBotRight ;
  int     iJ ;
  int     iNrBands ;
  float   fOfsetX ;
  int     iSlStatus ;
  float   fSlantLeft ;
  float   fSlantRight ;
  float   fTemp ;
  float   fTopLeft, fTopRight ;

  iSlStatus = iSailStatus[cSl] ;
  fSlantLeft = (fPlanFootLeft[cSl] - fPlanHeadLeft[cSl]) / (fSailHead[cSl] - fSailFoot[cSl]) ;
  fSlantRight = (fPlanFootRight[cSl] - fPlanHeadRight[cSl]) / (fSailHead[cSl] - fSailFoot[cSl]) ;
  iNrBands = iMaxReefs[cSl] + 1 ;

  fOfsetX = fMastDistance[0] ;
  if (cSl > 3)
    fOfsetX = fMastDistance[1] ;
  if (cSl > 7)
    fOfsetX = fMastDistance[2] ;
  if (cSl == 12)
    fOfsetX = fMastDistance[3] ;

  fBandTop = fSailHead[cSl] ;
  for (iJ=1; iJ <= iNrBands ; iJ++)
    {
    if (iJ == iNrBands) 
      fBandBase = fSailFoot[cSl] ;
    else
      fBandBase = fSailFoot[cSl] + (fSailHead[cSl] - fSailFoot[cSl]) * fSetFraction[iJ] ;
    switch (iSlStatus)
      {
      case 0:
        {
        // Set
        SelectObject(hdc, (HBRUSH) GetStockObject(LTGRAY_BRUSH)) ;
        if (cSl > 10)
          SelectObject(hdc, (HBRUSH) GetStockObject(GRAY_BRUSH)) ;
        SelectObject(hdc, (HPEN) GetStockObject(BLACK_PEN)) ;
        break ;
        }
      case 4:
        // Furled
        {
        SelectObject(hdc, (HBRUSH) GetStockObject(NULL_BRUSH)) ;
        SelectObject(hdc, (HPEN) GetStockObject(WHITE_PEN)) ;
        break ;
        }
      case 5:
        // Blown out
        {
        SelectObject(hdc, (HBRUSH) hBlownBrush) ;
        SelectObject(hdc, (HPEN) GetStockObject(BLACK_PEN)) ;
        break ;
        } 
      default :
        // Reefed
        {
        if (iJ < (iSlStatus+1))
          {
          // this band furled
          SelectObject(hdc, (HBRUSH) GetStockObject(NULL_BRUSH)) ;
          SelectObject(hdc, (HPEN) GetStockObject(WHITE_PEN)) ;
          }
        else
          {
          // this band set
          SelectObject(hdc, (HBRUSH) GetStockObject(LTGRAY_BRUSH)) ;
          if (cSl > 10)
            SelectObject(hdc, (HBRUSH) GetStockObject(GRAY_BRUSH)) ;
          SelectObject(hdc, (HPEN) GetStockObject(BLACK_PEN)) ;
          }
        }
      } 
 
    fTopLeft = fOfsetX + fPlanHeadLeft[cSl] + (fSlantLeft * (fSailHead[cSl]-fBandTop)) ;
    fTopRight = fOfsetX + fPlanHeadRight[cSl] + (fSlantRight * (fSailHead[cSl]-fBandTop)) ;
    fBotRight = fOfsetX + fPlanHeadRight[cSl] + (fSlantRight * (fSailHead[cSl]-fBandBase)) ;
    fBotLeft = fOfsetX + fPlanHeadLeft[cSl] + (fSlantLeft * (fSailHead[cSl]-fBandBase)) ;
    BeginPath(hdc) ;
    MoveToEx (hdc, fTopLeft*fSailScale,fBandTop*fSailScale,NULL) ;
    LineTo (hdc, fTopRight*fSailScale,fBandTop*fSailScale) ;
    LineTo (hdc, fBotRight*fSailScale,fBandBase*fSailScale) ;
    LineTo (hdc, fBotLeft*fSailScale,fBandBase*fSailScale) ;
    CloseFigure(hdc) ;
    EndPath(hdc) ;
    StrokeAndFillPath(hdc) ;
    fBandTop = fBandBase ;
    }
  // calculate Sailplan positions of corners of effective sail
  if (fEffSailHead[cSl] == fSailHead[cSl])
    {
    fEffPlanHeadLeft[cSl] = fPlanHeadLeft[cSl] ;
    fEffPlanHeadRight[cSl] = fPlanHeadRight[cSl] ;
    }
  else
    {
    fTemp = (fEffSailHead[cSl] - fSailFoot[cSl]) * 2.0 ;
    fSlantLeft = (fPlanFootLeft[cSl] - fPlanHeadLeft[cSl]) / 
      ((fSailHead[cSl] - fSailFoot[cSl]) * 2.0) ;
    fSlantRight = (fPlanFootRight[cSl] - fPlanHeadRight[cSl]) / 
      ((fSailHead[cSl] - fSailFoot[cSl]) * 2.0) ;
    fEffPlanHeadLeft[cSl] = fPlanFootLeft[cSl] - fSlantLeft * fTemp ;
    fEffPlanHeadRight[cSl] = fPlanFootRight[cSl] - fSlantRight * fTemp ;
    }
  }

/* ------------------------------
     JibChange
     Version date: 00.07.16
-------------------------------*/
void JibChange (int iJibCode) 
  {
  char  cTemp[2] ;

  GetWindowText (hwndButton[5+iJibCode], cTemp, 2) ;
  switch (cTemp[0])
    {
    case 'L' :
      iJibSet = 0;
      break ;
    case 'W' :
      iJibSet = 1;
      break ;
    case 'S' :
      iJibSet = 2;
      break ;
    default :
      MessageBox (hwnd, "oeps", "oeps", MB_OK) ;
    }
  JibSelect () ;
  SailPlan () ;
  SetFocus (hwndCheck[2]) ;
  }

/*-----------------------------------------
   JibSelect: process jib selection
   Version date: 99.06.26
  -----------------------------------------*/
void JibSelect()
  {
  float  fTemp ;
  HDC    hdc ;
  
  fFootWidth[12] = fJibFootWidth[iJibSet] ;
  fSailDepth[12] = fJibDepth[iJibSet] ;
  fSailFoot[12] = fLowestSailFoot[3] ;
  fSailHead[12] = fSailFoot[12] + fJibDepth[iJibSet] ;
  fMaxSpeed[12] = fJibTheorMaxSpeed[iJibSet] * 
    (0.9 + 0.2 * (float) rand()/RAND_MAX) ;
  fTemp = (fMastDistance[3] - fMastDistance[0]) / (fSailHead[2] - fSailFoot[12]) ;
  fTemp = ((fSailHead[2] - fSailFoot[12]) - fJibDepth[iJibSet]) * fTemp ;
  fJibWidth = ((fMastDistance[3] - fMastDistance[0]) - fTemp) ;
  fPlanHeadLeft[12] = -fJibWidth ;
  fPlanHeadRight[12] = fPlanHeadLeft[12] ;
  fPlanFootRight[12] = 0. ;
  fPlanFootLeft[12] = -fFootWidth[12] ;
  fMeanSailHeight[12] = (fSailHead[12] + fSailFoot[12]) / 2 ;

  hdc = GetDC (hwnd) ;
  SelectObject (hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xJib+16,yMessage,xJib+72,yMessage+17) ;
  SelectObject (hdc,GetStockObject(BLACK_PEN)) ;
  SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
  TextOut (hdc, xJib+44, yMessage, sJibName[iJibSet],strlen(sJibName[iJibSet])) ; 
  memcpy (&sSailName[12],&sJibName[iJibSet],strlen(sJibName[iJibSet])) ;
  SetTextAlign (hdc,TA_LEFT|TA_TOP) ;
  ReleaseDC (hwnd, hdc) ;
  CheckJibStatus() ;
  }  

/*-----------------------------------------
   SailPlan: Updates sailplan    
   Version date: 00.07.16
-----------------------------------------*/
void SailPlan () 
  {
  HBRUSH hBrush1, hBrush2, hBrush3, hBrush4;
  int    iI ;
  HDC    hdc, hdcMem ;
  float  fFactor ;
  int    iSl ;
  float  fScew ;
  float  fStep ;
  float  fTemp ;
  POINT  pPoint ;
  int    xStart, yStart ;

  bSailPlanIsChanged = FALSE ;
  hdc = GetDC (hwndChild[1]) ;
  hdcMem = CreateCompatibleDC (hdc) ;
  if (!bParalax)
    fScew = (-fMastDistance[2]+fHeadWidth[11]+10) /
      (fMastDistance[3]-fMastDistance[2]+fHeadWidth[11]+20.) ;
  else
    fScew=0.5 ; // paralax view
  xOrgPixels = xChildPixels * fScew ;
  SetMapMode (hdc, MM_ANISOTROPIC) ;
  SetWindowExtEx (hdc, 1000 * fScew, 850, NULL) ;
  SetViewportExtEx (hdc, xOrgPixels, -0.85*yChildPixels, NULL) ;
  SetViewportOrgEx (hdc, xOrgPixels, 0.85*yChildPixels, NULL) ;
  SetMapMode (hdcMem, MM_ANISOTROPIC) ;
  SetWindowExtEx (hdcMem, 1000 * fScew, 850, NULL) ;
  SetViewportExtEx (hdcMem, xOrgPixels, -0.85*yChildPixels, NULL) ;
  SetViewportOrgEx (hdcMem, xOrgPixels, 0.85*yChildPixels, NULL) ;
  SelectObject (hdcMem, hBitmap) ;
  SelectObject(hdcMem, (HPEN) CreatePen(PS_SOLID,0,RGB(0,56,207))) ; //sea blue
  SelectObject(hdcMem, hSeaBrush) ;
  Rectangle (hdcMem, -1000, 0, 1000, -150) ;
  DeleteObject(SelectObject(hdcMem,(HPEN) CreatePen(PS_SOLID,0,RGB(0,215,255)))) ; //sky blue
  SelectObject(hdcMem, hSkyBrush) ;
  Rectangle (hdcMem, -1000, 0, 1000, 850) ;
  DeleteObject(SelectObject(hdcMem,GetStockObject(BLACK_PEN))) ; 

  if (!bParalax)
    {
    // conventional sailplan

    // draw hull
    SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;
    fTemp = fSailScale * fShipLength/2.0 ;
    BeginPath(hdcMem) ;
    MoveToEx (hdcMem, -fTemp, 0., NULL) ;
    LineTo (hdcMem, -fTemp, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fQDeckLength*fSailScale - fTemp, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fQDeckLength*fSailScale - fTemp, fGunDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp - fFocleLength*fSailScale, fGunDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp - fFocleLength*fSailScale, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp - fTemp*0.2, 0.) ;
    CloseFigure(hdcMem) ;
    EndPath(hdcMem) ;
    StrokeAndFillPath(hdcMem) ;

    // draw masts
    for (iI=0 ; iI<=2 ; iI++)
      {
      MoveToEx (hdcMem, fMastDistance[iI]*fSailScale, 0., NULL) ;
      LineTo (hdcMem, fMastDistance[iI]*fSailScale,fMastHeight[iI]*fSailScale) ;
      }

    // draw sails
    for (iI = 0 ; iI < 13 ; iI++)
      DrawSail((HDC) hdcMem, (char) iSequence[1][iI]) ;

    }
  else
    {
    // view from direction of wind
   
    // draw barricade
    fFactor = sin(fAppWindAngle * DEGTORAD) ;
    if ( fabs(fAppWindAngle) <90 )
      fTemp = ((fShipLength/2.0)-fFocleLength)*fSailScale * fFactor; // forecastle 
    else 
      {
      fTemp = -((fShipLength/2.0)-fQDeckLength)*fSailScale * fFactor; // quarterdeck
      }
    SelectObject(hdcMem, hDeckBrush) ;
    fStep=fShipWidth*0.5*fSailScale*cos(fAppWindAngle*DEGTORAD) ;
    BeginPath(hdcMem) ;
    MoveToEx (hdcMem, fTemp-fStep, fGunDeckHeight*fSailScale, NULL) ;
    LineTo (hdcMem, fTemp-fStep, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp+fStep, fGunDeckHeight*fSailScale) ;
    CloseFigure(hdcMem) ;
    EndPath(hdcMem) ;
    StrokeAndFillPath(hdcMem) ;

    // draw hull
    fTemp = fShipLength*fSailScale/2.0 * fFactor;
    SelectObject(hdcMem, GetStockObject (BLACK_BRUSH)) ;
    BeginPath(hdcMem) ;
    MoveToEx (hdcMem, -fTemp, 0., NULL) ;
    LineTo (hdcMem, -fTemp, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fQDeckLength*fSailScale*fFactor - fTemp, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fQDeckLength*fSailScale*fFactor - fTemp, fGunDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp - fFocleLength*fSailScale*fFactor, fGunDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp - fFocleLength*fSailScale*fFactor, fQDeckHeight*fSailScale) ;
    fStep=Sgn(fAppWindAngle)*fShipWidth*0.5*fSailScale*cos(fAppWindAngle*DEGTORAD) ;
    LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fSailScale) ;
    LineTo (hdcMem, fTemp+fStep, 0.) ;
    CloseFigure(hdcMem) ;
    EndPath(hdcMem) ;
    StrokeAndFillPath(hdcMem) ;

    // draw masts and sails
    hBrush1= CreateSolidBrush(RGB(192,192,192)) ; //jib
    hBrush2= CreateSolidBrush(RGB(128,128,0)) ; //foremast sails
    hBrush3= CreateSolidBrush(RGB(128,128,128)) ; //mainmast sails
    hBrush4= CreateSolidBrush(RGB(192,192,192)) ; //mizzenmast sails

    for (iI = 0 ; iI < 13 ; iI++)
      { 
      iSl=iSequence[iSeq][iI] ;
      switch (iSl) 
        {
        case 12 :
          SelectObject(hdcMem, hBrush1) ;
          break ;
        case 0 :  
        case 1 :  
        case 2 :  
        case 3 :  
          if (iSl==0)
            DrawMast(hdcMem,0,fFactor) ;
          SelectObject(hdcMem, hBrush2) ;
          break ;
        case 4 :  
        case 5 :  
        case 6 :  
        case 7 :  
          if (iSl==4)
            DrawMast(hdcMem,1,fFactor) ;
          SelectObject(hdcMem, hBrush3) ;
          break ;
        case 8 :  
        case 9 :  
        case 10 :  
        case 11 :  
          if (iSl==8)
            DrawMast(hdcMem,2,fFactor) ;
          SelectObject(hdcMem, hBrush4) ;
        }
      if (iSailStatus[iSl]<4)
        {
        BeginPath (hdcMem) ;
        MoveToEx (hdcMem, fWindHeadLeft[iSl]*fSailScale, fEffSailHead[iSl]*fSailScale, NULL) ;
        LineTo (hdcMem, fWindHeadRight[iSl]*fSailScale, fEffSailHead[iSl]*fSailScale) ;
        LineTo (hdcMem, fWindFootRight[iSl]*fSailScale, fSailFoot[iSl]*fSailScale) ;
        LineTo (hdcMem, fWindFootLeft[iSl]*fSailScale, fSailFoot[iSl]*fSailScale) ;
        CloseFigure (hdcMem) ;
        EndPath (hdcMem) ;
        StrokeAndFillPath (hdcMem) ;
        }
      // redraw mast if sails not taken aback
      if (iSl==3 && (fPerpFract[0]>0.0 || fPerpFract[1]>0.0 || fPerpFract[2]>0.0 || fPerpFract[3]>0.0))
        DrawMast(hdcMem,0,fFactor) ;
      if (iSl==7 && (fPerpFract[4]>0.0 || fPerpFract[5]>0.0 || fPerpFract[6]>0.0 || fPerpFract[7]>0.0))
        DrawMast(hdcMem,1,fFactor) ;
      if (iSl==11 && (fPerpFract[8]>0.0 || fPerpFract[9]>0.0 || fPerpFract[10]>0.0))
        DrawMast(hdcMem,2,fFactor) ;
      } 
    SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;
    DeleteObject(hBrush1);
    DeleteObject(hBrush2);
    DeleteObject(hBrush3);
    DeleteObject(hBrush4);

    // draw bow and stern
    if ( fabs(fAppWindAngle) <90 )
      {
      ShowBow (hdcMem, fTemp) ;
      ShowStern (hdcMem, fTemp) ;
      }
    else
      {
      ShowStern (hdcMem, fTemp) ;
      ShowBow (hdcMem, fTemp) ;
      }
    }

  pPoint.x = 0 ;
  pPoint.y = 0 ;
  DPtoLP (hdcMem, &pPoint, 1) ;
  xStart = pPoint.x ;
  pPoint.x = xChildPixels ;
  pPoint.y = yChildPixels ;
  DPtoLP (hdcMem, &pPoint, 1) ;
  yStart = pPoint.y ;

  BitBlt (hdc,xStart,yStart,1000,1000,
          hdcMem, xStart,yStart, SRCCOPY) ;
  ReleaseDC (hwndChild[1], hdc) ;
  DeleteDC (hdcMem) ;
  }

/*-----------------------------------------
   ShowBow: Shown bow on perspective view    
   Version date: 99.09.19
-----------------------------------------*/
void ShowBow (HDC hdcMem, float fTemp) 
  {
  float fStep ;

  SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;
  fStep=fShipWidth*0.5*fSailScale*cos(fAppWindAngle*DEGTORAD) ;
  BeginPath(hdcMem) ;
  MoveToEx (hdcMem, fTemp-fStep, 0, NULL) ;
  LineTo (hdcMem, fTemp-fStep, fQDeckHeight*fSailScale) ;
  LineTo (hdcMem, fTemp+fStep, fQDeckHeight*fSailScale) ;
  LineTo (hdcMem, fTemp+fStep, 0*fSailScale) ;
  CloseFigure(hdcMem) ;
  EndPath(hdcMem) ;
  StrokeAndFillPath(hdcMem) ;
  if ( fabs(fAppWindAngle) >90. )
    {
    SelectObject(hdcMem, GetStockObject(WHITE_PEN));
    MoveToEx (hdcMem, fTemp, fQDeckHeight*fSailScale, NULL) ;
    LineTo (hdcMem, fTemp, 0) ;
    }
  SelectObject(hdcMem, GetStockObject(BLACK_PEN));
  }

/*-----------------------------------------
   ShowStern: Shown stern on perspective view    
   Version date: 00.07.16
-----------------------------------------*/
void ShowStern (HDC hdcMem, float fTemp) 
  {
  int   iI ;
  POINT pDrawStern[4] ;
  float fStep ;

  SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)) ;
  fStep=fShipWidth*0.5*fSailScale*cos(fAppWindAngle*DEGTORAD) ;
  BeginPath(hdcMem) ;
  MoveToEx (hdcMem, -fTemp-fStep, 0, NULL) ;
  LineTo (hdcMem, -fTemp-fStep, fQDeckHeight*fSailScale) ;
  LineTo (hdcMem, -fTemp+fStep, fQDeckHeight*fSailScale) ;
  LineTo (hdcMem, -fTemp+fStep, 0*fSailScale) ;
  CloseFigure(hdcMem) ;
  EndPath(hdcMem) ;
  StrokeAndFillPath(hdcMem) ;

  if ( fabs(fAppWindAngle) <90. )
    {
    SelectObject(hdcMem, hDeckBrush) ;
    for (iI=0; iI<4 ; iI++)
      {
      pDrawStern[iI].x=(pStern[iI].x*cos(fAppWindAngle * DEGTORAD)*fSailScale) - fTemp ;
      pDrawStern[iI].y=pStern[iI].y*fSailScale ;
      }
    SelectObject(hdcMem, hDeckBrush) ;
    BeginPath (hdcMem) ;
    PolyBezier (hdcMem,pDrawStern,4) ;
    CloseFigure (hdcMem) ;
    EndPath (hdcMem) ;
    StrokeAndFillPath (hdcMem) ;
    SelectObject(hdcMem, (HPEN) CreatePen(PS_SOLID,0,RGB(0,56,207))) ; //sea blue
    SelectObject(hdcMem, hSeaBrush) ;
    Rectangle (hdcMem, -1000, 0, 1000, -150) ;  // redraw sea over underwater stern
    DeleteObject(SelectObject(hdcMem,GetStockObject(BLACK_PEN))) ;
    }
  }

