/*-----------------------------------------
   STERNVIEW.C -- Plan view procedures module
   Version date: 03.03.22
  -----------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "external.h"

// Prototypes:
void     SternView () ;

extern   int Sgn(float) ;

/*-----------------------------------------
   SternView: Update stern view    
   Version date: 03.03.22
-----------------------------------------*/

void SternView() 
  {
  HDC   hdc, hdcMem ;
  float fDistance ;
  POINT pDrawStern[4] ;
  int   iI ;
  float fRadians, fSternScale ;
  float fX, fX2, fX3, fY ;

  bSternViewIsChanged = FALSE ; //set to True by events requiring redraw
  hdc = GetDC (hwndChild[3]) ;
  hdcMem = CreateCompatibleDC (hdc) ;
  SetMapMode (hdc, MM_ISOTROPIC) ;
  SetWindowExtEx (hdc, 300, 1200, NULL) ;
  SetViewportExtEx (hdc, cxSternView, -cySternView, NULL) ;
  SetViewportOrgEx (hdc, 0.48*cxSternView, 0.86*cySternView, NULL) ;
  SetMapMode (hdcMem, MM_ISOTROPIC) ;
  SetWindowExtEx (hdcMem, 300, 1200, NULL) ;
  SetViewportExtEx (hdcMem, cxSternView, -cySternView, NULL) ;
  SetViewportOrgEx (hdcMem, 0.48*cxSternView, 0.86*cySternView, NULL) ;

  SelectObject (hdcMem, hBitmap_3) ;
  SelectObject (hdcMem, (HPEN) hSeaBluePen) ;
  SelectObject (hdcMem, hSeaBrush) ;
  Rectangle (hdcMem, -280, 0, 280, -150) ;
  SelectObject (hdcMem, (HPEN) hSkyBluePen) ;
  SelectObject (hdcMem, hSkyBrush) ;
  Rectangle (hdcMem, -280, 1100, 280, 0) ;
  fSternScale = 1000/(fHighest+10) ;
  for (iI=0 ; iI <=3 ; iI++)
    {
    fRadians = fHeelingAngle * fDegToRad ;
    fX=pStern[iI].y * 0.9 * fSternScale * sin(fRadians);
    fY=pStern[iI].y * 0.9 * fSternScale * cos(fRadians);
    fRadians = (fHeelingAngle+90) * fDegToRad ;
    pDrawStern[iI].y=fY+(pStern[iI].x * 0.9 * fSternScale * cos(fRadians)) ;
    pDrawStern[iI].x=fX+(pStern[iI].x * 0.9 * fSternScale * sin(fRadians)) ;
    }
  fRadians = fHeelingAngle * fDegToRad ;
  fDistance = 0.9 * fSternScale * (fHighest+2.0) ;
  fX = fDistance * sin (fRadians) ;
  fY = fDistance * cos (fRadians) ;

  SelectObject (hdcMem, (HPEN) hHullPen) ;
  SelectObject (hdcMem, hDeckBrush) ;
  MoveToEx (hdcMem,0,0,NULL) ;
  LineTo (hdcMem,fX,fY) ;
  BeginPath (hdcMem) ;
  SelectObject (hdcMem,GetStockObject(BLACK_PEN)) ; 
  PolyBezier (hdcMem,pDrawStern,4) ;
  CloseFigure (hdcMem) ;
  EndPath (hdcMem) ;
  StrokeAndFillPath (hdcMem) ;

  // draw centre of effort (if not all yards are square and wind<>0)
  if ((fabs(fHeelingAngle) < 50 ) && (fWindSpeed > 0))
    {
    if ((fCOEHeight < fHighest) && (fCOEHeight > fGunDeckHeight))
      {
      fDistance = 0.9 * fSternScale * fCOEHeight ;
      fY = cos(fRadians) * fDistance ;
      fX = sin(fRadians) * fDistance ;
      if (Sgn(fHeelingAngle) > 0)
        {
        fX = fX - 50 ;
        fX2 = fX - 150 ;
        fX3 = fX - 50 ;
        }
      else
        {
        fX = fX + 50 ;
        fX2 = fX + 150 ;
        fX3 = fX + 50 ;
        }
      SelectObject(hdcMem, (HPEN) hRedPen) ;
      MoveToEx (hdcMem, fX, fY, NULL) ;
      LineTo (hdcMem, fX2, fY) ;
      MoveToEx (hdcMem, fX, fY, NULL) ;
      LineTo (hdcMem, fX3, fY+50) ;
      MoveToEx (hdcMem, fX, fY, NULL) ;
      LineTo (hdcMem, fX3, fY-50) ;
      SelectObject (hdcMem,GetStockObject(BLACK_PEN)) ;
      }
    }

  BitBlt (hdc,-300,-1200, 600,2400,hdcMem, -300,-1200, SRCCOPY) ;
  ReleaseDC (hwndChild[3], hdcMem) ;
  DeleteDC (hdcMem) ;
  }

